package FourierPackage;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Arrays;


/**Zeichnen des Koordinatensystems
 * und Berechnen der Funktionen
 * @author Matthias Klingler
 */
public class CoordPlane extends Component {

    private static final Color ACHSENFARBE = Color.black;
    private static final Font BESCHRIFTUNG = new Font("Arial",Font.PLAIN,12);
    private int planeHeight, planeWidth, cxLeft, cxRight, cyTop, cyDown, type;
    private double XMAX=-1, XMIN=1, YMAX=-1, YMIN=1, XUNIT, YUNIT, period=1.0;
    private boolean dynamic=false,wActive=false;
    private int[][] xyValues = new int[2][4000];
    private int[][] values = new int[2][6000];
    // Variable zur richtigen Einpassung des Strings
    private	int stringShift, n=0;
    // Inhalt des zu schreibenden Strings
    private	String stringValue;
    // Zu zeichnender Font
    private FontMetrics fm;



    public CoordPlane() {

    }

    @Override
    public void paint(Graphics g) {
        drawHelpLines(g);
        drawCoordPlane(g);
        nameCoordPlane(g);
    }


    /**
     * Diese Methode zeichnet die KoordinatenAchsen
     * @param g Graphics komponente
     */
    private void drawCoordPlane(Graphics g) {
        g.setColor(ACHSENFARBE);

        // Zwei 3 Pixel dicke Linien für die Achsen.
        g.fillRect(cxLeft, calcPosY(0)-1, cxRight-cxLeft-8, 3);
        g.fillRect(calcPosX(0)-1, cyDown+8, 3, cyTop);

        // Achsenpfeile :
        int[][] Arrays = {{cxRight+1,cxRight-8,cxRight-8},{calcPosY(0),calcPosY(0)+4,calcPosY(0)-4}};
        g.fillPolygon(Arrays[0], Arrays[1], 3);
        int[][] Arrays2 = {{calcPosX(0),calcPosX(0)+4,calcPosX(0)-4},{cyDown-1,cyDown+8,cyDown+8}};
        g.fillPolygon(Arrays2[0], Arrays2[1], 3);

        // Achsenbeschriftung :
        if(!wActive)
            g.drawString("t",cxRight-8,calcPosY(0)-5);
        else
            g.drawString("ω",cxRight-9,calcPosY(0)-5);
        g.drawString("y",calcPosX(0)+7,cyDown+8);
    }


    /**
     * Diese Methode beschriftet die KoordinatenAchsen
     * @param g Graphics komponente
     */
    private void nameCoordPlane(Graphics g)	{
        // font und farbe fuer achsenbeschriftung
        g.setColor(ACHSENFARBE);
        g.setFont(BESCHRIFTUNG);
        fm = getFontMetrics(BESCHRIFTUNG);

        //Werte Y-Achse (parallel verschoben von X-Achse)
        // Normale Y-Achse :
        if(type!=2) {
            for (double temp = (int) (YMIN); temp <= (int) (YMAX); temp++) {
                if (temp !=0) {                                                 // keine 0.0 einzeichnen
                    stringValue = String.valueOf((int)temp);
                    // Kleine Markierung (Strich) zeichnen
                    g.drawLine(calcPosX(0)+3, calcPosY(temp), calcPosX(0)-3, calcPosY(temp));
                    // Beschriftung zeichnen
                    if(temp>=0) {
                        if(temp<10)
                            g.drawString(stringValue,calcPosX(0)-10,calcPosY(temp)+4);
                        else
                            g.drawString(stringValue,calcPosX(0)-17,calcPosY(temp)+4);
                    } else {
                        if(temp>-10)
                            g.drawString(stringValue,calcPosX(0)-14,calcPosY(temp)+4);
                        else
                            g.drawString(stringValue,calcPosX(0)-21,calcPosY(temp)+4);
                    }
                }
            }
        }
        // Y-Achse in π angegeben
        else {
            for (double temp = ((int) (YMIN/(Math.PI/2)))*(Math.PI/2); temp <= 0; temp=temp+Math.PI/2) {
                if (temp !=0) {                                                 // keine 0.0 einzeichnen
                    double value = temp/Math.PI;
                    if(value==(int)value) {
                        stringValue = String.valueOf((int)value)+"π";
                        // Beschriftung zeichnen
                        g.drawString(stringValue,calcPosX(0)-21,calcPosY(temp)+4);
                    } else {
                        value=Math.round(value*10);
                        stringValue = String.valueOf(value/10)+"π";
                        // Beschriftung zeichnen
                        g.drawString(stringValue,calcPosX(0)-31,calcPosY(temp)+4);
                    }
                    // Kleine Markierung (Strich) zeichnen
                    g.drawLine(calcPosX(0)+3, calcPosY(temp), calcPosX(0)-3, calcPosY(temp));
                }
            }
            for (double temp =0; temp <= ((int) (YMAX/(Math.PI/2)))*(Math.PI/2); temp=temp+Math.PI/2) {
                if (temp !=0) {                                                 // keine 0.0 einzeichnen
                    double value = temp/Math.PI;
                    if(value==(int)value) {
                        stringValue = String.valueOf((int)value)+"π";
                        // Beschriftung zeichnen
                        g.drawString(stringValue,calcPosX(0)-18,calcPosY(temp)+4);
                    } else {
                        value=Math.round(value*10);
                        stringValue = String.valueOf(value/10)+"π";
                        // Beschriftung zeichnen
                        g.drawString(stringValue,calcPosX(0)-28,calcPosY(temp)+4);
                    }
                    // Kleine Markierung (Strich) zeichnen
                    g.drawLine(calcPosX(0)+3, calcPosY(temp), calcPosX(0)-3, calcPosY(temp));
                }
            }
        }

        if(!wActive) {
            //Werte X-Achse (parallel verschoben von Y-Achse)
            // Normale Beschriftung
            for (double temp = (int) (XMIN+1); temp <= (int) (XMAX-1); temp++) {
                if(temp!=0) {
                    stringValue = String.valueOf((int)temp);
                    // Berechnen um wieviel der Text nach links verschoben
                    // werden muss, damit er in der Mitte liegt.
                    stringShift = (int) (fm.stringWidth(stringValue) / 2);
                    // Kleine Markierung (Strich) zeichnen
                    g.drawLine(calcPosX(temp), calcPosY(0)+3, calcPosX(temp), calcPosY(0)-3);
                    // Beschriftung zeichnen
                    g.drawString(stringValue,calcPosX(temp)-stringShift,calcPosY(0)+14);
                }
            }
        } else {
            //Werte X-Achse (parallel verschoben von Y-Achse)
            // Beschriftung mit ω
            int counter=1;
            for (double temp = (0); temp <= ((int) (XMAX+1)); temp=temp+(2*Math.PI/period)) {
                if(temp!=0) {
                    stringValue = String.valueOf(counter).concat("ω");
                    // Berechnen um wieviel der Text nach links verschoben
                    // werden muss, damit er in der Mitte liegt.
                    stringShift = (int) (fm.stringWidth(stringValue) / 2);
                    // Kleine Markierung (Strich) zeichnen
                    g.drawLine(calcPosX(temp), calcPosY(0)+3, calcPosX(temp), calcPosY(0)-3);
                    int shift=0;

                    if((n<10)||(counter%2!=0)) {
                        // Beschriftung zeichnen
                        g.drawString(stringValue,calcPosX(temp)-stringShift,calcPosY(0)+14);
                        if(temp>=(10*(2*Math.PI/period)))
                            shift=(int)temp-3-((int)temp-8);
                        else
                            shift=0;
                        g.setFont(new Font("Arial",Font.PLAIN,7));
                        g.drawString("0",calcPosX(temp)-stringShift+16+shift,calcPosY(0)+16);
                        g.setFont(BESCHRIFTUNG);
                    }
                    counter++;
                }
            }
        }
    }


    /**
     * Diese Methode zeichnet die Hilfslinien ein
     * @param g Graphics komponente
     */
    private void drawHelpLines(Graphics g) {
        g.setColor(Color.LIGHT_GRAY);
        if(!wActive) {
            for(double i = XMIN; i < XMAX ; i++) {
                g.drawLine(calcPosX(i), 0, calcPosX(i), calcPosY(YMIN));
            }
        } else {
            fm = getFontMetrics(BESCHRIFTUNG);
            for(double i = -10; i < XMAX ; i++) {
                g.drawLine(calcPosX(i), 0, calcPosX(i), calcPosY(YMIN));

                g.setFont(new Font("Tahoma",Font.PLAIN,8));
                stringValue = String.valueOf((int)i);
                // Berechnen um wieviel der Text nach links verschoben
                // werden muss, damit er in der Mitte liegt.
                stringShift = (int) (fm.stringWidth(stringValue) / 2);

                // Beschriftung zeichnen
                g.setColor(Color.DARK_GRAY);
                g.drawString(stringValue,calcPosX(i)-stringShift+6,calcPosY(YMIN)-9);
                g.setColor(Color.lightGray);
                g.setFont(BESCHRIFTUNG);
            }
        }
        if(type==2) {
            for(double i = 0; i < YMAX ; i=i+Math.PI*0.5) {
                g.drawLine(0, calcPosY(i), cxRight, calcPosY(i));
            }
            for(double i = 0; i > YMIN ; i=i-Math.PI*0.5) {
                g.drawLine(0, calcPosY(i), cxRight, calcPosY(i));
            }
        } else {
            for(double i = 0; i < YMAX ; i++) {
                g.drawLine(0, calcPosY(i), cxRight, calcPosY(i));
            }
            for(double i = 0; i > YMIN ; i--) {
                g.drawLine(0, calcPosY(i), cxRight, calcPosY(i));
            }
        }
    }

    /**
     * Diese Methode löscht die Zeichenfläche
     * @param g Graphics komponente
     */
    public void clear(Graphics g) {
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, planeWidth+5, planeHeight+5);
    }


    /**
     * Die Methode calcPosX rechnet den (x)-Wert eines
     * Funktionswertes auf die tatsächlich (x)-Position (inPixel)
     * im Frame um.
     * @param xValue x-Wert des Punktes
     */
    public int calcPosX(double xValue) {
        return (int)((xValue-XMIN)*XUNIT);
    }

    /**
     * Die Methode calcPosY rechnet den (y)-Wert eines
     * Funktionswertes auf die tatsächlich (y)-Position (inPixel)
     * im Frame um.
     * @param yValue y-Wert des Punktes
     */
    public int calcPosY(double yValue) {
        if(!dynamic)
            return (int)(planeHeight-((yValue-YMIN)*XUNIT));
        else
            return (int)(planeHeight-((yValue-YMIN)*YUNIT));
    }

    /**
     * Die Methode calcValX rechnet den Pixel-Wert eines
     * Punktes auf die tatsächliche x-Koordinate um
     * @param PixelX Anzahl der Pixel von Links
     */
    public double calcValX(int PixelX) {
        return (XMIN+(PixelX/XUNIT));
    }

    /**
     * Die Methode calcValY rechnet den Pixel-Wert eines
     * Punktes auf die tatsächliche y-Koordinate um
     * @param PixelY Anzahl der Pixel von Oben
     */
    public double calcValY(int PixelY) {
        return (YMAX-(PixelY/YUNIT));
    }

    /**
     * Die Methode getXUNIT gibt den Wert für eine Einheit zurück
     */
    public double getXUNIT() {
        return XUNIT;
    }

    /**
     * Die Methode getYUNIT gibt den Wert für eine Einheit zurück
     */
    public double getYUNIT() {
        return YUNIT;
    }

    /**
     * Diese Methode setzt den minimalen und den
     * maximalen x-Wert
     * @param xMin Minimalwert auf x-Achse
     * @param xMax Maximalwert auf x-Achse
     */
    public void setXMinMax(double xMin, double xMax) {
        XMAX = xMax;
        XMIN = xMin;
    }

    /**
     * Diese Methode setzt den minimalen und den
     * maximalen y-Wert
     * @param yMin Minimalwert auf y-Achse
     * @param yMax Maximalwert auf y-Achse
     */
    public void setYMinMax(double yMin, double yMax) {
        YMAX = yMax;
        YMIN = yMin;
    }

    /**
     * Diese Methode setzt den minimalen und den
     * maximalen x- und y-Wert
     * @param xMin Minimalwert auf x-Achse
     * @param xMax Maximalwert auf x-Achse
     * @param yMin Minimalwert auf y-Achse
     * @param yMax Maximalwert auf y-Achse
     */
    public void setMinMax(double xMin, double xMax, double yMin, double yMax) {
        XMAX = xMax;
        XMIN = xMin;
        YMAX = yMax;
        YMIN = yMin;
    }

    /**
     * Diese Methode setzt den minimalen x-Wert
     * @param xMin Minimalwert auf x-Achse
     */
    public void setMin(double xMin) {
        XMIN = xMin;
    }

    /**
     * Diese Methode setzt den maximalen x-Wert
     * @param xMax Maximalwert auf x-Achse
     */
    public void setMax(double xMax) {
        XMAX = xMax;
    }

    /**
     * Diese Methode gibt an ob die Grafik
     * dynamisch skaliert wird
     * @param dyn dynamisches Anzeigen
     */
    public void setDynamic(boolean dyn) {
        if(dyn) {
            dynamic=true;
        } else {
            dynamic=false;
        }
    }

    /**
     * Diese Methode gibt an ob die Labels
     * der x-Achse mit w beschriftet werden sollen
     * @param wLabel w-Beschriftung
     */
    public void setLabelW(boolean wLabel) {
        wActive=wLabel;
    }

    /**
     * Diese Methode setzt den Typ der zu
     * zeichnenden Funktion (Die unteren beiden
     * Anzeigen)
     * @param typ Typ der Funktion
     */
    public void setType(int typ) {
        type=typ;
    }

    /**
     * Diese Methode übergibt die Periodendauer
     * für die Skalierung der x-Achse
     * @param T Periodendauer
     */
    public void setPeriod(double T) {
        period = T;
    }

    /**
     * Diese Methode übergibt die Anzahl n
     * für die Skalierung der x-Achse
     * @param amount Anzahl n
     */
    public void setN(int amount) {
        n = amount;
    }

    /**
     * Diese Methode setzt die Höhe und Breite
     * des Rahmens in welchem die Funktion gezeichnet wird
     * @param Height Höhe des Rahmens
     * @param Width Breite des Rahmens
     */
    public void setCoordSeize(int Height, int Width) {
        planeWidth = Width;
        planeHeight = Height;

        XUNIT = planeWidth / (java.lang.Math.sqrt((XMAX-XMIN)*(XMAX-XMIN)));
        YUNIT = planeHeight / (java.lang.Math.sqrt((YMAX-YMIN)*(YMAX-YMIN)));
        if(!dynamic) {
            if(type!=0) {
                YMAX = (planeHeight/(XUNIT*2));
                YMIN = (YMAX-(planeHeight/XUNIT));
            } else {
                YMAX = planeHeight/(XUNIT*2);
                YMIN = YMAX-(planeHeight/XUNIT);
            }
        }
        cxLeft = calcPosX(XMIN);
        cxRight = calcPosX(XMAX);
        cyTop = calcPosY(YMIN);
        cyDown = calcPosY(YMAX);
    }




    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Rechteckfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] rectangle(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        //  f(t) =  4*amp/PI *  ( sin(wt)+ 1/3 * sin(3wt)+ 1/5 * sin(5wt) + ... );
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            if(a%2!=0)
                aVals[a-1] = (4*amplitude/Math.PI) * ( (1.0/(a))* Math.sin(a*w*x) );
            else
                aVals[a-1] = 0;
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            if(b%2!=0)
                bVals[b-1] = (4*amplitude/Math.PI) * ( (1.0/(b))* Math.cos(b*w*x) );
            else
                bVals[b-1] = 0;
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }
        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert

            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Dreiecksfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] triangle(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        //  f(t) =  8*amp/PI² * ( sin(wt) + 1/3² * sin(3wt) + 1/5² * sin(5wt)+ ... );
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            if(a%2!=0)
                aVals[a-1] = (8*amplitude/(Math.PI * Math.PI)) * ((-1)*Math.pow((-1), ((a+1)/2))) * ( (1.0/(a*a)) * Math.sin(a*w*x) );
            else
                aVals[a-1] = 0;
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            if(b%2!=0)
                bVals[b-1] = (8*amplitude/(Math.PI * Math.PI)) * ((-1)*Math.pow((-1), ((b+1)/2))) * ( (1.0/(b*b)) * Math.cos(b*w*x) );
            else
                bVals[b-1] = 0;
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }

        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert

            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Sägezahnfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] sawtooth(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        //  f(t) =  (-1*) 2*amp/PI * ( sin(wt) + 1/2 * sin(2wt) + 1/3 * sin(3wt)+ ... );
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            aVals[a-1] = (2*amplitude/Math.PI)  * ( (1.0/a)  * Math.sin((a)*w*x) );
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            bVals[b-1] = (2*amplitude/Math.PI) * ( (1.0/b) * Math.cos((b)*w*x) );
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }

        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY((a0*0.5 + tempValue));               // y-Wert
            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Sinusfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] sinus(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        // f(t) = 2*amp/PI - 4*amp/PI * [ 1/(1*3) * cos(2wt) + 1/(3*5) * cos(4wt) + 1/(5*7) * cos(6wt) +... ]
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            if(a%2==0)
                aVals[a-1] = (-1) * (4*amplitude/Math.PI) * ( (1.0/((a-1)*(a+1))) * Math.cos(a*w*x) );
            else
                aVals[a-1] = 0;
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            if(b%2==0)
                bVals[b-1] = (1) * (4*amplitude/Math.PI) * ( (1.0/((b-1)*(b+1))) * Math.sin(b*w*x) );
            else
                bVals[b-1] = 0;
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }

        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert
            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Parabelfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] parabel(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        // f(t) = amp/3 + 4*amp/ PI² * ( 1/1² cos(wt) + 1/2² cos(2wt) + 1/3² cos(3wt) ... )
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            aVals[a-1] = (4*amplitude/(Math.PI*Math.PI)) * ( (1.0/(a*a)) * Math.cos((a)*w*x) );
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            bVals[b-1] = (4*amplitude/(Math.PI*Math.PI)) * ( (1.0/(b*b)) * Math.sin((b)*w*x) );
        }


        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }

        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert
            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Rechteckimpulsfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @param abstand Abstand a zwischen den Impulsen
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] rectangleImp(double amplitude, int valCount, double x, double y, double T, double abstand) {
        //  Eigentliche Funktion :
        //  f(t) =  4*amp/PI *  ( sin(wt)+ 1/3 * sin(3wt)+ 1/5 * sin(5wt) + ... );
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // ((4*amplitude/Math.PI * ( Math.cos((z*2-1)*w*a) / (z*2-1)) *  Math.sin((z*2-1)*w*i);

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            if(a%2!=0)
                aVals[a-1] = (4*amplitude/Math.PI) * ( (( Math.cos(a*w*abstand)/a)) * Math.sin(a*w*x) );
            else
                aVals[a-1] = 0;
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            if(b%2!=0)
                bVals[b-1] = (4*amplitude/Math.PI) * ( (( Math.cos(b*w*abstand)/b)) * Math.cos(b*w*x) );
            else
                bVals[b-1] = 0;
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }

        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b],tempAmp=amps[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;
            tempAmp=tempAmp*1000;
            tempAmp=Math.round(tempAmp)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((tempAmp+tempA)==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert
            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Fourier-Sinusimpulsfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param x Offsetverschiebung auf x-Achse (Phasenverschiebung)
     * @param y Offsetverschiebung auf y-Achse (änderung von a0)
     * @param T Periodendauer
     * @return Object[] Array mit { XY-Werte[][], Amplituden[], Phasen[], Sinus[], Cosinus[] }
     */
    public Object[] sinusImp(double amplitude, int valCount, double x, double y, double T) {
        //  Eigentliche Funktion :
        //  f(t) =  amp/PI + amp/2 *sin(wt) - 2*amp/PI * [ 1/(1*3)*cos(2wt) + 1/(3*5)*cos(4wt) + 1/(5*7)*cos(6wt) +...]
        double tempValue = 0;
        double w = 2* Math.PI / T;

        // Berechnung von a0
        double a0 = 2*y;

        // Berechnung von a
        double aVals[] = new double[valCount];
        for(int a=1 ; a <= aVals.length ; a++) {
            if(a==1)
                aVals[a-1] = amplitude*0.5 * Math.sin(a*w*x);
            else {
                if(a%2==0)
                    aVals[a-1] = (-1) * (2*amplitude/Math.PI) * ( (1.0/((a-1)*(a+1))) * Math.cos(a*w*x) );
                else
                    aVals[a-1] = 0;
            }
        }

        // Berechnung von b
        double bVals[] = new double[valCount];
        for(int b=1 ; b <= bVals.length ; b++) {
            if(b==1)
                bVals[b-1] = amplitude*0.5 * Math.cos(b*w*x);
            else {
                if(b%2==0)
                    bVals[b-1] = (1) * (2*amplitude/Math.PI) * ( (1.0/((b-1)*(b+1))) * Math.sin(b*w*x) );
                else
                    bVals[b-1] = 0;
            }
        }

        // Berechnung der Beträge   { ampn = wurzel( an² + bn²) }
        double amps[] = new double[valCount];
        for(int b=0 ; b < amps.length ; b++) {
            amps[b] = Math.sqrt((aVals[b])*(aVals[b])+(bVals[b])*(bVals[b]));
        }
        // Berechnung der Phasen    { phase = 2* Arctan( bn/(Ampn+an) ) }
        double phases[] = new double[valCount];
        for(int b=0 ; b < phases.length ; b++) {
            double tempA=aVals[b],tempB=bVals[b];
            tempA=tempA*1000;
            tempA=Math.round(tempA)/1000.0;
            tempB=tempB*1000;
            tempB=Math.round(tempB)/1000.0;

            if((tempA<0)&&(tempB==0)) {
                phases[b] = Math.PI;
            } else if((amps[b]+aVals[b])==0) {
                phases[b]=0;
            } else {
                phases[b] = 2* Math.atan( bVals[b]/(amps[b]+aVals[b]));
            }
        }

        int counter = 0;
        for(double i=(-xyValues[0].length/200) ; i< (xyValues[0].length/200) ; i=i+0.01) {
            for(int z=1 ; z<=valCount ; z++) {
                tempValue = tempValue + aVals[z-1] * Math.cos(z*w*i) +  bVals[z-1]*Math.sin(z*w*i);
            }
            xyValues[0][counter]= calcPosX(i);                                  // x-Wert
            xyValues[1][counter]= calcPosY(a0*0.5 + tempValue);                 // y-Wert
            tempValue = 0;
            counter++;
        }

        Object[] valuesObj= {xyValues,(int[][])arrayToKoordinates(amps,1,a0),(int[][])arrayToKoordinates(phases,2,a0),
                             (int[][])arrayToKoordinates(aVals,3,a0),(int[][])arrayToKoordinates(bVals,4)
                            };
        return valuesObj;
    }


    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Rechteckfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdRectangle(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        int counter = 0;
        double wx=0;
        double t=1.0*T;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {
            i = Math.round(i*100);
            i=i/100;
            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert
            wx =i%T;
            if(wx>=0) {
                if(wx<=t*0.5)
                    values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
                else
                    values[1][counter]= calcPosY(((-1)*amplitude)+Yoffset);         // y-Wert
            } else {
                if(wx<=(-1)*t*0.5)
                    values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
                else
                    values[1][counter]= calcPosY(((-1)*amplitude)+Yoffset);         // y-Wert
            }
            counter++;
        }
        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Dreiecksfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdTriangle(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        int counter = 0;
        double m=0;
        double b=amplitude;
        double wx=0;
        double t=1.0*T;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {
            i = Math.round(i*100);
            i=i/100;
            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert
            wx =i%T;

            if(wx>=0) {
                if ( wx<(t*0.25)) {
                    m=amplitude/(t*0.25);
                    b=0;
                } else if (( wx>=(t*0.25))&&(wx<(t*0.75))) {
                    m=(-1)*amplitude/(t*0.25);
                    b=2*amplitude;
                } else {
                    m=amplitude/(t*0.25);
                    b=(-4)*amplitude;
                }
            } else {
                if ( (-1)*wx<(t*0.25)) {
                    m=amplitude/(t*0.25);
                    b=0;
                } else if (( (-1)*wx>=(t*0.25))&&((-1)*wx<(t*0.75))) {
                    m=(-1)*amplitude/(t*0.25);
                    b=(-2)*amplitude;
                } else {
                    m=amplitude/(t*0.25);
                    b=(4)*amplitude;
                }
            }
            values[1][counter]= calcPosY((m*wx+b)+Yoffset);                     // y-Wert
            counter++;
        }

        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Sägezahnfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdSawtooth(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        int counter = 0;
        double m=0;
        double b=0;
        double wx=0;
        double t=1.0*T;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {
            i = Math.round(i*100);
            i=i/100;
            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert
            wx =i%T;

            if(wx>=0) {
                m=(-2)*amplitude/t;
                b=amplitude;

            } else {
                m=(-2)*amplitude/t;
                b=(-1)*amplitude;
            }
            values[1][counter]= calcPosY((m*wx+b)+Yoffset);                     // y-Wert
            counter++;
        }
        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Sinusfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdSinus(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        //  Eigentliche Funktion :
        // f(t) = sin(x);
        int counter = 0;
        double w = 2* Math.PI / T;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {
            values[0][counter]= calcPosX(i+Xoffset);                                                            // x-Wert
            values[1][counter]= calcPosY(Math.abs(amplitude*Math.sin(w*i))+Yoffset-2*(amplitude)/Math.PI);        // y-Wert
            counter++;
        }
        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Parabelfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdParabel(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        //  Eigentliche Funktion :
        // f(t) = 2*amp/PI - 4*amp/PI * [ 1/(1*3) * cos(2wt) + 1/(3*5) * cos(4wt) + 1/(5*7) * cos(6wt) +... ]
        int counter = 0;
        double w = 2* Math.PI / T;
        double wx=0;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {
            i = Math.round(i*100);
            i=i/100;
            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert
            wx =Math.abs(i%T);

            double t=1.0*T;

            values[1][counter]= calcPosY((( 4*amplitude / (t*t) ) *((wx-t*0.5)*(wx-t*0.5)))+Yoffset-(amplitude)/3.0);      // y-Wert

            counter++;
        }
        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Rechteckimpulsfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @param a Abstand zwischen den Impulsen
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdRectangleImp(double amplitude, int valCount, double Xoffset, double Yoffset, double T,double a) {
        //  Eigentliche Funktion :
        // y = 4*amp/PI * [ ( cos(wa) )/1 * sin(wt) + ( cos(3wa) )/3 * sin(3wt) + ( cos(5wa) )/5 * sin(5wt) + ...];
        int counter = 0;
        amplitude = amplitude*(-1);
        counter = 3000;
        double wx=0;

        for(double i=0 ; i< (values[0].length/200) ; i=i+0.01) {

            i = Math.round(i*100);
            i=i/100;
            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert
            wx =i%T;

            double t=1.0*T;

            if(( a < wx ) && ( wx < ((t*0.5) - a))) {
                amplitude = Math.abs(amplitude);
                values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
            } else if((((t*0.5) + a) < wx ) && ( wx < (t - a))) {
                amplitude = (-1)*Math.abs(amplitude);
                values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
            } else {
                values[1][counter]= calcPosY((0)+Yoffset);                      // y-Wert
            }
            counter++;
        }

        wx=0;
        counter=2999;
        for(double i=-0.01 ; i>= -(values[0].length/200) ; i=i-0.01) {
            i = Math.round(i*100);
            i=i/100;

            values[0][counter]= calcPosX(i+Xoffset);                            // x-Wert

            wx =Math.abs(i%T);
            double t=1.0*T;

            if(( a < wx ) && ( wx < ((t*0.5) - a))) {
                amplitude = (-1)*Math.abs(amplitude);
                values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
            } else if((((t*0.5) + a) < wx ) && ( wx < (t - a))) {
                amplitude = Math.abs(amplitude);
                values[1][counter]= calcPosY((amplitude)+Yoffset);              // y-Wert
            } else {
                values[1][counter]= calcPosY((0)+Yoffset);                      // y-Wert
            }
            counter--;
        }
        return values;
    }

    /**
     * Diese Methode berechnet die Daten für die
     * Standart-Sinusimpulsfunktion
     * @param amplitude Amplitude der Funktion
     * @param valCount Anzahl der Harmonischen ( n )
     * @param Xoffset Offsetverschiebung auf x-Achse
     * @param Yoffset Offsetverschiebung auf y-Achse
     * @param T Periodendauer
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    public int[][] stdSinusImp(double amplitude, int valCount, double Xoffset, double Yoffset, double T) {
        //  Eigentliche Funktion :
        // f(t) = sin(x);
        int counter = 0;
        double w = 2* Math.PI / T;
        double wx=0;

        for(double i=(-values[0].length/200) ; i< (values[0].length/200) ; i=i+0.01) {

            i = Math.round(i*100);
            i=i/100;

            values[0][counter]= calcPosX(i+Xoffset);                                    // x-Wert

            wx =Math.abs(i%T);
            double t=1.0*T;

            if(( 0 <= wx ) && ( wx <= t*0.5)) {
                if( i >= 0) {
                    values[1][counter]= calcPosY(amplitude*Math.sin(w*i)+Yoffset-(amplitude/(Math.PI)));      // y-Wert
                } else {
                    values[1][counter]= calcPosY(0+Yoffset-(amplitude/(Math.PI)));                            // y-Wert
                }
            } else if ( (t*0.5 <= wx) && (wx <= t) ) {
                if( i >= 0) {
                    values[1][counter]= calcPosY(0+Yoffset-(amplitude/(Math.PI)));                            // y-Wert
                } else {
                    values[1][counter]= calcPosY(amplitude*Math.sin(w*i)+Yoffset-(amplitude/(Math.PI)));      // y-Wert
                }
            }
            counter++;
        }
        return values;
    }


    /**
     * Diese Methode rechnet die Pixel eines Arrays
     * in Koordinaten zur Balkenanzeige um
     * @param vals Array mit Werten
     * @param typ Typ der Funktion (Amp=1,Pha=2,Sin=3,Cos=4)
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    private int[][] arrayToKoordinates(double[] vals,int typ) {

        if((dynamic)&&(vals.length!=0)) {
            double[] valscopy = vals.clone();
            if((type==typ)&&(valscopy.length!=0)) {
                Arrays.sort(valscopy);

                YMIN=(int)valscopy[0]-1;
                YMAX=(int)(valscopy[(valscopy.length)-1])+1;
                XUNIT = planeWidth / (java.lang.Math.sqrt((XMAX-XMIN)*(XMAX-XMIN)));
                YUNIT = planeHeight / (java.lang.Math.sqrt((YMAX-YMIN)*(YMAX-YMIN)));
                cyTop = calcPosY(YMIN);
                cyDown = calcPosY(YMAX);
            }
        }

        int counter=0;
        int[][] tempVals = new int[4][vals.length];
        for(int i=1 ; i<= vals.length ; i=i+1) {
            if(calcPosY(0)-calcPosY( vals[i-1])>=0) {
                tempVals[0][counter]= calcPosX(i*2*Math.PI/period);             // x1
                tempVals[1][counter]= calcPosY(vals[i-1]);                      // y1
                tempVals[2][counter]= calcPosX(i*2*Math.PI/period);             // x2
                tempVals[3][counter]= calcPosY(0);                              // y2
            } else {
                tempVals[0][counter]= calcPosX(i*2*Math.PI/period);             // x1
                tempVals[1][counter]= calcPosY(0);                              // y1
                tempVals[2][counter]= calcPosX(i*2*Math.PI/period);             // x2
                tempVals[3][counter]= calcPosY(vals[i-1]);                      // y2
            }
            counter++;
        }

        return tempVals;
    }

    /**
     * Diese Methode rechnet die Pixel eines Arrays
     * in Koordinaten zur Balkenanzeige um
     * @param vals Array mit Werten
     * @param typ Typ der Funktion (Amp=1,Pha=2,Sin=3,Cos=4)
     * @param a0 Nullte harmonische Schwingung
     * @return int[][] Array Koordinaten zum Zeichnen der Funktion
     */
    private int[][] arrayToKoordinates(double[] vals,int typ,double a0) {

        if(type==1)
            a0=Math.abs(a0);
        else if(type==2) {
            if(a0<0)
                a0=Math.PI;
            else
                a0=0;
        }

        if((dynamic)&&(vals.length!=0)) {
            double[] valscopy = vals.clone();
            if((type==typ)&&(valscopy.length!=0)) {
                Arrays.sort(valscopy);
                if((type!=3)&&(type!=2)) {
                    if(valscopy[0]<=Math.abs(a0))
                        YMIN=(int)valscopy[0]-1;
                    else
                        YMIN=(int)a0-1;
                    if(valscopy[(valscopy.length)-1]>=Math.abs(a0))
                        YMAX=(int)(valscopy[(valscopy.length)-1])+1;
                    else
                        YMAX=(int)Math.abs(a0)+1;
                } else if(type==2) {
                    YMIN = (int)-Math.PI-1;
                    YMAX = (int) Math.PI+1;
                } else {
                    if(valscopy[0]<=a0)
                        YMIN=(int)valscopy[0]-1;
                    else
                        YMIN=(int)a0-1;
                    if(valscopy[(valscopy.length)-1]>=a0)
                        YMAX=(int)(valscopy[(valscopy.length)-1])+1;
                    else
                        YMAX=(int)a0+1;

                }
                XUNIT = planeWidth / (java.lang.Math.sqrt((XMAX-XMIN)*(XMAX-XMIN)));
                YUNIT = planeHeight / (java.lang.Math.sqrt((YMAX-YMIN)*(YMAX-YMIN)));
                cyTop = calcPosY(YMIN);
                cyDown = calcPosY(YMAX);
            }
        }

        int counter=1;
        int[][] tempVals = new int[4][vals.length+1];
        if(calcPosY(0)-calcPosY( a0 )>=0) {
            tempVals[0][0]= calcPosX(0);                                    // x1
            tempVals[1][0]= calcPosY(a0);                                   // y1
            tempVals[2][0]= calcPosX(0);                                    // x2
            tempVals[3][0]= calcPosY(0);                                    // y2
        } else {
            tempVals[0][0]= calcPosX(0);                                    // x1
            tempVals[1][0]= calcPosY(0);                                    // y1
            tempVals[2][0]= calcPosX(0);                                    // x2
            tempVals[3][0]= calcPosY(a0);                                   // y2
        }

        for(int i=1 ; i<= vals.length ; i=i+1) {
            if(calcPosY(0)-calcPosY( vals[i-1])>=0) {
                tempVals[0][counter]= calcPosX(i*2*Math.PI/period);             // x1
                tempVals[1][counter]= calcPosY(vals[i-1]);                      // y1
                tempVals[2][counter]= calcPosX(i*2*Math.PI/period);             // x2
                tempVals[3][counter]= calcPosY(0);                              // y2

            } else {
                tempVals[0][counter]= calcPosX(i*2*Math.PI/period);             // x1
                tempVals[1][counter]= calcPosY(0);                              // y1
                tempVals[2][counter]= calcPosX(i*2*Math.PI/period);             // x2
                tempVals[3][counter]= calcPosY(vals[i-1]);                      // y2
            }
            counter++;
        }

        return tempVals;
    }

}