package FourierPackage;


import java.awt.*;
import javax.swing.JPanel;


/**Instanzen der Zeichenanzeige
 * @author Matthias Klingler
 */
public class RechteckMain extends JPanel {
    CoordPlane CoordSystem;
    int PanelW,PanelH;
    private int Amount=0;
    private double XMin=-10, XMax=10, YMin=-5, YMax=5;
    private int function=0,type=0;
    private double a=0,Amplitude=0.0, Xoffset=0.0, Yoffset=0.0, Period=1.0;
    private boolean basicActive=false;
    Graphics2D g2D;
    private Stroke strokeThin = new BasicStroke( 1 );
    private Stroke strokeThick = new BasicStroke( 3 );

    /** Creates new form Coordinates */
    public RechteckMain() {
        initComponents();
        CoordSystem = new CoordPlane();
    }

    /**
     * Diese Methode übergibt die Höhe und Breite
     * des Rahmens in welchem die Funktion gezeichnet wird
     * @param height Höhe des Rahmens
     * @param width Breite des Rahmens
     */
    public void setPanelSize(int height, int width) {
        PanelW=width;
        PanelH=height;
        CoordSystem.setCoordSeize(height, width);
    }

    /**
     * Diese Methode übergibt den minimalen und den
     * maximalen x-Wert
     * @param min Minimalwert auf x-Achse
     * @param max Maximalwert auf x-Achse
     */
    public void setXminmax(double min,double max) {
        XMin = min;
        XMax = max;
        CoordSystem.setXMinMax(XMin, XMax);
    }

    /**
     * Diese Methode übergibt den minimalen x-Wert
     * @param min Minimalwert auf x-Achse
     */
    public void setXmin(double min) {
        XMin = min;
        CoordSystem.setMin(XMin);
    }

    /**
     * Diese Methode übergibt den maximalen x-Wert
     * @param max Maximalwert auf x-Achse
     */
    public void setXmax(double max) {
        XMax = max;
        CoordSystem.setMax(XMax);
    }

    /**
     * Diese Methode übergibt den minimalen und den
     * maximalen x- und y-Wert
     * @param xmin Minimalwert auf x-Achse
     * @param xmax Maximalwert auf x-Achse
     * @param ymin Minimalwert auf y-Achse
     * @param ymax Maximalwert auf y-Achse
     */
    public void setXYminmax(double xmin,double xmax, double ymin,double ymax) {
        XMin = xmin;
        XMax = xmax;
        YMin = ymin;
        YMax = ymax;
        CoordSystem.setMinMax(XMin, XMax, YMin, YMax);
    }

    /**
     * Diese Methode übergibt die Verschiebung
     * auf der x-Achse
     * @param val Offset auf der x-Achse
     */
    public void setXoffset(int val) {
        Xoffset = ((double)val/10.0);
    }

    /**
     * Diese Methode übergibt die Verschiebung
     * auf der y-Achse
     * @param val Offset auf der y-Achse
     */
    public void setYoffset(int val) {
        Yoffset = ((double)val/10.0);
    }

    /**
     * Diese Methode gibt an ob die Grafik
     * dynamisch skaliert wird
     * @param dyn dynamisches Anzeigen
     */
    public void setDynamic(boolean dyn) {
        CoordSystem.setDynamic(dyn);
    }

    /**
     * Diese Methode übergibt die Anzahl n
     * der harmonischen Schwingungen
     * @param a Anzahl der Harmonischen
     */
    public void setAmount(int a) {
        Amount = a;
        CoordSystem.setN(a);
    }

    /**
     * Diese Methode übergibt die Amplitude
     * der Funktion
     * @param a Amplitude der Funktion
     */
    public void setAmplitude(double a) {
        Amplitude = a/10.0;
    }

    /**
     * Diese Methode übergibt die Periodendauer T
     * der Funktion
     * @param val Periodendauer
     */
    public void setT(int val) {
        Period = (double)val/10.0;
        CoordSystem.setPeriod((double)val/10.0);
    }

    /**
     * Diese Methode übergibt den Abstand
     * zwischen den Impulsen bei der Rechteck-
     * Impulsfunktion
     * @param val Abstand zwischen den Impulsen
     */
    public void setA(int val) {
        a = Period/4.0*(val/10.0);
    }

    /**
     * Diese Methode setzt die zu zeichnende Funktion
     * 0=Rechteck
     * 1=Dreieck
     * 2=Sägezahn
     * 3=Sinus
     * 4=Parabel
     * 5=Rechteckimpuls
     * 6=Sinusimpuls
     * @param val zu zeichnende Funktion
     */
    public void setFunction(int val) {
        function = val;
    }

    /**
     * Diese Methode gibt die zu zeichnende Funktion zurück
     * 0=Rechteck
     * 1=Dreieck
     * 2=Sägezahn
     * 3=Sinus
     * 4=Parabel
     * 5=Rechteckimpuls
     * 6=Sinusimpuls
     * @return int Wert der zu zeichnenden Funktion
     */
    public int getFunction() {
        return function;
    }

    /**
     * Diese Methode setzt den zu zeichnenden Typ
     * 0=XY-Werte (Fourierfunktion)
     * 1=Amplitude (Balkendiagramm)
     * 2=Phase (Balkendiagramm)
     * 3=Sinus (Balkendiagramm)
     * 4=Cosinus (Balkendiagramm)
     * @param typ zu zeichnender Typ
     */
    public void setType(int typ) {
        type = typ;
        CoordSystem.setType(typ);
    }

    /**
     * Diese Methode gibt den zu zeichnenden Typ zurück
     * 0=XY-Werte (Fourierfunktion)
     * 1=Amplitude (Balkendiagramm)
     * 2=Phase (Balkendiagramm)
     * 3=Sinus (Balkendiagramm)
     * 4=Cosinus (Balkendiagramm)
     * @return typ zu zeichnender Typ
     */
    public int getType() {
        return type;
    }

    /**
     * Diese Methode gibt an ob die Originalfunktion
     * angezeigt werden soll
     * @param active Anzeigen der Originalfunktion
     */
    public void setBasic(boolean active) {
        basicActive=active;
    }



    /**
     * Diese Methode gibt an ob w-Label auf der
     * x-Achse angezeigt werden sollen
     * @param wLabel Anzeigen der w-Label
     */
    public void setLabelW(boolean wLabel) {
        CoordSystem.setLabelW(wLabel);
    }

    /**
     * Diese Methode übergibt die Pixel-Koordinaten
     * zur Berechnung der XY-Koordinaten des aktuellen
     * Punktes.
     * @param pixelX Pixelkoordinate auf x-Achse
     * @param pixelY Pixelkoordinate auf y-Achse
     * @return String mit XY-Koordinaten
     */
    public String getCoordinate(int pixelX,int pixelY) {
        String st=null;
        double value=0;
        value = CoordSystem.calcValX(pixelX);
        value = Math.round(value*10);
        st=(Double.toString(value/10).trim());
        st=st.concat(" / ");
        value = CoordSystem.calcValY(pixelY);
        value = Math.round(value*10);
        st=st.concat(Double.toString(value/10)).trim();
        return st;
    }

    /**
     * Diese Methode zeichnet sämtliche Funktionen
     * @param g Graphics
     */
    @Override
    public void paintComponent(Graphics g) {
        g2D=(Graphics2D) g;

        // <editor-fold defaultstate="collapsed" desc="Rectangle">
        if(function==0) {
            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.rectangle(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];

                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdRectangle(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }

                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Triangle">
        else if(function==1) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.triangle(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdTriangle(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }

                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Sawtooth">
        else if(function==2) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.sawtooth(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdSawtooth(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }

                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Sinus">
        else if(function==3) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.sinus(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdSinus(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }

                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Parabel">
        else if(function==4) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.parabel(Amplitude,Amount,Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdParabel(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }

                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Rectangle Impuls">
        else if(function==5) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.rectangleImp(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period, a);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdRectangleImp(Amplitude,Amount,Xoffset,Yoffset, Period,a);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);
                }
                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

        // <editor-fold defaultstate="collapsed" desc="Sinus Impuls">
        else if(function==6) {

            /*Object : {xyValues,amps,phases,aVals,bVals}*/
            Object[] values = CoordSystem.sinusImp(Amplitude,Amount,(-1)*Xoffset,Yoffset, Period);
            CoordSystem.clear(g);
            CoordSystem.paint(g);
            g.setColor(Color.RED);

            /*Draw Function*/
            if(type==0) {
                int[][] xyValues = (int[][]) values[0];
                if(basicActive) {
                    int[][] valuesStd = CoordSystem.stdSinusImp(Amplitude,Amount,Xoffset,Yoffset, Period);
                    g.setColor(Color.BLUE);
                    g.drawPolyline(valuesStd[0],valuesStd[1], valuesStd[1].length);

                }
                g.setColor(Color.RED);
                g.drawPolyline(xyValues[0],xyValues[1], xyValues[1].length);
            }

            /*Draw Amplitudes*/
            else if(type==1) {
                int[][] amps = (int[][]) values[1];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<amps[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(amps[0][i], amps[1][i], amps[2][i], amps[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Phases*/
            else if(type==2) {
                if(Amplitude!=0) {
                    int[][] phases = (int[][]) values[2];
                    CoordSystem.paint(g);
                    g2D.setStroke(strokeThick);
                    for(int i=0; i<phases[0].length; i++) {
                        g.setColor(Color.RED);
                        g.drawLine(phases[0][i], phases[1][i], phases[2][i], phases[3][i]);
                    }
                    g2D.setStroke(strokeThin);
                }
            }

            /*Draw Sinus*/
            else if(type==3) {
                int[][] aVals = (int[][]) values[3];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<aVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(aVals[0][i], aVals[1][i], aVals[2][i], aVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }

            /*Draw Cosinus*/
            else if(type==4) {
                int[][] bVals = (int[][]) values[4];
                CoordSystem.paint(g);
                g2D.setStroke(strokeThick);
                for(int i=0; i<bVals[0].length; i++) {
                    g.setColor(Color.RED);
                    g.drawLine(bVals[0][i], bVals[1][i], bVals[2][i], bVals[3][i]);
                }
                g2D.setStroke(strokeThin);
            }
        }
        // </editor-fold>

    }




    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setPreferredSize(new java.awt.Dimension(400, 200));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 200, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
