
package source;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Die Klasse ParameterEingabe verwaltet die
 * Ein- und Ausgabewerte der Cosinusfunktion
 *
 * @author Horst Lauinger  31.10.2007
 */

public class ParameterEingabeCosinus {
    // Legt die Genauigkeit fest
    private double Genauigkeit = 100;

    // Formatierung der Anzeigen
    DecimalFormat df2 = new DecimalFormat( "0.00" );
    DecimalFormat df4 = new DecimalFormat( "0.0000" );

    // Referenzen auf die Zeichenbereiche
    private ZeichenbereichCosinus Zeichenbereich1 = null;
    private ZeichenbereichCosinus Zeichenbereich2 = null;

    // Bereich der in ParameterEingabe verwaltet wird
    private JPanel innerContent = null;

    // Gruppe von Elementen erstellen:
    // Beschriftung, Eingabefeld & Scrollbalken fr Amplitude
    // Beschriftung, Eingabefeld & Scrollbalken fr Kreisfrequenz
    // Beschriftung, Eingabefeld & Scrollbalken fr Phasenwinkel
    // Beschriftung, Eingabefeld & Scrollbalken fr Abgriff

    // Labels
    private JLabel LB_Amplitude_Wert = null;
    private JLabel LB_Kreisfrequenz_Wert = null;
    private JLabel LB_Phasenwinkel_Wert = null;
    private JLabel LB_Abgriff = null;
    private JLabel Logo = null;

    // Textfelder
    private JTextField TF_Amplitude_Wert= null;
    private JTextField TF_Kreisfrequenz_Wert = null;
    private JTextField TF_Phasenwinkel_Wert = null;
    private JTextField TF_Abgriff = null;

    // Slider (Scrollbalken)
    private JSlider sl_Amplitude_Wert = null;
    private JSlider sl_Kreisfrequenz_Wert = null;
    private JSlider sl_Phasenwinkel_Wert = null;
    private JSlider sl_Abgriff = null;

    // Interne Variablen
    private double Amplitude_Wert = 1;
    private double Kreisfrequenz_Wert = 1;
    private double Phasenwinkel_Wert = 0;
    private double Abgriff_Wert = 1;

    /**
     * Konstruktor erzeugt die Eingabefelder
     */
    public ParameterEingabeCosinus () {
        String Beschriftung;

        // wir legen alle Elemente auf ein Panel
        innerContent = new JPanel();
        // wir verwenden keinen Layoutmanager
        innerContent.setLayout(null);

        // Erzeugung der Text-Eingabefelder, mit dem jeweiligen Wert und
        // lassen maximal 10 Zeichen zu
        Beschriftung = df2.format(Amplitude_Wert);
        TF_Amplitude_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Kreisfrequenz_Wert);
        TF_Kreisfrequenz_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Phasenwinkel_Wert);
        TF_Phasenwinkel_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Abgriff_Wert);
        TF_Abgriff = new JTextField(Beschriftung,10);

        // wir erzeugen die Berschriftungslabels
        LB_Amplitude_Wert = new JLabel("Amplitude a:");
        LB_Kreisfrequenz_Wert = new JLabel("Kreisfrequenz b:");
        LB_Phasenwinkel_Wert = new JLabel("Phasenwinkel c:");
        LB_Abgriff = new JLabel("x-Wert:");
        Logo = new JLabel("");

        // Erzeugung der Schieberegler --> JSlider (min, max, value)
        sl_Amplitude_Wert = new JSlider(-250, 250, (int)(Genauigkeit * Amplitude_Wert));
        sl_Kreisfrequenz_Wert = new JSlider(-250, 250, (int) (Genauigkeit * Kreisfrequenz_Wert));
        sl_Phasenwinkel_Wert = new JSlider(-700, 700, (int) (Genauigkeit * Phasenwinkel_Wert));
        sl_Abgriff = new JSlider(-398, 398,(int)(Genauigkeit*Abgriff_Wert));

        // Positionierung der Elemente
        int left = 5;
        int width = 120;
        int height = 21;
        int abstand = 26;
        int top = 7 - abstand;

        // Elementbegrenzungen setzen
        LB_Amplitude_Wert.setBounds(left,top+=abstand,width,height);
        TF_Amplitude_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Amplitude_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Kreisfrequenz_Wert.setBounds(left,top+=abstand,width,height);
        TF_Kreisfrequenz_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Kreisfrequenz_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Phasenwinkel_Wert.setBounds(left,top+=abstand,width,height);
        TF_Phasenwinkel_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Phasenwinkel_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.3;
        LB_Abgriff.setBounds(left,top+=abstand,width,height);
        TF_Abgriff.setBounds(left,top+=abstand/1.5,width,height);
        sl_Abgriff.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.4;
        if (Ableitung.getIconLogo() != null)
            Logo.setIcon(Ableitung.getIconLogo());
        Logo.setBounds(left,top+=abstand,120,108);

        /**
         * Reaktion auf die nderung der Eingabefelder
         */
        TF_Amplitude_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Amplitude_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Amplitude_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Amplitude_Wert.hasFocus()) return;
                String Beschriftung1;
                Amplitude_Wert = sl_Amplitude_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Amplitude_Wert);
                TF_Amplitude_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Kreisfrequenz_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Kreisfrequenz_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Kreisfrequenz_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Kreisfrequenz_Wert.hasFocus()) return;
                String Beschriftung1;
                Kreisfrequenz_Wert = sl_Kreisfrequenz_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Kreisfrequenz_Wert);
                TF_Kreisfrequenz_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Phasenwinkel_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Phasenwinkel_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Phasenwinkel_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Phasenwinkel_Wert.hasFocus()) return;
                String Beschriftung1;
                Phasenwinkel_Wert = sl_Phasenwinkel_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Phasenwinkel_Wert);
                TF_Phasenwinkel_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Abgriff.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Abgriff.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Abgriff.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Abgriff.hasFocus()) return;
                String Beschriftung1;
                Abgriff_Wert = sl_Abgriff.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Abgriff_Wert);
                TF_Abgriff.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        // Gre des Eingabebereichs zulassen
        innerContent.setPreferredSize(new Dimension(left*2+width,20));

        // Widgets auf dem Panel plazieren
        innerContent.add(LB_Amplitude_Wert);
        innerContent.add(TF_Amplitude_Wert);
        innerContent.add(sl_Amplitude_Wert);

        innerContent.add(LB_Kreisfrequenz_Wert);
        innerContent.add(TF_Kreisfrequenz_Wert);
        innerContent.add(sl_Kreisfrequenz_Wert);

        innerContent.add(LB_Phasenwinkel_Wert);
        innerContent.add(TF_Phasenwinkel_Wert);
        innerContent.add(sl_Phasenwinkel_Wert);

        innerContent.add(LB_Abgriff);
        innerContent.add(TF_Abgriff);
        innerContent.add(sl_Abgriff);

        innerContent.add(Logo);
    }

    /**
     * Methode delete dient dazu eine Eingabe aus dem Speicher aufzulsen
     *
     * @param Fenster	Fenster auf dem die Elemente plaziert sind
     */
    public void delete(JFrame Fenster) {
        // Eingabefelder freigeben
        innerContent.remove(LB_Amplitude_Wert);
        innerContent.remove(TF_Amplitude_Wert);
        innerContent.remove(sl_Amplitude_Wert);

        innerContent.remove(LB_Kreisfrequenz_Wert);
        innerContent.remove(TF_Kreisfrequenz_Wert);
        innerContent.remove(sl_Kreisfrequenz_Wert);

        innerContent.remove(LB_Phasenwinkel_Wert);
        innerContent.remove(TF_Phasenwinkel_Wert);
        innerContent.remove(sl_Phasenwinkel_Wert);

        innerContent.remove(LB_Abgriff);
        innerContent.remove(TF_Abgriff);
        innerContent.remove(sl_Abgriff);

        innerContent.remove(Logo);

        // Referenzen lschen
        LB_Amplitude_Wert = null;
        LB_Kreisfrequenz_Wert = null;
        LB_Phasenwinkel_Wert = null;
        LB_Abgriff = null;
        Logo = null;

        TF_Amplitude_Wert = null;
        TF_Kreisfrequenz_Wert = null;
        TF_Phasenwinkel_Wert = null;
        TF_Abgriff = null;

        sl_Amplitude_Wert = null;
        sl_Kreisfrequenz_Wert = null;
        sl_Phasenwinkel_Wert = null;
        sl_Abgriff = null;

        // selbst auflsen
        Fenster.getContentPane().remove(innerContent);
        innerContent = null;
        Zeichenbereich1 = null;
        Zeichenbereich2 = null;
    }

    /**
     * setzt den Amplituden-Wert von aussen
     *
     * @param wert	Neuer Amplituden-Wert
     */
    public void setAmplitude_Wert(double wert) {
        Amplitude_Wert = wert;
        String Beschriftung = df2.format(Amplitude_Wert);
        TF_Amplitude_Wert.setText(Beschriftung);
        sl_Amplitude_Wert.setValue( (int) (Amplitude_Wert*Genauigkeit) );
    }

    /**
     * setzt den Kreisfrequenz-Wert von aussen
     *
     * @param wert	Neuer Kreisfrequenz-Wert
     */
    public void setKreisfrequenz_Wert(int wert) {
        Kreisfrequenz_Wert = wert;
        String Beschriftung = df2.format(Kreisfrequenz_Wert);
        TF_Kreisfrequenz_Wert.setText(Beschriftung);
        sl_Kreisfrequenz_Wert.setValue( (int) (Kreisfrequenz_Wert * Genauigkeit) );
    }

    /**
     * setzt den Phasenwinkel-Wert von aussen
     *
     * @param wert	Neuer Phasenwinkel-Wert
     */
    public void setPhasenwinkel_Wert(int wert) {
        Phasenwinkel_Wert = wert;
        String Beschriftung = df2.format(Phasenwinkel_Wert);
        TF_Phasenwinkel_Wert.setText(Beschriftung);
        sl_Phasenwinkel_Wert.setValue( (int) (Phasenwinkel_Wert*Genauigkeit) );
    }

    /**
     * liefert den Wert der Amplitude
     *
     * @return derzeitige Amplitude
     */
    public double getAmplitude_Wert() {
        return Amplitude_Wert ;
    }

    /**
     * liefert den Wert der Kreisfrequenz
     *
     * @return derzeitige Kreisfrequenz
     */
    public double getKreisfrequenz_Wert() {
        return Kreisfrequenz_Wert;
    }

    /**
     * liefert den Wert des Phasenwinkels
     *
     * @return derzeitiger Phasenwinkel
     */
    public double getPhasenwinkel_Wert() {
        return Phasenwinkel_Wert;
    }

    /**
     * setzt die Zeichenbereiche
     *
     * @param z	Zeichenbereich
     */
    public void setZeichenbereich(ZeichenbereichCosinus z) {
        if (Zeichenbereich1==null)
            Zeichenbereich1=z;
        else
            Zeichenbereich2=z;
    }

    /**
     * liefert den Abgriff
     *
     * @return derzeitiger Abgriff
     */
    public double getAbgriff() {
        return	Abgriff_Wert;
    }

    /**
     * setzt den Abgriff
     *
     * @param x	Neuer Abgriff
     */
    public void setAbgriff(double x) {
        // Abfangen der unzulssigen Werte
        Abgriff_Wert = x;
        if (Abgriff_Wert > -3.98 && Abgriff_Wert < 3.98) {
            sl_Abgriff.setValue( (int)(Genauigkeit*Abgriff_Wert) );
            String Beschriftung1;
            Beschriftung1 = df2.format(Abgriff_Wert);
            TF_Abgriff.setText(Beschriftung1);
        } else if (Abgriff_Wert > 3.98) {
            sl_Abgriff.setValue( (int)(Genauigkeit*3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(3.98);
            TF_Abgriff.setText(Beschriftung1);
            Abgriff_Wert = 3.98;
        } else {
            sl_Abgriff.setValue( (int)(Genauigkeit*-3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(-3.98);
            TF_Abgriff.setText(Beschriftung1);
            Abgriff_Wert = -3.98;
        }
        if (Zeichenbereich1 != null) Zeichenbereich1.Dorepaint();
        if (Zeichenbereich2 != null) Zeichenbereich2.Dorepaint();
    }

    /**
     * fgt die Eingabeleiste auf einem Fenster ein
     *
     * @param Fenster	Fenster auf dem die Leiste eingefgt werden soll
     */
    public void addToFrame(JFrame Fenster) {
        Fenster.getContentPane().add(innerContent, BorderLayout.WEST);
    }

    /**
     * bernimmt alle Werte bei der Texteingabe
     */
    private void eingabewerteEinlesen() {
        // Amplitude
        {
            String Hilf = TF_Amplitude_Wert.getText();
            Amplitude_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Amplitude_Wert < -2.50 ) {
                Amplitude_Wert = -2.50;
                TF_Amplitude_Wert.setText("-2,50");
            }
            if (Amplitude_Wert > 2.5 ) {
                Amplitude_Wert = 2.50;
                TF_Amplitude_Wert.setText("2,50");
            }
            sl_Amplitude_Wert.setValue( (int)(Genauigkeit*Amplitude_Wert) );
        }

        // Kreisfrequenz
        {
            String Hilf = TF_Kreisfrequenz_Wert.getText();
            Kreisfrequenz_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Kreisfrequenz_Wert < -2.50 ) {
                Kreisfrequenz_Wert = -2.50;
                TF_Kreisfrequenz_Wert.setText("-2,50");
            }
            if (Kreisfrequenz_Wert > 2.5 ) {
                Kreisfrequenz_Wert = 2.50;
                TF_Kreisfrequenz_Wert.setText("2,50");
            }
            sl_Kreisfrequenz_Wert.setValue( (int) (Genauigkeit*Kreisfrequenz_Wert) );
        }

        // Phasenwinkel
        {
            String Hilf = TF_Phasenwinkel_Wert.getText();
            Phasenwinkel_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Phasenwinkel_Wert < -7 ) {
                Phasenwinkel_Wert = -7;
                TF_Phasenwinkel_Wert.setText("-7,00");
            }
            if (Phasenwinkel_Wert > 7 ) {
                Phasenwinkel_Wert = 7;
                TF_Phasenwinkel_Wert.setText("7,00");
            }
            sl_Phasenwinkel_Wert.setValue( (int) (Genauigkeit * Phasenwinkel_Wert) );
        }

        // Abgriff
        {
            String Hilf = TF_Abgriff.getText();
            Abgriff_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Abgriff_Wert < -3.98 ) {
                Abgriff_Wert = -3.98;
                TF_Abgriff.setText("-3,98");
            }
            if (Abgriff_Wert > 3.98 ) {
                Abgriff_Wert = 3.98;
                TF_Abgriff.setText("3,98");
            }
            sl_Abgriff.setValue( (int) (Genauigkeit * Abgriff_Wert) );
        }
    }
}
