
package source;

/**.
 * Klasse Skalierung
 * bernimmt die Berechnungen fr die Anzeige der Cosinusfunktion.
 * @author Horst Lauinger  03.11.2007
 */
public class SkalierungCosinus {
    // Gre des Zeichenbereichs
    private int Width;
    private int Height;

    //Randbreite
    private int Rand = 20;

    // Breite der Grafik
    private double Breite = 4;

    // bestimmt ob die Darstellung abgeleitet ist
    private boolean isNormaled;

    // berechnete Werte fr Zeichenbereich
    private double von;
    private double bis;
    private double bereich_werte;
    private int bereich_grafik_X;
    private int bereich_grafik_Y;

    // Umfrechnungsfaktoren fr die Skalierung
    private double Skalierungs_Faktor_X;
    private double Skalierungs_Faktor_Y;
    private double Achsen_Verschiebung_X;

    private double Kreisfrequenz;
    private double Amplitude;
    private double Phasenwinkel;

    // Nullpunktoffset
    private int Offset_X;
    private int Offset_Y;

    // Referenz auf die Eingaben
    private ParameterEingabeCosinus parameterEingabeC = null;

    /**
     * Konstruktor
     * bernimmt die Parameter fr die Klasse
     * Referenz auf Eingaben und ob abgeleitete Darstellung
     *
     * @param PE 	Referenz auf das zugehrige Eingabefeld
     */
    public SkalierungCosinus(ParameterEingabeCosinus PE, boolean Normaled) {
        // Werte in den internen Variablen ablegen
        parameterEingabeC = PE;
        isNormaled = Normaled;

        // Standardmig Skalierung = 0, so muss sie spter neu berechnet werden
        Skalierungs_Faktor_X = 0;
        Skalierungs_Faktor_Y = 0;
    }

    /**
     * Verlangt zwingend neue Berechnung, mit neuer Skalierung
     *
     * @param x	Breite des Zeichenbereiches
     * @param y	Hhe des Zeichenbereiches
     */
    public void ForceRechnen(int x, int y) {
        // Werte auf 0 setzen, damit sie neu berechnet werden
        Skalierungs_Faktor_X = 0;
        Skalierungs_Faktor_Y = 0;
        // jetzt neue Daten berechnen
        Rechnen(x, y);
    }

    /**
     * Berechnet neue interne Werte
     *
     * @param x	Breite des Zeichenbereiches
     * @param y	Hhe des Zeichenbereiches
     */
    public void Rechnen(int x, int y) {
        Width = x;
        Height = y;

        bereich_grafik_X = Width - Rand * 4 ;
        bereich_grafik_Y = Height - Rand * 4 ;

        // Unterscheidung bei abgeleiteter Darstellung
        if (!isNormaled) {
            // nicht abgeleitet
            // Funktionswerte bernehmen
            Amplitude = parameterEingabeC.getAmplitude_Wert();
            Kreisfrequenz = parameterEingabeC.getKreisfrequenz_Wert();
            Phasenwinkel = parameterEingabeC.getPhasenwinkel_Wert();
        } else {
            // Ableitung
            Amplitude = - parameterEingabeC.getAmplitude_Wert()*parameterEingabeC.getKreisfrequenz_Wert();
            Kreisfrequenz = parameterEingabeC.getKreisfrequenz_Wert();
            Phasenwinkel = parameterEingabeC.getPhasenwinkel_Wert()-Math.PI/2;
        }

        // die Y-Achse ist um den Mittelwert verschoben, so steht er immer in der Mitte
        Achsen_Verschiebung_X = 0;

        // Breite links und rechts vom Mittelwert ergeben Zeichengrenzen
        von = - Breite;
        bis = + Breite;

        // Bereich ausrechnen
        bereich_werte = (bis - von);

        // X-Skalierung neu berechnen
//		if (Skalierungs_Faktor_X==0)
        Skalierungs_Faktor_X = bereich_grafik_X / bereich_werte ;

        // Y-Skalierung neu berechnen
//		if ( Skalierungs_Faktor_Y==0)
        Skalierungs_Faktor_Y = bereich_grafik_Y /2/3;

        // Die Offsets geben den Abstand zum Ursprung an
        Offset_X = Width / 2;
        Offset_Y = Height/2 +Rand /2 ;
    }

    /**
     * liefert die Breite
     *
     * @return Breite des Zeichenbereiches in Pixel
     */
    public int getWidth() {
        return Width;
    }

    /**
     * liefert die Hhe
     *
     * @return Hhe des Zeichenbereiches in Pixel
     */
    public int getHeight() {
        return Height;
    }

    /**
     * liefert Position der Y-Achse
     *
     * @return X-Position der Y-Achse in Pixel
     */
    public int getPosYAchse() {
        // Position berechnen
        int Interner_Wert = (int) (Offset_X + Achsen_Verschiebung_X * Skalierungs_Faktor_X);
        // prfen ob ausserhalb und bei Bedarf begrenzen
        if (Interner_Wert > Width-3*Rand) Interner_Wert = Width-3*Rand;
        if (Interner_Wert < 3*Rand) Interner_Wert = 3*Rand;

        return Interner_Wert;
    }

    /**
     * Zeigt an ob Y-Achse versetzt
     *
     * @return 0 wenn nicht versetzt, -1 wenn links versetzt, 1 wenn rechts versetzt
     */
    public int getIfVersetzteYAchse() {
        // Position berechnen
        int Interner_Wert = (int) (Offset_X + Achsen_Verschiebung_X * Skalierungs_Faktor_X);
        // prfen ob ausserhalb und bei Bedarf begrenzen
        if (Interner_Wert > Width-3*Rand) return 1;
        if (Interner_Wert < 3*Rand) return -1;
        // sonst Standardrckgabe, als Kennzeichung fr nicht verschoben
        return 0;
    }

    /**
     * liefert Position der X-Achse
     *
     * @return Y-Position der X-Achse in Pixel
     */
    public int getPosXAchse() {
        return Offset_Y;
    }

    /**
     * liefert die Randbreite
     *
     * @return Breite des Randes in Pixel
     */
    public int getRand() {
        return Rand;
    }

    /**
     * liefert den Abstand der Beschriftungen auf der Achse
     * abhngig vom verfgbaren Platz
     *
     * @return Abstand der Beschriftungen auf der X Achse
     */
    public double getBeschriftungAbstandX() {
        double returnWert;

        returnWert = 35.0/Skalierungs_Faktor_X;

        if (returnWert <= 0.001) return  0.001;
        if (returnWert <= 0.002) return  0.002;
        if (returnWert <= 0.005) return  0.005;
        if (returnWert <= 0.01) return  0.01;
        if (returnWert <= 0.02) return  0.02;
        if (returnWert <= 0.05) return  0.05;
        if (returnWert <= 0.1) return  0.1;
        if (returnWert <= 0.2) return  0.2;
        if (returnWert <= 0.5) return  0.5;
        if (returnWert <= 1) return  1;
        if (returnWert <= 2) return  2;
        if (returnWert <= 5) return  5;
        if (returnWert <= 10) return  10;
        if (returnWert <= 20) return  20;
        if (returnWert <= 50) return  50;

        return returnWert;
    }

    /**
     * liefert den Abstand der Beschriftungen auf der Achse
     * abhngig vom verfgbaren Platz
     *
     * @return Abstand der Beschriftungen auf der Y Achse
     */
    public double getBeschriftungAbstandY() {
        double int_wert = 18/Skalierungs_Faktor_Y;
        if (int_wert <= 0.001) return 0.001;
        if (int_wert <= 0.002) return 0.002;
        if (int_wert <= 0.005) return 0.005;
        if (int_wert <= 0.01) return 0.01;
        if (int_wert <= 0.02) return 0.02;
        if (int_wert <= 0.05) return 0.05;
        if (int_wert <= 0.10) return 0.10;
        if (int_wert <= 0.20) return 0.20;
        if (int_wert <= 0.50) return 0.50;
        if (int_wert <= 1.00) return 1.00;
        if (int_wert <= 2.00) return 2.00;
        if (int_wert <= 5.00) return 5.00;
        if (int_wert <= 10.00) return 10.00;

        return int_wert;
    }

    /**
     * Liefert den Skalierungsfaktor in X Richtung
     *
     * @return Skalierungsfaktor in X Richtung
     */
    public double getSkalierungs_Faktor_X() {
        return Skalierungs_Faktor_X;
    }

    /**
     * Liefert den Skalierungsfaktor in Y Richtung
     *
     * @return Skalierungsfaktor in Y Richtung
     */
    public double getSkalierungs_Faktor_Y() {
        return Skalierungs_Faktor_Y;
    }

    /**
     * liefert die linke Begrenzung
     *
     * @return Position (Untergrenze) ab der die Kurve dargestellt wird
     */
    public double getVon() {
        return von;
    }

    /**
     * liefert die rechte Textbegrenzung
     *
     * @return Position (Obergrenze) ab der der Text dargestellt wird
     */
    public double getTextBis() {
        return bis;
    }

    /**
     * liefert die linke TextBegrenzung
     *
     * @return Position (Untergrenze) ab der der Text dargestellt wird
     */
    public double getTextVon() {
        return von;
    }

    /**
     * liefert die rechte Begrenzung
     *
     * @return Position (Obergrenze) ab der die Kurve dargestellt wird
     */
    public double getBis() {
        return bis;
    }

    /**
     * liefert die Y-Textposition zum Y Wert
     *
     * @param value		gewnschte Position des Textes
     *
     * @return zugehriger skalierter Y Wert
     */
    public double getYAchseTextY(double value) {
        return Offset_Y -value* Skalierungs_Faktor_Y;
    }

    /**
     * liefert die X-Textposition zum X Wert
     *
     * @param value		gewnschte Position des Textes
     *
     * @return zugehriger skalierter X Wert
     */
    public double getXAchseTextX(double value) {
        return Offset_X +(value+Achsen_Verschiebung_X)* Skalierungs_Faktor_X;
    }

    /**
     * liefert den X Cosinus Wert
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter X Wert
     */
    public double getCosinusWertX(double value) {
        return Offset_X + (value) * Skalierungs_Faktor_X;
    }

    /**
     * liefert den Y Cosinus Wert
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double getCosinusWertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - Amplitude*Math.cos(Kreisfrequenz*value+Phasenwinkel) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den x-Abgriff y-Wert
     *
     * @param value		gewnschte Steigung der Tangente (mathematisch)
     *
     * @return zugehriger skalierter y-Wert (Zeichenbereich)
     */
    public double getSteigung(double value) {
        double Steigung;
        Steigung = Amplitude*Kreisfrequenz*Math.cos(Kreisfrequenz*value+Phasenwinkel);
        return Steigung;

    }

    /**
     * liefert Realen Funktionswert zu einem Wert in der Darstellung
     *
     * @param x 	Wert in der Darstellung
     *
     * @return realer (mathematischer) Funktionswert
     */
    public double getRealYWert(double x) {
        double hilf = ((double)Offset_Y - x) / Skalierungs_Faktor_Y;
        return hilf ;
    }

    /**
     * liefert Realen Funktionswert zu einem Wert in der Darstellung
     *
     * @param x 	Wert in der Darstellung
     *
     * @return realer (mathematischer) Funktionswert
     */
    public double getRealXWert(int x) {
        double hilf = ((double)x - (double)Offset_X) / Skalierungs_Faktor_X - Achsen_Verschiebung_X;
        return hilf ;
    }

    /**
     * liefert den skalierten x-Wert
     *
     * @param x
     * @return	skalierter x-Wert
     */
    public double getSkalXWert (double x) {
        double hilf = (x + Achsen_Verschiebung_X - (Skalierungs_Faktor_X/(double)Offset_X)/20);
        return hilf;
    }
}
