
package source;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.text.DecimalFormat;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import javax.imageio.ImageIO;
import java.io.IOException;

/**
 * Klasse Zeichenbereich
 * Erzeugt den eigentlichen Zeichenbereich auf dem die Grafik dargestellt wird
 * @author Horst Lauinger  20.11.2007
 */
public class ZeichenbereichLNFunk extends JPanel {
    /**
     * automatisch generierte UID
     */
    private static final long serialVersionUID = 1L;
    // Pfeilgren
    private int Pfeil_laenge = 12;
    private int Pfeil_breite = 3;

    // Referenz auf die Eingabeparameter
    private ParameterEingabeLNFunktion parameterEingabeLN = null;

    // Tolleranzbereich fr das Verschieben des Abgriffs
    private double Klick_Tolleranz = 0.1;
    // erlaubt Verschieben des Abgriffs
    private boolean CanDrag = false;

    // Referenz auf die Skalierungsklasse
    private SkalierungLNFunk skalierung = null;

    // bestimmt ob dieser Zeichenbereich normiert dargestellt werden soll
    private boolean IsNormaled = false;

    double value;
    private Image Formel;
    private Image FormelA;

    /**
     * Konstruktor
     * erzeugt neuen Zeichenbereich
     *
     * @param w 	Referenz auf die Eingaben
     */
    public ZeichenbereichLNFunk(ParameterEingabeLNFunktion w, boolean isNormaled) {
        // Parameter bernehmen und sichern
		
		try{
			Formel  = ImageIO.read(Ableitung.class.getResourceAsStream("/images/LNFunk.PNG"));
			Formel  = ImageIO.read(Ableitung.class.getResourceAsStream("/images/LNFunkA.PNG"));
		}catch(IOException ioe){
		}
			
        parameterEingabeLN = w;
        parameterEingabeLN.setZeichenbereich(this);
        IsNormaled = isNormaled;

        // Mausaktionen verwenden
        addMouseListener(new MyMouseListener());
        addMouseMotionListener(new MyMouseMotionListener());

        // neue Skalierungsberechnung erzeugen
        skalierung = new SkalierungLNFunk(parameterEingabeLN, IsNormaled);
    }

    /**
     * lscht den Zeichenbereich
     * und gibt interne Referenzen frei
     */
    public void delete() {
        parameterEingabeLN = null;
        skalierung = null;
    }

    /**
     * veranlasst ein Neuzeichnen der Visualisierung
     */
    public void Dorepaint() {
        this.repaint();
    }

    /**
     * erzwingt eine Neuberechnung der Skalierung
     */
    public void DoRecalc() {
        skalierung.ForceRechnen(getWidth(), getHeight());
        this.repaint();
    }

    /**
     * Neuzeichnen der Grafik erforderlich
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    protected void paintComponent( Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // bergeordnetes Zeichen aktivieren
        super.paintComponent(g);
        // Skalierung aktualisieren
        skalierung.Rechnen(getWidth(), getHeight());
        // einzelne Bereiche neu Zeichnen
        PrintBackground(g);
        PrintAbgriff(g);
        PrintAchsen(g);
        PrintLNFunktion(g);
        PrintTangente (g);

        // bei Normierter Darstellung entsprechende Einblendung
        if (IsNormaled) {
            g2d.setColor(Color.black);
            g2d.drawImage(FormelA, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); // 1420/13, 34,
        } else {
            g2d.setColor(Color.black);
            g2d.drawImage(Formel, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); // 1798/13, 22,
        }
    }

    /**
     * Zeichnet x-Abgriff
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintAbgriff (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Abgriff blau darstellen
        g2d.setColor(Color.BLUE);

        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf;
        int Punkt_x, Punkt_y;

        // x-Abgriff holen
        lauf=parameterEingabeLN.getAbgriff();
        // Umrechnen in Grafikkoordinaten
        Punkt_x = (int) skalierung.getLNFunkWertX(lauf);
        Punkt_y = (int) skalierung.getLNFunkWertY(lauf);
        // Sicherstellen dass darstellbar
        if (Punkt_x > 2*skalierung.getRand() && Punkt_x < skalierung.getWidth()-2*skalierung.getRand()) {
            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8) {
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, skalierung.getPosXAchse()+ skalierung.getHeight()/2);
                } else {
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                }
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, skalierung.getPosXAchse()- skalierung.getHeight()/2-10, Punkt_x, skalierung.getPosXAchse()+ skalierung.getHeight()/2-2*skalierung.getRand());
                g2d.setStroke(OldStoke);
            }
        }
        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }

    private void PrintTangente(Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Tangente grn darstellen
        g2d.setColor(Color.GREEN);
        Font bold = new Font ("Arial",Font.BOLD,12);

        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf;
        double Punkt_x, Punkt_y ,Punkt_xo, Punkt_yo, Puffx, Puffy;

        // x-Abgriff holen
        lauf = parameterEingabeLN.getAbgriff();

        // Umrechnen in Grafikkoordinaten
        Punkt_xo =  skalierung.getLNFunkWertX(lauf-0.025);
        Punkt_yo =  skalierung.getLNFunkWertY(lauf-0.025);
        Punkt_x =  skalierung.getLNFunkWertX(lauf+0.025);
        Punkt_y =  skalierung.getLNFunkWertY(lauf+0.025);

        // Tangente nur zeichnen, wenn Abgriff innerhalb der Zeichengrenzen
        if (Punkt_x>= 2*skalierung.getRand() && Punkt_xo<= (skalierung.getWidth()-2*skalierung.getRand())) {
            // Lnge der Tangente begrenzen und zeichnen
            for ( int t=0; (skalierung.getWidth()-2*skalierung.getRand() > Punkt_xo)&&(2*skalierung.getRand()<=Punkt_yo)&&(skalierung.getHeight()-skalierung.getRand()>=Punkt_yo) && t < 40; t++) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawLine((int)Punkt_xo,(int) Punkt_yo, (int)(Punkt_x),(int) (Punkt_y));
                // x- und y-Punkte weiterreichen
                Puffx = Punkt_x;
                Puffy = Punkt_y;
                Punkt_x += Punkt_x - Punkt_xo;
                Punkt_y += Punkt_y - Punkt_yo;
                Punkt_xo = Puffx;
                Punkt_yo = Puffy;
            }

            // Umrechnen in Grafikkoordinaten
            Punkt_xo =  skalierung.getLNFunkWertX(lauf-0.025);
            Punkt_yo =  skalierung.getLNFunkWertY(lauf-0.025);
            Punkt_x =  skalierung.getLNFunkWertX(lauf+0.025);
            Punkt_y =  skalierung.getLNFunkWertY(lauf+0.025);

            for ( int s=0; (2*skalierung.getRand()< Punkt_x) &&(2*skalierung.getRand()<=Punkt_yo)&&(skalierung.getHeight()-skalierung.getRand()>=Punkt_yo) && s < 40; s++) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawLine((int)Punkt_x,(int) Punkt_y, (int)(Punkt_xo),(int) (Punkt_yo));
                Puffx = Punkt_xo;
                Puffy = Punkt_yo;
                Punkt_xo -= Punkt_x - Punkt_xo;
                Punkt_yo -= Punkt_y - Punkt_yo;
                Punkt_x = Puffx;
                Punkt_y = Puffy;
            }

            g2d.setColor(Color.GRAY);
            g.setFont(bold);
            if ((skalierung.getHeight()-10)>skalierung.getLNFunkWertY(lauf) && (skalierung.getLNFunkWertY(lauf)-20)>skalierung.getRand()) {
                if (IsNormaled)	// nur im zweiten Zeichenbereich zeichnen
                    g.drawString(""+Math.round(skalierung.getSteigung(lauf)*100)/100., (int)skalierung.getLNFunkWertX(lauf)+6,(int) skalierung.getLNFunkWertY(lauf));
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawString(""+Math.round(skalierung.getRealYWert(skalierung.getLNFunkWertY(lauf))*100)/100., (int)skalierung.getLNFunkWertX(lauf)-10,(int) skalierung.getLNFunkWertY(lauf)-5);

            }
            g2d.setColor(Color.GREEN);
        }
        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }

    /**
     * Zeichnet den eigentlichen Verlauf der Kurve
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintLNFunktion (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        if (!IsNormaled) {	// Mit Rot Zeichnen
            g2d.setColor(Color.RED);
            g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
        } else {	// Mit Grn Zeichnen
            g2d.setColor(Color.GREEN);
            g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
        }
        // Sichtbaren Bereich durchlaufen
        for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.3d/skalierung.getSkalierungs_Faktor_X()) {
            // Koordinaten umrechnen
            int Punkt_x = (int) skalierung.getLNFunkWertX(lauf);
            int Punkt_y = (int) skalierung.getLNFunkWertY(lauf);
            if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
        }
    }

    /**
     * Zeichnet die Achsen
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintAchsen (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Falls die Y-Achse nicht durch x=0 geht, dies mit Zeichen deutlichmachen
        if (skalierung.getIfVersetzteYAchse()<0) {
            g2d.setColor(Color.BLACK);
            g.drawString("//", (int)(3.01*skalierung.getRand()), skalierung.getPosXAchse()+5);
        }
        if (skalierung.getIfVersetzteYAchse()>0) {
            g2d.setColor(Color.BLACK);
            g.drawString("//", (int)(skalierung.getWidth()-3.4*skalierung.getRand()), skalierung.getPosXAchse()+5);
        }

        // X-Achse
        g2d.setColor(Color.BLACK);
        g.drawLine(2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()), skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()));

        // Y-Achse
        g.drawLine(skalierung.getPosYAchse(), 2*skalierung.getRand(), skalierung.getPosYAchse(), skalierung.getHeight()-2*skalierung.getRand());

        // Pfeile an die Achsen
        for (int i=Pfeil_laenge; i>=0; i--) {
            // Oben
            g.drawLine(skalierung.getPosYAchse() - Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);
            g.drawLine(skalierung.getPosYAchse() + Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);

            //Rechts
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()-Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()+Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
        }

        // Beschriftungen
        Font Old_font = g.getFont();
        Font font = new Font ("Arial", Font.PLAIN, 13);
        g.setFont(font);
        // X - Achse
        if (IsNormaled)
            g.drawString("x", (int)skalierung.getWidth()-2*skalierung.getRand()+5, (int)(1*skalierung.getPosXAchse()+13));
        else
            g.drawString("x", skalierung.getWidth()-2*skalierung.getRand()+5, (int)(1*skalierung.getPosXAchse()+13));

        // Y - Achse
        if (IsNormaled)
            g.drawString("f'(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        //g.drawString("\u03C6(u)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        else
            g.drawString("f(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        //g.drawString("\u03C6(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);

        g.setFont(Old_font);

        // Achsenbeschriftung
        // X - Achsen Beschriftung von 0 .. oo
        for (double lauf=skalierung.getBeschriftungAbstandX(); lauf <= 5.5  ; lauf+=skalierung.getBeschriftungAbstandX()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );

            if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
            if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
            if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
            int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height ) ;

            // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

            if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2.5*skalierung.getRand()) {
                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, (int)(1.0*Punkt_y_Text));

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
            }
        }

        // X - Achsen Beschriftung von -oo .. 0
        for (double lauf=-skalierung.getBeschriftungAbstandX(); lauf > skalierung.getTextVon()  ; lauf-=skalierung.getBeschriftungAbstandX()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );

            if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
            if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
            if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
            int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height) ;

            // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

            if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2*skalierung.getRand()) {
                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, (int)(1*Punkt_y_Text));

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
            }
        }

        // Y - Achsen Beschriftung von 0 .. oo
        for (double lauf=0; lauf <= 3*1.1 ; lauf+=skalierung.getBeschriftungAbstandY()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );
            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
            int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

            // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

            // Text ausgeben
            g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

            // Kleine Markierungen anbringen, wo der Text steht
            g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
        }

        // Y - Achsen Beschriftung von -oo .. 0
        for (double lauf=0; lauf >= -3*0.95  ; lauf-=skalierung.getBeschriftungAbstandY()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );
            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
            int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

            // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

            // Text ausgeben
            g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

            // Kleine Markierungen anbringen, wo der Text steht
            g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
        }
    }

    /**
     * Hintergund wei darstellen
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintBackground (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Rahmen
        Color Farbe = new Color(0xffdcdc00);
        g2d.setColor(Farbe);
        g2d.setColor(Color.WHITE);
        g2d.fill(new Rectangle(skalierung.getRand(),skalierung.getRand(),skalierung.getWidth()-skalierung.getRand()*2,skalierung.getHeight()-skalierung.getRand()*2));
    }

    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseListener extends MouseAdapter {
        /**
         * Methode wird aufgerufen, falls Maustaste gedrckt wurde
         *
         * @param evt 	MouseEvent
         */
        public void mousePressed(MouseEvent evt) {
            // Mausposition holen
            int posX = evt.getX();
            // aus Mauskoordinaten Orginalwert errechnen
            skalierung.Rechnen(getWidth(), getHeight());
            double Calculated_X_Value = skalierung.getRealXWert(posX);

            // Tolleranz bercksichtigen
            Klick_Tolleranz = 0.1d ; //* parameterEingabe.getStandardabweichung();

            // Gltiger Bereich?
            if (
                Calculated_X_Value >= parameterEingabeLN.getAbgriff() - Klick_Tolleranz
                &&
                Calculated_X_Value <= parameterEingabeLN.getAbgriff() + Klick_Tolleranz
            )
                CanDrag = true;
        }

        /**
         * Methode wird aufgerufen, falls die "Maus" das Koordinatensystem verlassen hat, d.h.
         * im Ausgabemen, darf nun keine Mausposition mehr angezeigt werden.
         *
         * @param evt 	MouseEvent
         */
        public void mouseExited(MouseEvent evt) {
            if (CanDrag) {
                CanDrag = false;
            }
            setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        }

        /**
         * Methode wird aufgerufen, falls die Maustaste losgelassen wurde, d.h.
         * ein Punkt wurde verschoben bzw. aus dem Koordinatensystem gezogen und
         * wird gelscht.
         *
         * @param evt 	MouseEvent
         */
        public void mouseReleased(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag) {
                // ende Verschiebung
                CanDrag = false;
                // derzeige Mausposition
                int posX = evt.getX();
                // wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);

                parameterEingabeLN.setAbgriff(skalierung.getSkalXWert(Calculated_X_Value));
            }
        }
    }

    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseMotionListener extends MouseMotionAdapter {
        /**
         * Methode wird aufgerufen, falls Maus bewegt wurde.
         * Maus Cursor wird angepasst
         *
         * @param evt 	MouseEvent
         */
        public void mouseMoved(MouseEvent evt) {
            if (CanDrag != true) {
                // derzeige Mausposition
                int posX = evt.getX();
                // Wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);

                Klick_Tolleranz = 0.05d ; // * parameterEingabe.getStandardabweichung();

                // Mauszeiger ber Abgrifflinie -> Zeiger ndern
                if (
                    Calculated_X_Value >= parameterEingabeLN.getAbgriff() - Klick_Tolleranz
                    &&
                    Calculated_X_Value <= parameterEingabeLN.getAbgriff() + Klick_Tolleranz
                ) {
                    setCursor(new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR  ));
                } else {	// Wenn nicht, normalen Cursor einstellen
                    setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR   ));
                }
            }
        }

        /**
         * Methode wird aufgerufen, falls Maus mit gedrckter Maustaste bewegt wurde.
         *
         * @param evt 	MouseEvent
         */
        public void mouseDragged(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag == true) {
                // Mausposition holen
                int posX = evt.getX();
                // Orginalwert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);

                parameterEingabeLN.setAbgriff(skalierung.getSkalXWert(Calculated_X_Value));
            }
        }
    }
}
