
package source;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Die Klasse ParameterEingabe verwaltet die
 * Ein- und Ausgabewerte der Cosinusfunktion
 *
 * @author Sergej Khltau  14.06.2008
 */

public class ParameterEingabeCosinus {
    // Legt die Genauigkeit fest
    private double Genauigkeit = 100;

    // Formatierung der Anzeigen
    DecimalFormat df2 = new DecimalFormat( "0.00" );
    DecimalFormat df4 = new DecimalFormat( "0.0000" );

    // Referenzen auf die Zeichenbereiche
    private ZeichenbereichCosinus Zeichenbereich1 = null;
    private ZeichenbereichCosinus Zeichenbereich2 = null;

    // Bereich der in ParameterEingabe verwaltet wird
    private JPanel innerContent = null;

    // Gruppe von Elementen erstellen:
    // Beschriftung, Eingabefeld & Scrollbalken fr Amplitude
    // Beschriftung, Eingabefeld & Scrollbalken fr Kreisfrequenz
    // Beschriftung, Eingabefeld & Scrollbalken fr Phasenwinkel
    // Beschriftung, Eingabefeld & Scrollbalken fr Abgriff

    // Labels
    private JLabel LB_Amplitude_Wert = null;
    private JLabel LB_Kreisfrequenz_Wert = null;
    private JLabel LB_Phasenwinkel_Wert = null;
    private JLabel LB_Untergrenze = null;
    private JLabel LB_Obergrenze = null;
    private JLabel Logo = null;

    // Textfelder
    private JTextField TF_Amplitude_Wert= null;
    private JTextField TF_Kreisfrequenz_Wert = null;
    private JTextField TF_Phasenwinkel_Wert = null;
    private JTextField TF_Untergrenze = null;
    private JTextField TF_Obergrenze = null;

    // Slider (Scrollbalken)
    private JSlider sl_Amplitude_Wert = null;
    private JSlider sl_Kreisfrequenz_Wert = null;
    private JSlider sl_Phasenwinkel_Wert = null;
    private JSlider sl_Untergrenze = null;
    private JSlider sl_Obergrenze = null;

    // Interne Variablen
    private double Amplitude_Wert = 1;
    private double Kreisfrequenz_Wert = 1;
    private double Phasenwinkel_Wert = 0;
    private double Untergrenze_Wert = -1;
    private double Obergrenze_Wert = 1;

    /**
     * Konstruktor erzeugt die Eingabefelder
     */
    public ParameterEingabeCosinus () {
        String Beschriftung;
        // wir legen alle Elemente auf ein Panel
        innerContent = new JPanel();
        // wir verwenden keinen Layoutmanager
        innerContent.setLayout(null);

        // Erzeugung der Text-Eingabefelder, mit dem jeweiligen Wert und
        // lassen maximal 10 Zeichen zu
        Beschriftung = df2.format(Amplitude_Wert);
        TF_Amplitude_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Kreisfrequenz_Wert);
        TF_Kreisfrequenz_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Phasenwinkel_Wert);
        TF_Phasenwinkel_Wert = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Untergrenze_Wert);
        TF_Untergrenze = new JTextField(Beschriftung,10);

        Beschriftung = df2.format(Obergrenze_Wert);
        TF_Obergrenze = new JTextField(Beschriftung,10);

        // wir erzeugen die Berschriftungslabels
        LB_Amplitude_Wert = new JLabel("Amplitude A:");
        LB_Kreisfrequenz_Wert = new JLabel("Kreisfrequenz \u03c9:");
        LB_Phasenwinkel_Wert = new JLabel("Phasenwinkel \u03c6:");
        LB_Untergrenze = new JLabel("Untergrenze a:");
        LB_Obergrenze = new JLabel("Obergrenze b:");
        Logo = new JLabel("");

        // Erzeugung der Schieberegler --> JSlider (min, max, value)
        sl_Amplitude_Wert = new JSlider(-250, 250,(int)(Genauigkeit * Amplitude_Wert));
        sl_Kreisfrequenz_Wert = new JSlider(-250, 250, (int) (Genauigkeit * Kreisfrequenz_Wert));
        sl_Phasenwinkel_Wert = new JSlider(-300, 300, (int) (Genauigkeit * Phasenwinkel_Wert));
        sl_Untergrenze = new JSlider(-398, 398, (int) (Genauigkeit * Untergrenze_Wert));
        sl_Obergrenze = new JSlider(-398, 398,(int)(Genauigkeit* Obergrenze_Wert));

        // Positionierung der Elemente
        int left = 5;
        int width = 120;
        int height = 21;
        int abstand = 26;
        int top = 7 - abstand;

        // Elementbegrenzungen setzen
        LB_Amplitude_Wert.setBounds(left,top+=abstand,width,height);
        TF_Amplitude_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Amplitude_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Kreisfrequenz_Wert.setBounds(left,top+=abstand,width,height);
        TF_Kreisfrequenz_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Kreisfrequenz_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Phasenwinkel_Wert.setBounds(left,top+=abstand,width,height);
        TF_Phasenwinkel_Wert.setBounds(left,top+=abstand/1.5,width,height);
        sl_Phasenwinkel_Wert.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Untergrenze.setBounds(left,top+=abstand,width,height);
        TF_Untergrenze.setBounds(left,top+=abstand/1.5,width,height);
        sl_Untergrenze.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.1;
        LB_Obergrenze.setBounds(left,top+=abstand,width,height);
        TF_Obergrenze.setBounds(left,top+=abstand/1.5,width,height);
        sl_Obergrenze.setBounds(left,top+=abstand,width,(int)(height));

        top+=abstand*0.4;
        if (Ableitung.getIconLogo() != null)
            Logo.setIcon(Ableitung.getIconLogo());
        Logo.setBounds(left,top+=abstand,120,108);

        /**
         * Reaktion auf die nderung der Eingabefelder
         */
        TF_Amplitude_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Amplitude_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Amplitude_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Amplitude_Wert.hasFocus()) return;
                String Beschriftung1;
                Amplitude_Wert = sl_Amplitude_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Amplitude_Wert);
                TF_Amplitude_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Kreisfrequenz_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Kreisfrequenz_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Kreisfrequenz_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Kreisfrequenz_Wert.hasFocus()) return;
                String Beschriftung1;
                Kreisfrequenz_Wert = sl_Kreisfrequenz_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Kreisfrequenz_Wert);
                TF_Kreisfrequenz_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Phasenwinkel_Wert.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Phasenwinkel_Wert.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Phasenwinkel_Wert.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Phasenwinkel_Wert.hasFocus()) return;
                String Beschriftung1;
                Phasenwinkel_Wert = sl_Phasenwinkel_Wert.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Phasenwinkel_Wert);
                TF_Phasenwinkel_Wert.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Untergrenze.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Untergrenze.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Untergrenze.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Untergrenze.hasFocus()) return;
                String Beschriftung1;
                Untergrenze_Wert = sl_Untergrenze.getValue()/Genauigkeit;
                Beschriftung1 = df2.format(Untergrenze_Wert);
                TF_Untergrenze.setText(Beschriftung1);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        TF_Obergrenze.addActionListener(
        new ActionListener() {
            public void actionPerformed( ActionEvent e) {
                if (!TF_Obergrenze.hasFocus()) return;
                eingabewerteEinlesen();
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        sl_Obergrenze.addChangeListener(
        new ChangeListener() {
            public void stateChanged(ChangeEvent  e) {
                if (!sl_Obergrenze.hasFocus()) return;
                String Beschriftung2;
                Obergrenze_Wert = sl_Obergrenze.getValue()/Genauigkeit;
                Beschriftung2 = df2.format(Obergrenze_Wert);
                TF_Obergrenze.setText(Beschriftung2);
                if (Zeichenbereich1!=null) Zeichenbereich1.Dorepaint();
                if (Zeichenbereich2!=null) Zeichenbereich2.Dorepaint();
            }
        }
        );

        // Gre des Eingabebereichs zulassen
        innerContent.setPreferredSize(new Dimension(left*2+width,20));

        // Widgets auf dem Panel plazieren
        innerContent.add(LB_Amplitude_Wert);
        innerContent.add(TF_Amplitude_Wert);
        innerContent.add(sl_Amplitude_Wert);

        innerContent.add(LB_Kreisfrequenz_Wert);
        innerContent.add(TF_Kreisfrequenz_Wert);
        innerContent.add(sl_Kreisfrequenz_Wert);

        innerContent.add(LB_Phasenwinkel_Wert);
        innerContent.add(TF_Phasenwinkel_Wert);
        innerContent.add(sl_Phasenwinkel_Wert);

        innerContent.add(LB_Untergrenze);
        innerContent.add(TF_Untergrenze);
        innerContent.add(sl_Untergrenze);

        innerContent.add(LB_Obergrenze);
        innerContent.add(TF_Obergrenze);
        innerContent.add(sl_Obergrenze);

        innerContent.add(Logo);
    }

    /**
     * Methode delete dient dazu eine Eingabe aus dem Speicher aufzulsen
     *
     * @param Fenster	Fenster auf dem die Elemente plaziert sind
     */
    public void delete(JFrame Fenster) {
        // Eingabefelder freigeben
        innerContent.remove(LB_Amplitude_Wert);
        innerContent.remove(TF_Amplitude_Wert);
        innerContent.remove(sl_Amplitude_Wert);

        innerContent.remove(LB_Kreisfrequenz_Wert);
        innerContent.remove(TF_Kreisfrequenz_Wert);
        innerContent.remove(sl_Kreisfrequenz_Wert);

        innerContent.remove(LB_Phasenwinkel_Wert);
        innerContent.remove(TF_Phasenwinkel_Wert);
        innerContent.remove(sl_Phasenwinkel_Wert);

        innerContent.remove(LB_Untergrenze);
        innerContent.remove(TF_Untergrenze);
        innerContent.remove(sl_Untergrenze);

        innerContent.remove(LB_Obergrenze);
        innerContent.remove(TF_Obergrenze);
        innerContent.remove(sl_Obergrenze);

        innerContent.remove(Logo);

        // Referenzen lschen
        LB_Amplitude_Wert = null;
        LB_Kreisfrequenz_Wert = null;
        LB_Phasenwinkel_Wert = null;
        LB_Untergrenze = null;
        LB_Obergrenze = null;
        innerContent.remove(Logo);

        TF_Amplitude_Wert = null;
        TF_Kreisfrequenz_Wert = null;
        TF_Phasenwinkel_Wert = null;
        TF_Untergrenze = null;
        TF_Obergrenze = null;

        sl_Amplitude_Wert = null;
        sl_Kreisfrequenz_Wert = null;
        sl_Phasenwinkel_Wert = null;
        sl_Untergrenze = null;
        sl_Obergrenze = null;

        // selbst auflsen
        Fenster.getContentPane().remove(innerContent);
        innerContent = null;
        Zeichenbereich1 = null;
        Zeichenbereich2 = null;
    }

    /**
     * setzt den Amplituden-Wert von aussen
     *
     * @param wert	Neuer Amplituden-Wert
     */
    public void setAmplitude_Wert(double wert) {
        Amplitude_Wert = wert;
        String Beschriftung = df2.format(Amplitude_Wert);
        TF_Amplitude_Wert.setText(Beschriftung);
        sl_Amplitude_Wert.setValue( (int) (Amplitude_Wert*Genauigkeit) );
    }

    /**
     * setzt den Kreisfrequenz-Wert von aussen
     *
     * @param wert	Neuer Kreisfrequenz-Wert
     */
    public void setKreisfrequenz_Wert(double wert) {
        Kreisfrequenz_Wert = wert;
        String Beschriftung = df2.format(Kreisfrequenz_Wert);
        TF_Kreisfrequenz_Wert.setText(Beschriftung);
        sl_Kreisfrequenz_Wert.setValue( (int) (Kreisfrequenz_Wert * Genauigkeit) );
    }

    /**
     * setzt den Untergrenze-Wert
     *
     * @param x	Neue Untergrenze
     */
    public void setUntergrenze(double x) {
        // Abfangen der unzulssigen Werte
        Untergrenze_Wert = x;
        if (Untergrenze_Wert > -3.98 && Untergrenze_Wert < 3.98) {
            sl_Untergrenze.setValue( (int)(Genauigkeit*Untergrenze_Wert) );
            String Beschriftung1;
            Beschriftung1 = df2.format(Untergrenze_Wert);
            TF_Untergrenze.setText(Beschriftung1);
        } else if (Untergrenze_Wert > 3.98) {
            sl_Untergrenze.setValue( (int)(Genauigkeit*3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(3.98);
            TF_Untergrenze.setText(Beschriftung1);
            Untergrenze_Wert = 3.98;
        } else {
            sl_Untergrenze.setValue( (int)(Genauigkeit*-3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(-3.98);
            TF_Untergrenze.setText(Beschriftung1);
            Untergrenze_Wert = -3.98;
        }
        if (Zeichenbereich1 != null) Zeichenbereich1.Dorepaint();
        if (Zeichenbereich2 != null) Zeichenbereich2.Dorepaint();
    }

    /**
     * liefert den Wert der Amplitude
     *
     * @return derzeitige Amplitude
     */
    public double getAmplitude_Wert() {
        return Amplitude_Wert ;
    }

    /**
     * liefert den Wert der Kreisfrequenz
     *
     * @return derzeitige Kreisfrequenz
     */
    public double getKreisfrequenz_Wert() {
        return Kreisfrequenz_Wert;
    }
    /**
     * liefert den Wert des Phasenwinkels
     *
     * @return derzeitiger Phasenwinkel
     */
    public double getPhasenwinkel_Wert() {
        return Phasenwinkel_Wert;
    }

    /**
     * liefert den Wert des Untergrenze_Wertes
     *
     * @return derzeitiger Untergrenze_Wert
     */
    public double getUntergrenze() {
        return Untergrenze_Wert;
    }

    /**
     * setzt die Zeichenbereiche
     *
     * @param z	Zeichenbereich
     */
    public void setZeichenbereich(ZeichenbereichCosinus z) {
        if (Zeichenbereich1==null)
            Zeichenbereich1=z;
        else
            Zeichenbereich2=z;
    }

    /**
     * liefert den Obergrenze-Wert
     *
     * @return derzeitiger Abgriff
     */
    public double getObergrenze() {
        return	Obergrenze_Wert;
    }

    /**
     * setzt den Obergrenze-Wert
     *
     * @param x	Neue Obergrenze
     */
    public void setObergrenze(double x) {
        // Abfangen der unzulssigen Werte
        Obergrenze_Wert = x;
        if (Obergrenze_Wert > -3.98 && Obergrenze_Wert < 3.98) {
            sl_Obergrenze.setValue( (int)(Genauigkeit*Obergrenze_Wert) );
            String Beschriftung1;
            Beschriftung1 = df2.format(Obergrenze_Wert);
            TF_Obergrenze.setText(Beschriftung1);
        } else if (Obergrenze_Wert > 3.98) {
            sl_Obergrenze.setValue( (int)(Genauigkeit*3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(3.98);
            TF_Obergrenze.setText(Beschriftung1);
            Obergrenze_Wert = 3.98;
        } else {
            sl_Obergrenze.setValue( (int)(Genauigkeit*-3.98) );
            String Beschriftung1;
            Beschriftung1 = df2.format(-3.98);
            TF_Obergrenze.setText(Beschriftung1);
            Obergrenze_Wert = -3.98;
        }
        if (Zeichenbereich1 != null) Zeichenbereich1.Dorepaint();
        if (Zeichenbereich2 != null) Zeichenbereich2.Dorepaint();
    }

    /**
     * fgt die Eingabeleiste auf einem Fenster ein
     *
     * @param Fenster	Fenster auf dem die Leiste eingefgt werden soll
     */
    public void addToFrame(JFrame Fenster) {
        Fenster.getContentPane().add(innerContent, BorderLayout.WEST);
    }

    /**
     * bernimmt alle Werte bei der Texteingabe
     */
    private void eingabewerteEinlesen() {
        // Amplitude
        {
            String Hilf = TF_Amplitude_Wert.getText();
            Amplitude_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Amplitude_Wert < -2.5 ) {
                Amplitude_Wert = -2.5;
                TF_Amplitude_Wert.setText("-2,50");
            }
            if (Amplitude_Wert > 2.5 ) {
                Amplitude_Wert = 2.50;
                TF_Amplitude_Wert.setText("2,50");
            }
            sl_Amplitude_Wert.setValue( (int)(Genauigkeit*Amplitude_Wert) );
        }

        // Kreisfrequenz
        {
            String Hilf = TF_Kreisfrequenz_Wert.getText();
            Kreisfrequenz_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Kreisfrequenz_Wert <= -2.5 ) {
                Kreisfrequenz_Wert = -2.5;
                TF_Kreisfrequenz_Wert.setText("-2,50");
            }
            if (Kreisfrequenz_Wert > 2.5 ) {
                Kreisfrequenz_Wert = 2.50;
                TF_Kreisfrequenz_Wert.setText("2,50");
            }
            sl_Kreisfrequenz_Wert.setValue( (int) (Genauigkeit*Kreisfrequenz_Wert) );
        }

        // Phasenwinkel
        {
            String Hilf = TF_Phasenwinkel_Wert.getText();
            Phasenwinkel_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Phasenwinkel_Wert < -3 ) {
                Phasenwinkel_Wert = -3;
                TF_Phasenwinkel_Wert.setText("-3,00");
            }
            if (Phasenwinkel_Wert > 3 ) {
                Phasenwinkel_Wert = 3;
                TF_Phasenwinkel_Wert.setText("3,00");
            }
            sl_Phasenwinkel_Wert.setValue( (int) (Genauigkeit * Phasenwinkel_Wert) );
        }

        // Obergrenze
        {
            String Hilf = TF_Obergrenze.getText();
            Obergrenze_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Obergrenze_Wert < -3.98 ) {
                Obergrenze_Wert = -3.98;
                TF_Obergrenze.setText("-3,98");
            }
            if (Obergrenze_Wert > 3.98 ) {
                Obergrenze_Wert = 3.98;
                TF_Obergrenze.setText("3,98");
            }
            sl_Obergrenze.setValue( (int) (Genauigkeit * Obergrenze_Wert) );
        }

        // Untergrenze
        {
            String Hilf = TF_Untergrenze.getText();
            Untergrenze_Wert = Double.parseDouble(Hilf.replace(',','.'));
            // ungltige Werte nicht zulassen
            if (Untergrenze_Wert < -3.98 ) {
                Untergrenze_Wert = -3.98;
                TF_Untergrenze.setText("-3,98");
            }
            if (Untergrenze_Wert > 3.98 ) {
                Untergrenze_Wert = 3.98;
                TF_Untergrenze.setText("3,98");
            }
            sl_Untergrenze.setValue( (int) (Genauigkeit * Untergrenze_Wert) );
        }
    }
}
