
package source;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.SwingConstants;

import javax.imageio.ImageIO;
import java.io.IOException;

/**
 * StartApplet stellt den Einstiegspunkt
 * fr ein Java Applet zur Verfgung
 *
 * @author Sergej Khltau 29.04.2008
 */

public class StartApplet extends JApplet {
    /**
     * automatisch generierte UID
     */
    private static final long serialVersionUID = 1L;

    /**
     * Frame in welchem das Applet ausgefhrt wird
     */
    private Ableitung ableitung;

    /**
     * Button "Start", um den Frame in welchem das Applet abluft
     * zu erzeugen und das "Haupt-" Applet zu starten.
     */
    private JButton buttonStart;

    /**
     * Zeiger auf Logos
     */
    private static ImageIcon iconLogo = null;
    private static ImageIcon symbolPressedLogo[];
    private static ImageIcon symbolUnPressedLogo[];

    /**
     * Eventhandler fr GUI-Elemente
     */
    private Adapter adapter = new Adapter();

    /**
     * Hintergrundfarbe, damit sich das Applet in die Hintergrundfarbe der
     * Homepage "einschmiegt".
     */
    private Color bgcolor;

    /**
     * Referenz auf Applet
     */
    private JApplet applet = null;;

    /**
     * Funktion wird aufgerufen, wenn der Browser das Applet initialisiert
     */
    public void init() {
        // Referenz fr weitere Aktionen
        applet = this;

        /**
         * Hintergrundfarbe, selbe Farbe wie HTML zuweisen
         */
        bgcolor = new Color(235,238,247);

        /**
         * "Null"-Layout verwenden
         */
        getContentPane().setLayout(null);

        /**
         * Hintergrundfarbe setzen
         */
        getContentPane().setBackground(bgcolor);

		try{
			iconLogo = new ImageIcon(ImageIO.read(Ableitung.class.getResourceAsStream("/images/Logo.GIF")));
		}catch(IOException IOE){
		}

        /**
         * Start Button erzeugen und formatieren
         */
        buttonStart = new JButton("Applet starten ", iconLogo);
        buttonStart.setBackground(bgcolor);
        buttonStart.setFont(new Font("Sans Serif", Font.BOLD, 14));
        buttonStart.setHorizontalAlignment(SwingConstants.LEFT);
        buttonStart.setIconTextGap(20);
        buttonStart.addActionListener(adapter);
        buttonStart.setBounds(10,10,280,100);
        buttonStart.addMouseListener(new MouseAdapter() {
            public void mouseEntered(MouseEvent evt) {
                setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
            }

            public void mouseExited(MouseEvent evt) {
                setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
            }
        });
        getContentPane().add(buttonStart);
    }

    /**
     * liefert Referenz auf Logo
     *
     * @return ImageIcon auf das Logo
     */
    public static ImageIcon getIconLogo() {
        return iconLogo;
    }

    /**
     * liefert Referenz auf gedrckte Symbole
     *
     * @return ImageIcon Array auf die gedrckten Symbole
     */
    public static ImageIcon [] getsymbolPressedLogo() {
        return symbolPressedLogo;
    }

    /**
     * liefert Referenz auf nicht gedrckten Symbole
     *
     * @return ImageIcon Array auf die nicht gedrckten Symbole
     */
    public static ImageIcon [] getsymbolUnPressedLogo() {
        return symbolUnPressedLogo;
    }

    /**
     * Nachdem die init()-Methode durchlaufen wurde,
     * wird diese Methode ausgefhrt.
     */
    public void start() {
    }

    /**
     * Diese Methode wird ausgefhrt, falls das
     * Applet beendet wird.
     */
    public void stop() {
    }

    /**
     * Eventhandler fr <br>
     * <ul>
     * 	<li>Fenster schlieen</li>
     * 	<li>Start-Button, zum Starten des Grundfunktionen-Applets</li></br>
     * </ul>
     */
    class Adapter extends WindowAdapter implements ActionListener {
        // * Methode wird aufgerufen, falls Fenster in welchem Applet
        // * abluft geschlossen wird.

        public void windowClosing(WindowEvent evt) {
        }

        // * Methode wird aufgerufen, falls mit der Maus auf Button
        //* "Start Applet" geklickt wurde.

        public void actionPerformed(ActionEvent evt) {
            //Start-Button wurde geklickt.
            if (evt.getSource() == buttonStart) {
                //Es wird ein neues Fenster geffnet,
                //und alle Icons bergeben, welche geladen werden sollen.
                ableitung = new Ableitung();
                Ableitung.main1(applet);
            }
        }
    }
}
