
package source;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.text.DecimalFormat;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import javax.imageio.ImageIO;
import java.io.IOException;

/**
 * Klasse Zeichenbereich
 * Erzeugt den eigentlichen Zeichenbereich auf dem die Grafik dargestellt wird
 * @author Sergej Khltau  28.06.2008
 */
public class ZeichenbereichPolynom extends JPanel {
    /**
     * automatisch generierte UID
     */
    private static final long serialVersionUID = 1L;
    // Pfeilgren
    private int Pfeil_laenge = 12;
    private int Pfeil_breite = 3;

    // Referenz auf die Eingabeparameter
    private ParameterEingabePolynom parameterEingabeP = null;

    // fr das Verschieben der Vertrauensbereiche
    private double Klick_Tolleranz = 0.05;
    // Unterscheidung welche Grenze verschoben wird
    private int Is_Bond;
    // erlaubt Verschieben
    private boolean CanDrag = false;
    // bei Verschiebung den Abstand zur letzten Aktualisierung
    private double Drag_Last = 0;
    // bei Verschiebung alte Untergrenze
    private double Drag_Old_Untergrenze = 0;
    // bei Verschiebung alte Obergrenze
    private double Drag_Old_Obergrenze = 0;
    // Referenz auf die Skalierungsklasse
    private SkalierungPolynom skalierung = null;

    // bestimmt ob dieser Zeichenbereich normiert dargestellt werden soll
    private boolean IsNormaled = false;

    private int GradWahl = 0;

    double value;

    private Image Formel2;
    private Image Formel2A;
    private Image Formel3;
    private Image Formel3A;
    private Image Formel4;
    private Image Formel4A;

    /**
     * Konstruktor
     * erzeugt neuen Zeichenbereich
     *
     * @param w 	Referenz auf die Eingaben
     */
    public ZeichenbereichPolynom(ParameterEingabePolynom w, boolean isNormaled) {
        // Parameter bernehmen und sichern
		
		try{
			Formel2  = ImageIO.read(Ableitung.class.getResourceAsStream("/images/2Poly.PNG"));
			Formel2A = ImageIO.read(Ableitung.class.getResourceAsStream("/images/2PolyA.PNG"));
			Formel3  = ImageIO.read(Ableitung.class.getResourceAsStream("/images/3Poly.PNG"));
			Formel3A = ImageIO.read(Ableitung.class.getResourceAsStream("/images/3PolyA.PNG"));
			Formel4  = ImageIO.read(Ableitung.class.getResourceAsStream("/images/4Poly.PNG"));
			Formel4A = ImageIO.read(Ableitung.class.getResourceAsStream("/images/4PolyA.PNG"));
			
		}catch(IOException ioe){
		}
		
        parameterEingabeP = w;
        parameterEingabeP.setZeichenbereich(this);
        IsNormaled = isNormaled;
        GradWahl = parameterEingabeP.getGrad();

        // Mausaktionen verwenden
        addMouseListener(new MyMouseListener());
        addMouseMotionListener(new MyMouseMotionListener());

        // neue Skalierungsberechnung erzeugen
        skalierung = new SkalierungPolynom(parameterEingabeP, IsNormaled, GradWahl);
    }

    /**
     * lscht den Zeichenbereich
     * und gibt interne Referenzen frei
     */
    public void delete() {
        parameterEingabeP = null;
        skalierung = null;
    }

    /**
     * veranlasst ein neuzeichnen der Visualisierung
     */
    public void Dorepaint() {
        this.repaint();
    }

    /**
     * erzwingt eine Neuberechnung der Skalierung
     */
    public void DoRecalc() {
        skalierung.ForceRechnen(getWidth(), getHeight());
        this.repaint();
    }

    /**
     * Neuzeichnen der Grafik erforderlich
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    protected void paintComponent( Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // bergeordnetes Zeichen aktivieren
        super.paintComponent(g);
        // Skalierung aktualisieren
        skalierung.Rechnen(getWidth(), getHeight());
        // einzelne Teile neu zeichnen
        PrintBackground(g);
        PrintIntegral(g);
        PrintAchsen(g);
        PrintPolynomKurve(g);
        PrintUntergrenze(g);
        PrintObergrenze(g);
        PrintWerte (g);

        // Fallunterscheidung welchen Grad das Polynom hat
        switch (parameterEingabeP.getGrad()) {
        case 0: // Grad 1
            // bei abgeleiteter Funktion entsprechende Einblendung
            if (IsNormaled)
                g2d.drawImage(Formel2A, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); // 1592/13, 22,
            else
                g2d.drawImage(Formel2, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //  1906/13, 22,
            break;
        case 1:  // Grad 2 (Parabel)
//			 bei abgeleiteter Funktion entsprechende Einblendung
            if (IsNormaled)
                g2d.drawImage(Formel3A, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //  13343/73, 22,
            else
                g2d.drawImage(Formel3, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //  13970/73, 22,
            break;
        case 2:  // Grad 3
//			 bei abgeleiteter Funktion entsprechende Einblendung
            if (IsNormaled)
                g2d.drawImage(Formel4A, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //  17743/73, 22,
            else
                g2d.drawImage(Formel4, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //  17270/73, 22,
            break;
        }
    }


    /**
     * Zeichnet Obergenze
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintObergrenze (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // Abgriff blau darstellen
        g2d.setColor(Color.GRAY);
        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf;
        int Punkt_x, Punkt_y;

        // Fallunterscheidung welchen Grad das Polynom hat
        switch (parameterEingabeP.getGrad()) {
        case 0:  // Grad 1
            // Obergrenze holen
            lauf=parameterEingabeP.getObergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly1WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8) {
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                } else {
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                }
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("b", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;

        case 1:  // Grad 2
            // Obergrenze holen
            lauf=parameterEingabeP.getObergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly2WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8)
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                else
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("b", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;
        case 2:  // Grad 3
            // Obergrenze holen
            lauf=parameterEingabeP.getObergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly3WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8)
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                else
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("b", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;
        }
        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }


    /**
     * Zeichnet Obergenze
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintUntergrenze (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // Abgriff blau darstellen
        g2d.setColor(Color.GRAY);
        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf;
        int Punkt_x, Punkt_y;

        // Fallunterscheidung welchen Grad das Polynom hat
        switch (parameterEingabeP.getGrad()) {
        case 0:  // Grad 1
            // Untergrenze holen
            lauf=parameterEingabeP.getUntergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly1WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8) {
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                } else {
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                }
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("a", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;

        case 1:  // Grad 2
            // Untergrenze holen
            lauf=parameterEingabeP.getUntergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly2WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8)
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                else
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("a", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;
        case 2:  // Grad 3
            // Untergrenze holen
            lauf=parameterEingabeP.getUntergrenze();
            // Umrechnen in Grafikkoordinaten
            Punkt_x = (int) skalierung.getPolyWertX(lauf);
            Punkt_y = (int) skalierung.getPoly3WertY(lauf);

            if (!IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8)
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)(skalierung.getPosXAchse()+ skalierung.getHeight()/2));
                else
                    g.drawLine(Punkt_x,(int)( skalierung.getHeight()), Punkt_x, skalierung.getRand());
                g2d.setStroke(OldStoke);
            }
            if (IsNormaled) {
                g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                g.drawLine(Punkt_x, (int)skalierung.getPosXAchse()- skalierung.getHeight()-(int)skalierung.getOffset_Y(), Punkt_x, (int) skalierung.getPosXAchse()+ skalierung.getHeight()*14/15-(int)skalierung.getOffset_Y());
                g2d.setStroke(OldStoke);
                g.drawString("a", Punkt_x-2, (int) skalierung.getPosXAchse()+ skalierung.getHeight()-(int)skalierung.getOffset_Y());

            }
            break;
        }
        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }

    private void PrintWerte(Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Tangente grn darstellen
        g2d.setColor(Color.GREEN);
        Font bold = new Font ("Arial",Font.BOLD,12);

        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf = parameterEingabeP.getObergrenze();
        double lauf2 = parameterEingabeP.getUntergrenze();

        // Fallunterscheidung welchen Grad das Polynom hat
        switch (parameterEingabeP.getGrad()) {
        case 0:  // Grad 1
            g2d.setColor(Color.GRAY);
            g.setFont(bold);
            if ((skalierung.getHeight()-10)>skalierung.getPoly1WertY(lauf) && (skalierung.getPoly1WertY(lauf)-20)>skalierung.getRand()) {
                if (IsNormaled)	// nur im zweiten Zeichenbereich zeichnen
                    g.drawString("I(b) = "+Math.round((skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2)))*100)/100., (int)skalierung.getPolyWertX(lauf)+6,(int) skalierung.getPoly1WertY(lauf)-5);
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawString(""+Math.round(skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))*100)/100., (int)skalierung.getPolyWertX(lauf)-10,(int) skalierung.getPoly1WertY(lauf)-5);

            }
            // alten Stift wieder einstellen
            g2d.setStroke(OldStoke);
            //g2d.setColor(Color.GREEN);
            break;

        case 1:  // Grad 2
            g2d.setColor(Color.GRAY);
            g.setFont(bold);
            if ((skalierung.getHeight()-10)>skalierung.getPoly2WertY(lauf) && (skalierung.getPoly2WertY(lauf)-20)>skalierung.getRand()) {
                if (IsNormaled)	// nur im zweiten Zeichenbereich zeichnen
                    g.drawString("I(b) = "+Math.round((skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2)))*100)/100., (int)skalierung.getPolyWertX(lauf)+6,(int) skalierung.getPoly2WertY(lauf)-5);
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawString(""+Math.round(skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))*100)/100., (int)skalierung.getPolyWertX(lauf)-10,(int) skalierung.getPoly2WertY(lauf)-5);

            }
            // alten Stift wieder einstellen
            g2d.setStroke(OldStoke);
            //g2d.setColor(Color.GREEN);
            break;

        case 2:  // Grad 3
            g2d.setColor(Color.GRAY);
            g.setFont(bold);
            if ((skalierung.getHeight()-10)>skalierung.getPoly3WertY(lauf) && (skalierung.getPoly3WertY(lauf)-20)>skalierung.getRand()) {
                if (IsNormaled)	// nur im zweiten Zeichenbereich zeichnen
                    g.drawString("I(b) = "+Math.round((skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2)))*100)/100., (int)skalierung.getPolyWertX(lauf)+6,(int) skalierung.getPoly3WertY(lauf)-5);
                if (!IsNormaled)	// nur im ersten Zeichenbereich zeichnen
                    g.drawString(""+Math.round(skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))*100)/100., (int)skalierung.getPolyWertX(lauf)-10,(int) skalierung.getPoly3WertY(lauf)-5);

            }
            // alten Stift wieder einstellen
            g2d.setStroke(OldStoke);
            //g2d.setColor(Color.GREEN);
            break;
        }
        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }


    /** Zeichnet das Integral
     * @ param g Graphics Objekt auf die Zeichenflche
     */
    private void PrintIntegral (Graphics g) {
        switch (parameterEingabeP.getGrad()) {
        case 0:  // Grad 1

            if (!IsNormaled) { //fr oberen Zeichenbereich
                for (double lauf=skalierung.getUntergrenze(); lauf < skalierung.getObergrenze(); lauf+=0.5d/skalierung.getPosXAchse()) {
                    // Koordinaten umrechnen
                    int Punkt_x = (int) skalierung.getPolyWertX(lauf);
                    int Punkt_y = (int) skalierung.getPoly1WertY(lauf);
                    // Referzenz fr 2D Grafik
                    Graphics2D g2d = (Graphics2D) g;
                    if ( Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        if ((int)skalierung.getPoly1WertY(lauf)>skalierung.getRand()/2) {
                            g2d.setColor(Color.white);

                            if(Punkt_y > skalierung.getHeight()+skalierung.getRand()- (int)skalierung.getPoly1WertY(lauf)) {
                                g2d.setColor(Color.GREEN);
                                if ( Punkt_y>( skalierung.getHeight()-skalierung.getRand())) {
                                    if ((int)skalierung.getPoly1WertY(lauf)>skalierung.getRand()/2) {
                                        g2d.setColor(g2d.getBackground());
                                        g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                                    }
                                }

                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            } else {
                                g2d.setColor(Color.BLUE);
                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            }
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)skalierung.getPosXAchse());
                        }
                    }
                }
            }
            break;
        case 1:  // Grad 2
            if (!IsNormaled) { //fr oberen Zeichenbereich
                for (double lauf=skalierung.getUntergrenze(); lauf < skalierung.getObergrenze(); lauf+=0.5d/skalierung.getPosXAchse()) {
                    // Koordinaten umrechnen
                    int Punkt_x = (int) skalierung.getPolyWertX(lauf);
                    int Punkt_y = (int) skalierung.getPoly2WertY(lauf);
                    // Referzenz fr 2D Grafik
                    Graphics2D g2d = (Graphics2D) g;
                    if ( Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        if ((int)skalierung.getPoly2WertY(lauf)>skalierung.getRand()/2) {
                            g2d.setColor(Color.white);

                            if(Punkt_y > skalierung.getHeight()+skalierung.getRand()- (int)skalierung.getPoly2WertY(lauf)) {
                                g2d.setColor(Color.GREEN);
                                if ( Punkt_y>( skalierung.getHeight()-skalierung.getRand())) {
                                    if ((int)skalierung.getPoly2WertY(lauf)>skalierung.getRand()/2) {
                                        g2d.setColor(g2d.getBackground());
                                        g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                                    }
                                }

                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            } else {
                                g2d.setColor(Color.BLUE);
                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            }
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)skalierung.getPosXAchse());
                        }
                    }
                }
            }
            break;
        case 2:  // Grad 3
            if (!IsNormaled) { //fr oberen Zeichenbereich
                for (double lauf=skalierung.getUntergrenze(); lauf < skalierung.getObergrenze(); lauf+=0.5d/skalierung.getPosXAchse()) {
                    // Koordinaten umrechnen
                    int Punkt_x = (int) skalierung.getPolyWertX(lauf);
                    int Punkt_y = (int) skalierung.getPoly3WertY(lauf);
                    // Referzenz fr 2D Grafik
                    Graphics2D g2d = (Graphics2D) g;
                    if ( Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        if ((int)skalierung.getPoly3WertY(lauf)>skalierung.getRand()/2) {
                            g2d.setColor(Color.white);

                            if(Punkt_y > skalierung.getHeight()+skalierung.getRand()- (int)skalierung.getPoly3WertY(lauf)) {
                                g2d.setColor(Color.GREEN);
                                if ( Punkt_y>( skalierung.getHeight()-skalierung.getRand())) {
                                    if ((int)skalierung.getPoly3WertY(lauf)>skalierung.getRand()/2) {
                                        g2d.setColor(g2d.getBackground());
                                        g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                                    }
                                }

                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            } else {
                                g2d.setColor(Color.BLUE);
                                // Flche mit berechneten Punkten vom Integral bis zur X-Achse zeichnen

                            }
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, (int)skalierung.getPosXAchse());
                        }
                    }
                }
            }
            break;
        }
    }



    /**
     * Zeichnet den eigentlichen Verlauf der Kurve
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintPolynomKurve (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
            int Punkt_x = (int) skalierung.getPolyWertX(lauf);
            double lauf2= skalierung.getUntergrenze();
            double hilf;
            int Punkt_y;
            switch (parameterEingabeP.getGrad()) {
            case 0:  // Grad 1
                hilf = skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2));
                if (!IsNormaled) {	// Mit Rot Zeichnen
                    Punkt_y = (int) skalierung.getPoly1WertY(lauf);
                    g2d.setColor(g2d.getBackground());
                    g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {

                        g2d.setColor(Color.RED);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                    }
                } else {
                    Punkt_y = (int) skalierung.getPoly1WertY(lauf);
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        g2d.setColor(Color.GREEN);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    if((Math.round((skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2)))*100)/100.) >4|| (Math.round((skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2)))*100)/100.) <-3.2) {
                        g2d.setColor(g2d.getBackground());
                        g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.2660551111111)*hilf);
                }
                g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);


                break;

            case 1:  // Grad 1
                hilf = skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2));
                if (!IsNormaled) {	// Mit Rot Zeichnen
                    Punkt_y = (int) skalierung.getPoly2WertY(lauf);
                    g2d.setColor(g2d.getBackground());
                    g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {

                        g2d.setColor(Color.RED);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                    }
                } else {
                    Punkt_y = (int) skalierung.getPoly2WertY(lauf);
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        g2d.setColor(Color.GREEN);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    if((Math.round((skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2)))*100)/100.) >4|| (Math.round((skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2)))*100)/100.) <-3.2) {
                        g2d.setColor(g2d.getBackground());
                        g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.2660551111111)*hilf);
                }
                g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                break;

            case 2:  // Grad 1
                hilf = skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2));
                if (!IsNormaled) {	// Mit Rot Zeichnen
                    Punkt_y = (int) skalierung.getPoly3WertY(lauf);
                    g2d.setColor(g2d.getBackground());
                    g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {

                        g2d.setColor(Color.RED);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

                    }
                } else {
                    Punkt_y = (int) skalierung.getPoly3WertY(lauf);
                    if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand())) {
                        g2d.setColor(Color.GREEN);
                        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    if((Math.round((skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2)))*100)/100.) >4|| (Math.round((skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2)))*100)/100.) <-3.2) {
                        g2d.setColor(g2d.getBackground());
                        g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                    }
                    skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.2660551111111)*hilf);
                }
                g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                break;
            }


        }
    }






















    /*if (!IsNormaled)	// Mit Rot Zeichnen
    {
    g2d.setColor(Color.RED);
    g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    }
    else	// Mit Grn Zeichnen
    {
    	g2d.setColor(Color.GREEN);
    	g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    }

    // Fallunterscheidung welchen Grad das Polynom hat
    switch (parameterEingabeP.getGrad())
    {
    case 0:  // Grad 1
    	// Sichtbaren Bereich durchlaufen
    	for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X())
    		{
    		// Koordinaten umrechnen
    		int Punkt_x = (int) skalierung.getPolyWertX(lauf);
    		int Punkt_y = (int) skalierung.getPoly1WertY(lauf);
    		double lauf2= skalierung.getUntergrenze();
    		double hilf;
    		hilf = skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2));

    		if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    		{
    			if (!IsNormaled)	// Mit Rot Zeichnen
    			{
    				g2d.setColor(Color.RED);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			//if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))		//man knnte auch noch "Punkt_y>0 &&" dazu nehmen

    			}

    			else	// Mit Grn Zeichnen
    			{

    			g2d.setColor(Color.GREEN);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.29)*hilf);

    			//if((Math.round((skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2)))*100)/100.) >5.0 || (Math.round((skalierung.getRealYWert(skalierung.getPoly1WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly1WertY(lauf2)))*100)/100.) <-6.2)
    			//{
    			//	g2d.setColor(g2d.getBackground());
    			//	g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			//}
    			}

    		}
    			// Sichtbaren Bereich durchlaufen
    		if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    			g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
    		}
    	break;
    case 1:  // Grad 2
    	for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X())
    	{
    		// Koordinaten umrechnen
    		int Punkt_x = (int) skalierung.getPolyWertX(lauf);
    		int Punkt_y = (int) skalierung.getPoly2WertY(lauf);
    		double lauf2= skalierung.getUntergrenze();
    		double hilf;
    		hilf = skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2));

    		if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    		{
    			if (!IsNormaled)	// Mit Rot Zeichnen
    			{
    				g2d.setColor(Color.RED);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			//if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))		//man knnte auch noch "Punkt_y>0 &&" dazu nehmen

    			}

    			else	// Mit Grn Zeichnen
    			{
    			g2d.setColor(Color.GREEN);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.29)*hilf);
    			if((Math.round((skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2)))*100)/100.) >4.0 || (Math.round((skalierung.getRealYWert(skalierung.getPoly2WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly2WertY(lauf2)))*100)/100.) <-3.2)
    			{
    				g2d.setColor(g2d.getBackground());
    				g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			}
    			}

    		}
    			// Sichtbaren Bereich durchlaufen
    		if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    			g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
    		}
    	break;
    case 2:  // Grad 3
    	// Sichtbaren Bereich durchlaufen
    	for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X())
    	{
    		// Koordinaten umrechnen
    		int Punkt_x = (int) skalierung.getPolyWertX(lauf);
    		int Punkt_y = (int) skalierung.getPoly3WertY(lauf);
    		double lauf2= skalierung.getUntergrenze();
    		double hilf;
    		hilf = skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2));

    		if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    		{
    			if (!IsNormaled)	// Mit Rot Zeichnen
    			{
    				g2d.setColor(Color.RED);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			//if (Punkt_y<( skalierung.getHeight()-skalierung.getRand()))		//man knnte auch noch "Punkt_y>0 &&" dazu nehmen

    			}

    			else	// Mit Grn Zeichnen
    			{
    			g2d.setColor(Color.GREEN);
    			g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			skalierung.setOffset_Y(((skalierung.getHeight()/2-skalierung.getRand())*0.29)*hilf);
    			if((Math.round((skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2)))*100)/100.) >4.0 || (Math.round((skalierung.getRealYWert(skalierung.getPoly3WertY(lauf))-skalierung.getRealYWert(skalierung.getPoly3WertY(lauf2)))*100)/100.) <-3.2)
    			{
    				g2d.setColor(g2d.getBackground());
    				g2d.setStroke(new BasicStroke(0,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
    			}
    			}

    		}
    			// Sichtbaren Bereich durchlaufen
    		if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
    			g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
    		}
    	break;
    }

    }/*

    /**
    * Zeichnet die Achsen
    *
    * @param g 	Graphics Objekt auf die Zeichenflche
    */
    private void PrintAchsen (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // X-Achse
        g2d.setColor(Color.BLACK);
        g.drawLine(2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()), skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()));

        // Y-Achse
        g.drawLine(skalierung.getPosYAchse(), 2*skalierung.getRand(), skalierung.getPosYAchse(), skalierung.getHeight()-2*skalierung.getRand());

        // Pfeile an die Achsen
        for (int i=Pfeil_laenge; i>=0; i--) {
            // Oben
            g.drawLine(skalierung.getPosYAchse() - Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);
            g.drawLine(skalierung.getPosYAchse() + Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);

            //Rechts
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()-Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()+Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
        }

        // Beschriftungen
        Font Old_font = g.getFont();
        Font font = new Font ("Arial", Font.PLAIN, 13);
        g.setFont(font);
        // X - Achse
        if (IsNormaled)
            g.drawString("x", (int)skalierung.getWidth()-2*skalierung.getRand()+5, (int)(1*skalierung.getPosXAchse()+13));
        else
            g.drawString("t", skalierung.getWidth()-2*skalierung.getRand()+5, (int)(1*skalierung.getPosXAchse()+13));

        // Y - Achse
        if (IsNormaled)
            g.drawString("I(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        //g.drawString("\u03C6(u)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        else
            g.drawString("f(t)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);
        //g.drawString("\u03C6(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);

        g.setFont(Old_font);

        // Achsenbeschriftung
        // X - Achsen Beschriftung von 0 .. oo
        for (double lauf=skalierung.getBeschriftungAbstandX()/2; lauf <= skalierung.getTextBis()  ; lauf+=skalierung.getBeschriftungAbstandX()/2) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );

            if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
            if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
            if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
            int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height ) ;

            // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

            if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2*skalierung.getRand()) {
                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, (int)(1.0*Punkt_y_Text));

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
            }
        }

        // X - Achsen Beschriftung von -oo .. 0
        for (double lauf=-skalierung.getBeschriftungAbstandX()/2; lauf > skalierung.getTextVon()  ; lauf-=skalierung.getBeschriftungAbstandX()/2) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );

            if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
            if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
            if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
            int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height) ;

            // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

            if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2*skalierung.getRand()) {
                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, (int)(1*Punkt_y_Text));

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
            }
        }

        // Y - Achsen Beschriftung von 0 .. oo
        for (double lauf=0; lauf <= 3*1.1 ; lauf+=skalierung.getBeschriftungAbstandY()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );
            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
            int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

            // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

            // Text ausgeben
            g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

            // Kleine Markierungen anbringen, wo der Text steht
            g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
        }

        // Y - Achsen Beschriftung von -oo .. 0
        for (double lauf=0; lauf >= -3*0.95  ; lauf-=skalierung.getBeschriftungAbstandY()) {
            // Formatierung der Achsenbeschriftung
            DecimalFormat df = new DecimalFormat( "0.0" );
            String Beschriftung = df.format(lauf);

            // Platzbedarf fr Schrift ausrechnen
            FontMetrics fm = getFontMetrics(getFont());
            int Beschriftung_Width = fm.stringWidth(Beschriftung);
            int Beschriftung_Height = fm.getHeight();

            // X und Y - Koordinate fr Textausgabe
            int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
            int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

            // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
            int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

            // Text ausgeben
            g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

            // Kleine Markierungen anbringen, wo der Text steht
            g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
        }
    }

    /**
     * Hintergund wei darstellen
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintBackground (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Rahmen
        Color Farbe = new Color(0xffdcdc00);
        g2d.setColor(Farbe);
        g2d.setColor(Color.WHITE);
        g2d.fill(new Rectangle(skalierung.getRand(),skalierung.getRand(),skalierung.getWidth()-skalierung.getRand()*2,skalierung.getHeight()-skalierung.getRand()*2));
    }


    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseListener extends MouseAdapter {
        /**
         * Methode wird aufgerufen, falls Maustaste gedrckt wurde
         *
         * @param evt 	MouseEvent
         */
        public void mousePressed(MouseEvent evt) {
            // Mausposition holen
            int posX = evt.getX();
            // aus Mauskoordinaten Orginalwert errechnen
            skalierung.Rechnen(getWidth(), getHeight());	// Dies knnte man ausklammern!
            double Calculated_X_Value = skalierung.getRealXWert(posX);
            // Tolleranz bercksichtigen
            Klick_Tolleranz = 0.1d ; // * parameterEingabe.getStandardabweichung();

            // Gltiger Bereich?
            if (
                Calculated_X_Value >= parameterEingabeP.getUntergrenze() - Klick_Tolleranz
                &&
                Calculated_X_Value <= parameterEingabeP.getObergrenze() + Klick_Tolleranz
            ) {
                // Hier auf jeden Fall in einem Bereich in dem Reagiert wird!
                CanDrag = true;
                Is_Bond = 0;
                // unterscheiden ob Unter-, Obergrenze
                if (Calculated_X_Value <= parameterEingabeP.getUntergrenze() + Klick_Tolleranz)
                    Is_Bond = -1;
                if (Calculated_X_Value >= parameterEingabeP.getObergrenze() - Klick_Tolleranz)
                    Is_Bond = 1;
                Drag_Last = Calculated_X_Value;
                Drag_Old_Untergrenze = parameterEingabeP.getUntergrenze();
                Drag_Old_Obergrenze = parameterEingabeP.getObergrenze();
            }

        }



        /**
         * Methode wird aufgerufen, falls die "Maus" das Koordinatensystem verlassen hat, d.h.
         * im Ausgabemen, darf nun keine Mausposition mehr angezeigt werden.
         *
         * @param evt 	MouseEvent
         */
        public void mouseExited(MouseEvent evt) {
            if (CanDrag) {
                CanDrag = false;
            }
            setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        }

        /**
         * Methode wird aufgerufen, falls die Maustaste losgelassen wurde, d.h.
         * ein Punkt wurde verschoben bzw. aus dem Koordinatensystem gezogen und
         * wird gelscht.
         *
         * @param evt 	MouseEvent
         */
        public void mouseReleased(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag) {
                // ende Verschiebung
                CanDrag = false;
                // derzeige Mausposition
                int posX = evt.getX();
                // wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);
                if (Is_Bond==-1) {
                    // Untergrenze
                    parameterEingabeP.setUntergrenze(Calculated_X_Value);
                }
                if (Is_Bond==+1) {
                    // Obergrenze
                    parameterEingabeP.setObergrenze(Calculated_X_Value);
                }

            }
        }
    }

    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseMotionListener extends MouseMotionAdapter {
        /**
         * Methode wird aufgerufen, falls Maus bewegt wurde.
         * Maus Cursor wird angepasst
         *
         * @param evt 	MouseEvent
         */
        public void mouseMoved(MouseEvent evt) {
            if (CanDrag != true) {
                // derzeige Mausposition
                int posX = evt.getX();
                // Wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);
                Klick_Tolleranz = 0.1d ;
                // Mauszeiger ber Abgrifflinie -> Zeiger ndern
                if (
                    Calculated_X_Value >= parameterEingabeP.getUntergrenze() - Klick_Tolleranz
                    &&
                    Calculated_X_Value <= parameterEingabeP.getObergrenze() + Klick_Tolleranz
                ) {

                    // Auf jeden Fall im interresanten Bereich

                    if (Calculated_X_Value <= parameterEingabeP.getUntergrenze() + Klick_Tolleranz) {
                        // Unter/Obergrenzen
                        setCursor(new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR  ));
                    } else {
                        if (Calculated_X_Value >= parameterEingabeP.getObergrenze() - Klick_Tolleranz) {
                            // Unter/Obergrenzen
                            setCursor(new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR  ));
                        }

                    }


                } else {
                    // Wenn nicht, normalen Cursor einstellen
                    setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR   ));
                }
            }
        }

        /**
         * Methode wird aufgerufen, falls Maus mit gedrckter Maustaste bewegt wurde.
         *
         * @param evt 	MouseEvent
         */
        public void mouseDragged(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag == true) {
                // Mausposition holen
                int posX = evt.getX();
                // Orginalwert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);

                if (Is_Bond==-1) {
                    // Untergrenze
                    parameterEingabeP.setUntergrenze(Calculated_X_Value);
                }
                if (Is_Bond==+1) {
                    // Obergrenze
                    parameterEingabeP.setObergrenze(Calculated_X_Value);
                }


            }
        }

    }

}
