import java.awt.*;
import java.awt.event.*;

/*
 * Diese Klasse zeichnet ein einfaches Koordinatensystem,
 * beschriftet dieses, zeichnet das Richtungsfeld und eventuelle Lsungen
 */
public class EasyCoord extends Component implements ComponentListener {
    //------------------------------------------------------
    // Konstanten
    //------------------------------------------------------

    // Erfllt eigentlich keinen Zweck, aber Eclipse will, dass diese erstellt wird.

    private static final long serialVersionUID = -8029634734071358144L;

    // Font fr die Beschriftung des Koordinatensystems

    private static final Font BESCHRIFTUNG = new Font("Arial",Font.BOLD,12);

    // Farbe der Achsen und der Beschriftung (schwarz)

    private static final Color ACHSENFARBE = Color.black;

    // Wertebereich des Koordinatensystems.

    private static final double XMAX =  3.5,
                                XMIN = -3.5,
                                YMAX =  2.5,
                                YMIN = -2.5;

    // Breite und Hhe des Frames

    private int frameWidth;
    private int frameHeight;

    // aktuelle Werte der Mauskoordinaten in Pixel

    public static int xmaus=0;
    public static int ymaus=0;

    // aktuelle Werte der Mauskoordinaten

    public static double xmaus1=0;
    public static double ymaus1=0;

    // Werte der Hhenlinien

    private int[] xWerte;
    private int[][] yWerte;

    // Werte der speziellen Lsung

    private int[] xWerte1;
    private int[] yWerte1;
    private int[] yWerte2;

    // Variblen fr gewhlte Optionen

    public int nr;
    private int hnr=1;
    private boolean hoehe;
    public boolean spez;

    //------------------------------------------------------
    // Konstruktor und Methoden
    //------------------------------------------------------

    public EasyCoord(int value) {
        // Konstruktor der berklasse aufrufen
        super();
        this.addComponentListener(this);

    }

    //------------------------------------------------------
    // Methoden fr das Zeichnen
    //------------------------------------------------------

    /*
     * Diese Funktion wird immer aufgerufen, wenn das Objekt
     * neu gezeichnet werden muss. Sie ist eine berschrieben Methode
     * aus der berklasse.
    */
    public void paint(Graphics g) {
        if (nr==0) g.drawString("Bitte DGL anwhlen",115,125);
        if (nr!=0) {
            // Richtungsfeld zeichnen
            drawArrow(g);
        }
        if (nr!=0 && hoehe) {
            // max 6 Hhenlinien zeichnen
            for (hnr = 0; hnr < 6; hnr++) {
                drawPolylineFat(xWerte, yWerte[hnr], 1, g, Color.BLUE);
            }
        }

        if (nr!=0 && spez) {
            drawPolylineFat(xWerte1, yWerte1, 1, g, Color.GREEN);
            if(nr==3 || nr==4 || nr==7 || nr==8)
                drawPolylineFat(xWerte1, yWerte2, 1, g, Color.GREEN);
            // Punkt zeichnen
            drawCircle(xmaus,ymaus,8, g, Color.BLACK);
        }
        zeichneKoordinatenSystem(g);
        zeichneBeschriftung(g);
    }

    // Zeichnen der Achsen des Koordinatensystems. g = Grafikobjekt, auf dem gezeichnet wird

    private void zeichneKoordinatenSystem(Graphics g) {
        // zuerst Wertebereich in tatschliche (Pixel-)
        // Koordinaten umrechnen.
        int cxMin = calcPosX(XMIN);
        int cxMax = calcPosX(XMAX);
        int cyMin = calcPosY(YMIN);
        int cyMax = calcPosY(YMAX);

        g.setColor(ACHSENFARBE);

        // 2 Pixel dicke Linien fr die Achsen
        g.fillRect(cxMin, calcPosY(0)-1, cxMax-cxMin, 2);
        g.fillRect(calcPosX(0)-1, cyMax, 2, cyMin-cyMax);
    }

    // Zeichnen der Beschriftung der Koordinatenachsen. g = Grafikobjekt, auf dem gezeichnet wird

    private void zeichneBeschriftung(Graphics g) {
        // Temporre Variable fr For-Schleife
        double temp;
        // Variable zur richtigen Einpassung des Strings
        int stringShift;
        // Inhalt des zu schreibenden Strings
        String stringValue;

        // Font und Farbe fr Achsenbeschriftung
        g.setColor(ACHSENFARBE);
        g.setFont(BESCHRIFTUNG);

        FontMetrics fm = getFontMetrics(BESCHRIFTUNG);

        // Pfeil der x-Achse
        drawLineFat(calcPosX(XMAX)-8, calcPosY(0)-5, calcPosX(XMAX)+1, calcPosY(0), 1, g);
        drawLineFat(calcPosX(XMAX)-8, calcPosY(0)+4, calcPosX(XMAX)+1, calcPosY(0)-1, 1, g);

        // Pfeil der y-Achse
        drawLineFat(calcPosX(0)-5, calcPosY(YMAX)+9, calcPosX(0), calcPosY(YMAX), 1, g);
        drawLineFat(calcPosX(0)+5, calcPosY(YMAX)+9, calcPosX(0), calcPosY(YMAX), 1, g);

        // "x" und "y" an den Achsen
        g.drawString("x",680,calcPosY(0)+15);
        g.drawString("y",calcPosX(0)-14,10);

        // Werte Y-Achse (parallel verschoben von X-Achse)
        for (temp = (int) (YMIN); temp <= (int) (YMAX); temp++) {
            if (temp !=0) { // zwei mal 0.0 am koordinatensystem vermeiden
                stringValue = String.valueOf((double)temp);

                stringShift = (int) (fm.stringWidth(stringValue) / 2);

                // Kleine Markierung (Strich) zeichnen
                g.drawLine(calcPosX(0)+3, calcPosY(temp), calcPosX(0)-3, calcPosY(temp));
                // Beschriftung zeichnen
                g.drawString(stringValue,calcPosX(0)-stringShift-18,calcPosY(temp)+4);
            }
        }
        // Werte X-Achse (parallel verschoben von Y-Achse)
        for (temp = (int) (XMIN); temp <= (int) (XMAX); temp++) {
            stringValue = String.valueOf(temp);
            // Berechnen um wieviel der Text nach links verschoben
            // werden mu, damit er in der Mitte liegt.
            stringShift = (int) (fm.stringWidth(stringValue) / 2);

            // Kleine Markierung (Strich) zeichnen
            g.drawLine(calcPosX(temp), calcPosY(0)+3, calcPosX(temp), calcPosY(0)-3);
            // Beschriftung zeichnen mit Ausnahme 0
            if (temp==0) {
                g.drawString(String.valueOf(0),calcPosX(temp)-5-stringShift,calcPosY(0)+17);
            } else {
                g.drawString(stringValue,calcPosX(temp)-stringShift,calcPosY(0)+17);
            }
        }
    }


    //------------------------------------------------------
    // Methoden zum Berechnen
    //------------------------------------------------------

    // Berechnung des richtigen Richtungsfeldes
    public void RFeld(int value) {
        nr=value;

        repaint();
    }

    public void Hoehenlinien(boolean value) {
        //Sollen Hhenlinien gezeichnet werden?
        hoehe = value;
        // Berechnung der richtigen Hhenlinien
        Berechne();

        repaint();
    }

    // Umrechnen der Pixel in Koordinaten
    public void spezLoesung() {

        xmaus1=(xmaus-344)/98.0;
        ymaus1=-(ymaus-245)/98.0;
        spezLoesung(xmaus1, ymaus1);
    }

    // Berechnungs-Funktion fr spezielle Lsung (Auflsen nach C)
    public void spezLoesung(double x, double y) {

        int temp;
        double xValue=0;
        double yValue=0;
        // einige Lsungen bestehen aus 2 Teilen
        double yValue1=0;

        //Berechnung der Konstante fr spez. Lsung
        double Konstante=1.5;
        if (nr==1) {
            Konstante = (y-((x*x)/2));
        }
        if (nr==2) {
            Konstante = ((Math.log(Math.abs(y))-x));
        }
        if (nr==3) {
            Konstante = (x*x+y*y);
        }
        if (nr==4) {
            Konstante = (x*x-y*y);
        }
        if (nr==5) {
            Konstante = y/x;
        }
        if (nr==6) {
            Konstante = x*y-x*x/2;
        }
        if (nr==7) {
            Konstante = (x*x+Math.sqrt(y+1));
        }
        if (nr==8) {
            if(x!=0) {
                Konstante = ((x*x+y*y)/(x));
            } else Konstante=y*y/0.01;
        }
        if (nr==9) {
            Konstante = ((-1/y)-(x*x/2));
        }
        if (nr==10) {
            Konstante = y*Math.exp(x*x);
        }
        if (nr==11) {
            Konstante = (y-Math.sin(x)+Math.cos(x))*Math.exp(x);
        }

        // Arrays neu initialisieren
        xWerte1 = new int[frameWidth];
        yWerte1 = new int[frameWidth];
        yWerte2 = new int[frameWidth];

        // For-Schleife berechnet fr maximale Genauigkeit an jedem Pixel den jeweiligen f-Wert

        for (temp = 0; temp < frameWidth; temp++) {

            // Alle X-Werte berechnen, in Pixel umrechnen und ins Array schreiben.

            xValue = XMIN + (temp*((XMAX-XMIN)/frameWidth));
            xWerte1[temp] = calcPosX(Math.min(Math.max(xValue, XMIN-1), XMAX+1));

            // alle Y-Werte berechnen, in Pixel umrechnen und ins Array schreiben

            if (nr==1) yValue = ((xValue*xValue/2)+Konstante);
            if (nr==2) {
                if (y>0) {
                    yValue = (Math.exp(xValue+Konstante));
                } else yValue = -(Math.exp(xValue+Konstante));
            }
            if (nr==3) {
                yValue = (Math.sqrt(Konstante-xValue*xValue));
                yValue1 = -(Math.sqrt(Konstante-xValue*xValue));
            }
            if (nr==4) {
                yValue = (Math.sqrt(xValue*xValue-Konstante));
                yValue1 = (-Math.sqrt(xValue*xValue-Konstante));
            }
            if (nr==5) {
                yValue = Konstante*xValue;
            }
            if (nr==6) {
                yValue = Konstante/xValue + xValue/2;
            }
            if (nr==7) {
                if (Math.abs(xValue) <= Math.sqrt(Konstante)) {
                    yValue = -1+Math.pow((Konstante-xValue*xValue), 2);
                } else yValue = 20;
                if((xmaus==345 && ymaus==344) ||(xmaus==344 && ymaus==343)) {
                    yValue1=-1;
                    xmaus1=0;
                    ymaus1=-1;
                }
            }
            if (nr==8) if(xValue!=0) {
                    yValue= Math.sqrt(Konstante*xValue-xValue*xValue);
                    yValue1= -Math.sqrt(Konstante*xValue-xValue*xValue);
                }
            if (nr==9) {
                yValue = -(2/(2*Konstante+xValue*xValue));
            }

            if (nr==10) {
                yValue = Konstante*Math.exp(-(xValue*xValue));
            }

            if (nr==11) {
                yValue = Konstante*Math.exp(-xValue)+Math.sin(xValue)-Math.cos(xValue);
            }


            yWerte1[temp] = calcPosY(Math.min(Math.max(yValue, YMIN-1), YMAX+1));
            if(nr==3 || nr==4 || nr==7 || nr==8) yWerte2[temp] = calcPosY(Math.min(Math.max(yValue1, YMIN-1), YMAX+1));

            // fehlerhafte Berechnungen aus dem Sichtfeld schieben
            if(yWerte1[temp]==0) yWerte1[temp]=-1;
            if(yWerte2[temp]==0) yWerte2[temp]=-1;
        }

        // Neuzeichnen, um das Ergebnis darzustellen
        repaint();
    }

    // Berechnungs-Funktion fr Hhenlinien (Auflsen nach Y)

    public void Berechne() {
        // Temporre Variable fr Schleife.
        int temp;
        // Variable fr x-Werte
        double xValue;
        // Variable fr y-Werte
        double yValue;

        // Arrays neu initialisieren
        xWerte = new int[frameWidth];
        yWerte = new int[6][frameWidth];

        // For-Schleife berechnet fr maximale Genauigkeit an jedem Pixel den jeweiligen f-Wert

        for (temp = 0; temp < frameWidth; temp++) {

            // Alle X-Werte berechnen, in Pixel umrechnen und ins Array schreiben.

            xValue = XMIN + (temp*((XMAX-XMIN)/frameWidth));
            xWerte[temp] = calcPosX(Math.min(Math.max(xValue, XMIN-1), XMAX+1));

            // nochmalige Schleife fr alle Hhenlinien,
            // alle Y-Werte berechnen, in Pixel umrechnen und ins Array schreiben
            for (hnr = 0; hnr < 6; hnr++) {

                yValue = hoehef(xValue);

                yWerte[hnr][temp] = calcPosY(Math.min(Math.max(yValue, YMIN-1), YMAX+1));
                // fehlerhafte Berechnungen aus dem Sichtfeld schieben
                if(yWerte[hnr][temp]==0) yWerte[hnr][temp]=-1;
            }
        }

        // Neuzeichnen, um das Ergebnis darzustellen
        repaint();
    }

    // Berechnet einige sinnvolle Hhenlinien einer DGL

    private double hoehef(double xWert) {
        // Funktionen fr jede Nummer/Hhenlinie
        if (nr==1 && hnr==0) {
            return ((xWert*xWert)/2)-4;
        }
        if (nr==1 && hnr==1) {
            return ((xWert*xWert)/2)-2;
        }
        if (nr==1 && hnr==2) {
            return ((xWert*xWert)/2);
        }
        if (nr==1 && hnr==3) {
            return ((xWert*xWert)/2)+2;
        }

        if (nr==2 && hnr==0) {
            return Math.exp(xWert);
        }
        if (nr==2 && hnr==1) {
            return Math.exp(xWert+2);
        }
        if (nr==2 && hnr==2) {
            return Math.exp(xWert-3);
        }
        if (nr==2 && hnr==3) {
            return -Math.exp(xWert);
        }
        if (nr==2 && hnr==4) {
            return -Math.exp(xWert+2);
        }
        if (nr==2 && hnr==5) {
            return -Math.exp(xWert-3);
        }

        if (nr==3 && hnr==0) {
            return Math.sqrt(1-xWert*xWert);
        }
        if (nr==3 && hnr==1) {
            return Math.sqrt(3-xWert*xWert);
        }
        if (nr==3 && hnr==2) {
            return Math.sqrt(8-xWert*xWert);
        }
        if (nr==3 && hnr==3) {
            return -Math.sqrt(1-xWert*xWert);
        }
        if (nr==3 && hnr==4) {
            return -Math.sqrt(3-xWert*xWert);
        }
        if (nr==3 && hnr==5) {
            return -Math.sqrt(8-xWert*xWert);
        }

        if (nr==4 && hnr==0) {
            return Math.sqrt(xWert*xWert+2);
        }
        if (nr==4 && hnr==1) {
            return Math.sqrt(xWert*xWert-1);
        }
        if (nr==4 && hnr==2) {
            return Math.sqrt(xWert*xWert-6);
        }
        if (nr==4 && hnr==3) {
            return -Math.sqrt(xWert*xWert+2);
        }
        if (nr==4 && hnr==4) {
            return -Math.sqrt(xWert*xWert-1);
        }
        if (nr==4 && hnr==5) {
            return -Math.sqrt(xWert*xWert-6);
        }

        if (nr==5 && hnr==0) {
            return (1*xWert);
        }
        if (nr==5 && hnr==1) {
            return (2*xWert);
        }
        if (nr==5 && hnr==2) {
            return (-2*xWert);
        }
        if (nr==5 && hnr==3) {
            return (-0.5*xWert);
        }

        if (nr==6 && hnr==0) {
            return (0.5/xWert+xWert/2);
        }
        if (nr==6 && hnr==1) {
            return (-1/xWert+xWert/2);
        }
        if (nr==6 && hnr==2) {
            return (2/xWert+xWert/2);
        }

        if (nr==7 && hnr==0) {
            if(Math.abs(xWert) <= Math.sqrt(1)) return -1+Math.pow((1-(xWert*xWert)),2);
            else return 20;
        }
        if (nr==7 && hnr==1) {
            if(Math.abs(xWert) <= Math.sqrt(1.8)) return -1+Math.pow((1.8-(xWert*xWert)),2);
            else return 20;
        }
        if (nr==7 && hnr==2) {
            if(Math.abs(xWert) <= Math.sqrt(4)) return -1+Math.pow((4-(xWert*xWert)),2);
            else return 20;
        }
        if (nr==7 && hnr==3) {
            return -1;
        }

        if (nr==8 && hnr==0) {
            return Math.sqrt(-xWert*xWert+2*xWert*1);
        }
        if (nr==8 && hnr==1) {
            return Math.sqrt(-xWert*xWert+2*xWert*2);
        }
        if (nr==8 && hnr==2) {
            return Math.sqrt(-xWert*xWert+2*xWert*-1.5);
        }
        if (nr==8 && hnr==3) {
            return -Math.sqrt(-xWert*xWert+2*xWert*1);
        }
        if (nr==8 && hnr==4) {
            return -Math.sqrt(-xWert*xWert+2*xWert*2);
        }
        if (nr==8 && hnr==5) {
            return -Math.sqrt(-xWert*xWert+2*xWert*-1.5);
        }

        if (nr==9 && hnr==0) {
            return -(2/(1+xWert*xWert));
        }
        if (nr==9 && hnr==1) {
            return -(2/(4+xWert*xWert));
        }
        if (nr==9 && hnr==2) {
            return -(2/(-4+xWert*xWert));
        }

        if (nr==10 && hnr==0) {
            return Math.exp(-xWert*xWert);
        }
        if (nr==10 && hnr==1) {
            return 2*Math.exp(-xWert*xWert);
        }
        if (nr==10 && hnr==2) {
            return -2*Math.exp(-xWert*xWert);
        }

        if (nr==11 && hnr==0) {
            return Math.exp(-xWert)+Math.sin(xWert)-Math.cos(xWert);
        }
        if (nr==11 && hnr==1) {
            return 4*Math.exp(-xWert)+Math.sin(xWert)-Math.cos(xWert);
        }
        if (nr==11 && hnr==2) {
            return -2*Math.exp(-xWert)+Math.sin(xWert)-Math.cos(xWert);
        }
        if (nr==11 && hnr==3) {
            return -0.1*Math.exp(-xWert)+Math.sin(xWert)-Math.cos(xWert);
        }

        else return 0;
    }


    //------------------------------------------------------
    // Von Studienarbeit bernohmene Methoden
    //------------------------------------------------------

    /**
     * Die Methode calcPosX berechnet fr den x-Wert eines
     * Funktionswertes die x-Position in Pixel.
     * @param xValue x-Wert des Punktes
     */
    private int calcPosX(double xValue) {
        int xPosition;

        xPosition = (int) frameWidth - (int) ((frameWidth/(XMAX-XMIN)*XMAX)-(frameWidth/(XMAX-XMIN)*xValue));

        return(xPosition);
    }

    /**
     * Die Methode calcPosY berechnet fr den y-Wert eines
     * Funktionswertes die y-Position in Pixel.
     * @param yValue y-Wert des Punktes
     */
    private int calcPosY(double yValue) {
        int yPosition;

        yPosition =  (int) ((frameHeight/(YMAX-YMIN)*YMAX)-(frameHeight/(YMAX-YMIN)*yValue));

        return(yPosition);
    }

    /**
     * Diese Funktion zeichnet eine Linie whlbarer Dicke
     * @param x1 X-Koordinate des Startpunktes
     * @param y1 Y-Koordinate des Startpunktes
     * @param x2 X-Koordinate des Endpunktes
     * @param y2 Y-Koordinate des Endpunktes
     * @param size Gewnschte Dicke der Linie
     * @param g Das Grafikobjekt, auf dem gezeichnet wird
     */
    private void drawLineFat(int x1, int y1, int x2, int y2, int size, Graphics g) {
        double m;
        double c;

        int x;
        int y;
        // Korrektur der Rundungsfehler um y=0
        if((y2-y1)>200) {
            // Spezialfall DGL 7 ("endet" nicht in x-Achse)
            if (nr==7) {
                drawLineFat( x2, y2, x2, 245, 0, g);
            } else drawLineFat( x2, y2, x2, 245, size, g);
        } else if((y1-y2)>200) {
            // Spezialfall DGL 7 ("endet" nicht in x-Achse)
            if (nr==7) {
                drawLineFat( x1, y1, x1, 245, 0, g);
            } else if(nr==5 || nr==6 || nr ==9) {}
            else drawLineFat( x1, y1, x1, 245, size, g);
        } else if(!((x2 == x1) && (y2 == y1))) {
            if(Math.abs(x2-x1) >= Math.abs(y2-y1)) {
                m = ((double) (y2-y1)) / ((double) (x2-x1));
                c = -(m*x1)+y1;
                for(x = x1; x != x2; x += Math.signum((x2-x1))) {
                    y = (int) Math.round(m * x + c);
                    g.fillOval(x-size,y-size,2*size,2*size);
                }
            } else {
                m = ((double) (x2-x1)) / ((double) (y2-y1));
                c = -(m*y1)+x1;

                for(y = y1; y != y2; y += Math.signum((y2-y1))) {
                    x = (int) Math.round(m * y + c);
                    g.fillOval(x-size,y-size,2*size,2*size);
                }
            }
        } else {
            g.fillOval(x1-size,y1-size,2*size,2*size);
        }
    }

    // Zeichnen einer Linien
    // bergabe der x- und y-Koordinaten, der Dicke, des Grafikobjekts und der Farbe

    private void drawPolylineFat(int[] xPoints, int[] yPoints, int size, Graphics g, Color c) {
        int length = Math.min(xPoints.length, yPoints.length);
        int temp;
        g.setColor(c);

        if(length > 1) {
            for(temp = 0; temp < length-1; temp++) {
                drawLineFat(xPoints[temp], yPoints[temp], xPoints[temp+1], yPoints[temp+1], size, g);
            }
        } else if(length > 0) {
            drawLineFat(xPoints[0], yPoints[0], xPoints[0], yPoints[0], size, g);
        }
    }

    // Zeichnen der Richtungsfeld-Pfeile/Striche
    // bergabe der Endkoordinaten und des Grafikobjekts, auf dem gezeichnet wird.

    private void drawArrow(Graphics g) {

        for(int x=-330; x<350; x=x+44) {
            for(int y=-226; y<250; y=y+41) {
                //Zeichnen der Linien
                double x1=x/100.0;
                double y1=y/100.0;
                //bergabe der Funktionsnummer und Festlegen der Steigung
                double s=0;
                if (nr==1) {
                    s=x1;
                }
                if (nr==2) {
                    s=y1;
                }
                if (nr==3) {
                    s=-(x1/y1);
                }
                if (nr==4) {
                    s=(x1/y1);
                }
                if (nr==5) {
                    s=(y1/x1);
                }
                if (nr==6) {
                    s=((x1-y1)/x1);
                }
                if (nr==7) {
                    s=(-4*x1*Math.sqrt(y1+1));
                }
                if (nr==8) {
                    s=((y1*y1)-(x1*x1))/(2*x1*y1);
                }
                if (nr==9) {
                    s=(x1*y1*y1);
                }
                if (nr==10) {
                    s=-2*x1*y1;
                }
                if (nr==11) {
                    s=-y1+2*Math.sin(x1);
                }

                // Umrechnung auf Pfeillnge 20 mittels Satz des Pythagoras
                double c=Math.sqrt(1+s*s) ;
                double faktor=20/c;

                // Berechnung der Seiten des Dreiecks
                int x2= (int) faktor;
                int y2= (int) (faktor*s);
                g.setColor(Color.RED);
                // Zeichnen der Linie um Mittelpunkt
                g.drawLine(x+344-(x2/2), 244-y+(y2/2), x+344+(x2/2), 244-y-(y2/2));
                // Zeichnen des Mittelpunktes
                drawPoint(x+344, 244-y, 4, g, Color.RED);
            }
        }
    }

    private void drawPoint(int x, int y, int durchmesser, Graphics g, Color c) {
        g.setColor(c);
        g.drawOval(x-durchmesser/2, y-durchmesser/2, durchmesser, durchmesser);
        g.drawOval(x-durchmesser/2, y-durchmesser/2, durchmesser-1, durchmesser-1);
        g.drawOval(x-durchmesser/2, y-durchmesser/2, durchmesser-2, durchmesser-2);
        g.drawOval(x-durchmesser/2, y-durchmesser/2, durchmesser-3, durchmesser-3);
        g.fillOval(x-durchmesser/2, y-durchmesser/2, durchmesser, durchmesser);
    }

    private void drawCircle(int x, int y, int durchmesser, Graphics g, Color c) {
        g.setColor(c);
        g.drawOval(x-durchmesser/2, y-durchmesser/2, durchmesser, durchmesser);
        g.drawOval(x-(durchmesser-2)/2, y-(durchmesser-2)/2, durchmesser-2, durchmesser-2);
    }

    //------------------------------------------------------
    // Implementierte Methoden
    //------------------------------------------------------

    /**
     * Gre gendert? Dann neue Breite und Hhe bernehmen,
     * paint(Graphics) wird automatisch ausgefhrt
     */
    public void componentResized(ComponentEvent event) {
        frameWidth = this.getWidth();
        frameHeight = this.getHeight();

        // Unbedingt neu berechnen, da sich Pixel-Koordinaten
        // gendert haben.
        Berechne();
    }

    public void componentMoved(ComponentEvent event) {
    }

    public void componentShown(ComponentEvent event) {
    }

    public void componentHidden(ComponentEvent event) {
    }

}