import java.applet.AppletContext;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import java.net.URL;
import java.text.DecimalFormat;

import javax.imageio.ImageIO;
import java.io.IOException;


/**
 * Die Klasse <b>EasyFrame</b> ist das ''Herz'' des Applets.
 * Sie erbt von JFrame (= Fensterklasse), und ordnet innerhalb
 * des Fensters folgende Bedienelemente an:<br>
 * <ul>
 * 	<li>Menleiste</li>
 * 	<li>Koordinatensystem</li>
 *  <li>Textfeld und Button</li>
 * </ul>
 * <br><br>
 * Es werden verschiedene Listener implementiert, diese sind
 * dafr zustndig, auf Ereignisse zu reagieren, z.Bsp.<br>
 * <ul>
 *  <li>Klick auf Button/Men (ActionListener),</li>
 *  <li>Fenstergrsse gendert (ComponentListener)</li>
 * </ul>
 * <br>
 * Wird ein Listener implementiert, so mssen in der Klasse
 * dessen Methoden implementiert werden. Seht einfach weiter
 * unten nach, die Methoden actionPerformed(ActionEvent) und
 * componentResized(componentEvent).
 * Gerst bernommen von Matthias Stirner
 */
public class EasyFrame extends javax.swing.JFrame implements
    ActionListener,	ComponentListener, MouseListener, MouseMotionListener, KeyListener {

    //------------------------------------------------------
    // Konstanten
    //------------------------------------------------------

    /**
     * Erfllt eigentlich keinen Zweck, aber Eclipse will,
     * da diese erstellt wird.
     */
    private static final long serialVersionUID = 1471052336179988906L;

    // Maximaler einstellbarer Wert.

    private static final double MAXVALUE = 9.99;

    // Minimaler einstellbarer Wert.

    private static final double MINVALUE = -9.99;

    // Standartwert

    private static final int DEFVALUE = 0;

    // Titel des Applets

    private static final String TITLE =
        "Richtungsfeld einer Differentialgleichung";

    //------------------------------------------------------
    // Globale Variablen
    //------------------------------------------------------

    /**
     * AppletContext, wird spter fr das ffnen der
     * Hilfedatei bentigt; wird im Konstruktor
     * bernommern.
     */
    private AppletContext appContext;

    // Verzeichnis der Icon-Dateien,

    private String imageBase;

    // Verzeichnis der HTML-Hilfsdatei

    private String helpFile;

    //------------------------------------------------------
    // Alle Bedienelemente werden vorher deklariert
    //------------------------------------------------------


    // Menleiste

    private JMenuBar menuBar;

    // Mens und Untermens.

    private JMenu dateiMenu, hilfeMenu;

    private JMenuItem endeMenuItem, zurueckMenuItem, htmlhelpMenuItem,
            aboutMenuItem;

    // Die 11 DGL-Buttons.

    private JToggleButton DGLButton1;

    private JToggleButton DGLButton2;

    private JToggleButton DGLButton3;

    private JToggleButton DGLButton4;

    private JToggleButton DGLButton5;

    private JToggleButton DGLButton6;

    private JToggleButton DGLButton7;

    private JToggleButton DGLButton8;

    private JToggleButton DGLButton9;

    private JToggleButton DGLButton10;

    private JToggleButton DGLButton11;

    // Trennstrich

    private JLabel TrennLabel;
    private JLabel PfeilLabel;

    // Anzeige der Lsung

    private JLabel LoesungLabel;

    // HTW-Logo

    private JLabel HTWLabel;

    // Ein Panel welches das Koordinatensystem enthlt.

    private JPanel coordPanel;

    // Das eigentliche Koordinatensystem.

    private EasyCoord coordSys;

    // Beschriftung fr das Hhenlinien-Textfeld.

    private JLabel HoeheLabel;
    private JLabel HoeheLabel1;

    // Checkbox fr die Hhenlinien

    private JCheckBox HoeheBox ;

    // Beschriftung fr das Spez.Lsung-Textfeld.

    private JLabel SpezLabel;
    private JLabel SpezLabel1;

    // Checkbox fr die Spez.Lsung

    public static JCheckBox SpezBox ;

    // Beschriftung fr das Punkt-Textfeld.

    private JLabel PunktLabel;
    private JLabel xLabel;
    private JLabel yLabel;

    // Das Punkt-Textfeld am unteren Ende des Fensters. Hier x und y eingeben.

    private JAlphanumericTextField xTextfeld;

    private JAlphanumericTextField yTextfeld;

    // Alle DGLs zu einer Gruppe gruppieren

    ButtonGroup group = new ButtonGroup();

    //Dezimalformat mit 2 Kommastellen

    DecimalFormat df = new DecimalFormat("0.00");

    // Icon der Lsung der aktuellen DGL

	Icon LoesungIcon;

    // Icon aller Lsungen
    Icon iconL0;
    Icon iconL1;
    Icon iconL2;
    Icon iconL3;
    Icon iconL4;
    Icon iconL5;
    Icon iconL6;
    Icon iconL7;
    Icon iconL8;
    Icon iconL9;
    Icon iconL10;
    Icon iconL11;


    //------------------------------------------------------
    // Konstruktor und Methoden
    //------------------------------------------------------

    /**
     * bernimmt die bergabeparameter, speichert diese
     * zur spteren Verwendung, und macht dann weiter mit
     * dem nchsten Schritt - initGUI.
     * startet initGUI()
     * @param imagePath Pfad zu den Icondateien.
     * @param helpPath Pfad zur Hilfedatei.
     * @param ac Fr ffnen der Hilfedatei notwendig.
     */
    public EasyFrame(String imagePath, String helpPath, AppletContext ac) {
        // Konstruktor der berklasse (Frame) ausfhren
        super();

        // bergabeparameter zur spteren Verwendung speichern
        imageBase = imagePath;
        helpFile = helpPath;
        appContext = ac;

        initGUI();
    }


    // Ordnet alle Bedienelemente an

    private void initGUI() {
        initMenu();

        // Kritische Operationen immer mit try{} und catch{}
        // durchgehen, so strzt bei einem Fehler nicht das
        // ganze Applet ab.
        try {
            // DefaultCloseOperation: Wenn Fenster geschlossen
            // wird, alle Daten aus dem Speicher werfen.
            setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
            // Gre des Main-Frames (ohne Menleiste)
            setSize(830, 668);
            // Gre nicht vernderbar
            setResizable(false);
            // damit auf Grennderung reagiert werden kann
            this.addComponentListener(this);
            setTitle(TITLE);

            // NULL-Layout fr volle Kontrolle ber Postionierung der Elemente.
            this.getContentPane().setLayout(null);

            {
                DGLButton1 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/1.gif"))));
                this.getContentPane().add(DGLButton1);
                group.add(DGLButton1);
                DGLButton1.setToolTipText("Richtungsfeld zeichnen");
                DGLButton1.addActionListener(this);
            }
            {
                DGLButton2 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/2.gif"))));
                this.getContentPane().add(DGLButton2);
                group.add(DGLButton2);
                DGLButton2.setToolTipText("Richtungsfeld zeichnen");
                DGLButton2.addActionListener(this);
            }
            {
                DGLButton3 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/3.gif"))));
                this.getContentPane().add(DGLButton3);
                group.add(DGLButton3);
                DGLButton3.setToolTipText("Richtungsfeld zeichnen");
                DGLButton3.addActionListener(this);
            }
            {
                DGLButton4 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/4.gif"))));
                this.getContentPane().add(DGLButton4);
                group.add(DGLButton4);
                DGLButton4.setToolTipText("Richtungsfeld zeichnen");
                DGLButton4.addActionListener(this);
            }
            {
                DGLButton5 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/5.gif"))));
                this.getContentPane().add(DGLButton5);
                group.add(DGLButton5);
                DGLButton5.setToolTipText("Richtungsfeld zeichnen");
                DGLButton5.addActionListener(this);
            }
            {
                DGLButton6 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/6.gif"))));
                this.getContentPane().add(DGLButton6);
                group.add(DGLButton6);
                DGLButton6.setToolTipText("Richtungsfeld zeichnen");
                DGLButton6.addActionListener(this);
            }
            {
                DGLButton7 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/7.gif"))));
                this.getContentPane().add(DGLButton7);
                group.add(DGLButton7);
                DGLButton7.setToolTipText("Richtungsfeld zeichnen");
                DGLButton7.addActionListener(this);
            }
            {
                DGLButton8 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/8.gif"))));
                this.getContentPane().add(DGLButton8);
                group.add(DGLButton8);
                DGLButton8.setToolTipText("Richtungsfeld zeichnen");
                DGLButton8.addActionListener(this);
            }
            {
                DGLButton9 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/9.gif"))));
                this.getContentPane().add(DGLButton9);
                group.add(DGLButton9);
                DGLButton9.setToolTipText("Richtungsfeld zeichnen");
                DGLButton9.addActionListener(this);
            }
            {
                DGLButton10 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/10.gif"))));
                this.getContentPane().add(DGLButton10);
                group.add(DGLButton10);
                DGLButton10.setToolTipText("Richtungsfeld zeichnen");
                //berechnenButton9.setBounds(120, 435, 100, 30);
                DGLButton10.addActionListener(this);
            }

            {
                DGLButton11 = new JToggleButton(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/11.gif"))));
                this.getContentPane().add(DGLButton11);
                group.add(DGLButton11);
                DGLButton11.setToolTipText("Richtungsfeld zeichnen");
                DGLButton11.addActionListener(this);
            }
            // HTW-Logo
            {
                HTWLabel = new JLabel(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/HTW.gif"))));
                this.getContentPane().add(HTWLabel);
            }
            // Trennstrich
            {
                TrennLabel = new JLabel(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/Trennstrich.gif"))));
                this.getContentPane().add(TrennLabel);
            }
            {
                PfeilLabel = new JLabel(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/Pfeil.gif"))));
                this.getContentPane().add(PfeilLabel);
            }
            // Label fr die Lsung
            {
                LoesungLabel = new JLabel(new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L0.gif"))));
                this.getContentPane().add(LoesungLabel);
                LoesungLabel.setBorder(BorderFactory.createTitledBorder("allgemeine Lsung"));
            }
            // Koordinatensystem
            {
                coordPanel = new JPanel();
                this.getContentPane().add(coordPanel);
                coordPanel.setBorder(BorderFactory
                                     .createTitledBorder(""));
                coordPanel.setLayout(new BorderLayout());
                {
                    coordSys = new EasyCoord(DEFVALUE);
                    coordSys.addMouseListener(this);
                    coordSys.addMouseMotionListener(this);
                    coordPanel.add(coordSys);

                }
            }
            // Lsungsbeispiele
            {
                HoeheLabel = new JLabel();
                this.getContentPane().add(HoeheLabel);
                HoeheLabel.setText("Lsungsbeispiele");
            }
            {
                HoeheLabel1 = new JLabel();
                this.getContentPane().add(HoeheLabel1);
                HoeheLabel1.setText("einblenden");
            }
            {
                HoeheBox = new JCheckBox();
                this.getContentPane().add(HoeheBox);
                HoeheBox.addActionListener(this);
            }
            // Spezielle Lsung
            {
                SpezLabel = new JLabel();
                this.getContentPane().add(SpezLabel);
                SpezLabel.setText("Spezielle Lsung");
            }
            {
                SpezLabel1 = new JLabel();
                this.getContentPane().add(SpezLabel1);
                SpezLabel1.setText("einblenden");
            }
            {
                SpezBox = new JCheckBox();
                this.getContentPane().add(SpezBox);
                SpezBox.addActionListener(this);
            }
            {
                PunktLabel = new JLabel();
                this.getContentPane().add(PunktLabel);
                PunktLabel.setText("Punkt");
            }
            {
                xLabel = new JLabel();
                this.getContentPane().add(xLabel);
                xLabel.setText("x");
            }
            {
                yLabel = new JLabel();
                this.getContentPane().add(yLabel);
                yLabel.setText("y");
            }
            {
                xTextfeld = new JAlphanumericTextField(1.00 ,MINVALUE,MAXVALUE);
                this.getContentPane().add(xTextfeld);
                xTextfeld.setHorizontalAlignment(SwingConstants.TRAILING);
                xTextfeld.addKeyListener(this);
            }
            {
                yTextfeld = new JAlphanumericTextField(1.00 ,MINVALUE,MAXVALUE);
                this.getContentPane().add(yTextfeld);
                yTextfeld.setHorizontalAlignment(SwingConstants.TRAILING);
                yTextfeld.addKeyListener(this);
            }

            // Frame in der Mitte des Bildschirms positionieren.
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            this.setLocation((screenSize.width - getSize().width) / 2,
                             (screenSize.height - getSize().height) / 2);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //Ordnet die Men-Elemente an

    private void initMenu() {
        try {
		
			try{
				
				LoesungIcon = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L0.gif")));
				
				iconL0 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L0.gif")));
				iconL1 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L1.gif")));
				iconL2 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L2.gif")));
				iconL3 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L3.gif")));
				iconL4 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L4.gif")));
				iconL5 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L5.gif")));
				iconL6 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L6.gif")));
				iconL7 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L7.gif")));
				iconL8 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L8.gif")));
				iconL9 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L9.gif")));
				iconL10 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L10.gif")));
				iconL11 = new ImageIcon(ImageIO.read(EasyFrame.class.getResourceAsStream("/images/L11.gif")));	
				
			}catch(IOException IOE){
			}
		
            menuBar = new JMenuBar();
            setJMenuBar(menuBar);
            {
                dateiMenu = new JMenu();
                // Men zur Menleiste hinzufgen.
                menuBar.add(dateiMenu);
                dateiMenu.setText("Datei");
                {

                    zurueckMenuItem = new JMenuItem();
                    // Men zum Men hinzufgen.
                    dateiMenu.add(zurueckMenuItem);
                    zurueckMenuItem.setText("Alles zurcksetzen");
                    // Schnelltaste frs Zurcksetzen (ALT + Z)
                    zurueckMenuItem.setAccelerator(KeyStroke.getKeyStroke('Z', Event.ALT_MASK));
                    zurueckMenuItem.addActionListener(this);

                }
                {
                    // Seperator zur besseren bersicht
                    dateiMenu.addSeparator();
                }
                {
                    endeMenuItem = new JMenuItem();
                    dateiMenu.add(endeMenuItem);
                    endeMenuItem.setText("Beenden");
                    // Schnelltaste fr Beenden (ALT + Q)
                    endeMenuItem.setAccelerator(KeyStroke.getKeyStroke('Q', Event.ALT_MASK));
                    endeMenuItem.addActionListener(this);
                }
            }
            {
                hilfeMenu = new JMenu();
                menuBar.add(hilfeMenu);
                hilfeMenu.setText("?");
                {
                    htmlhelpMenuItem = new JMenuItem();
                    hilfeMenu.add(htmlhelpMenuItem);
                    htmlhelpMenuItem.setText("HTML-Anleitung");
                    htmlhelpMenuItem.setAccelerator(KeyStroke.getKeyStroke('H', 0));
                    htmlhelpMenuItem.addActionListener(this);
                }
                {
                    // Seperator zur besseren bersicht
                    hilfeMenu.addSeparator();
                }
                {
                    aboutMenuItem = new JMenuItem();
                    hilfeMenu.add(aboutMenuItem);
                    aboutMenuItem.setText("ber...");
                    aboutMenuItem.addActionListener(this);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void mouseClicked(MouseEvent event) {
        if (coordSys.spez==true && coordSys.nr!=0) {

            EasyCoord.xmaus=event.getX();
            EasyCoord.ymaus=event.getY();
            coordSys.spezLoesung();
            xTextfeld.setText(String.valueOf(df.format(EasyCoord.xmaus1)));
            yTextfeld.setText(String.valueOf(df.format(EasyCoord.ymaus1)));
            xTextfeld.lastValid=EasyCoord.xmaus1;
            yTextfeld.lastValid=EasyCoord.ymaus1;
        }

    }

    public void mouseDragged(MouseEvent event) {
        if (coordSys.spez==true && coordSys.nr!=0) {

            EasyCoord.xmaus=event.getX();
            EasyCoord.ymaus=event.getY();
            coordSys.spezLoesung();
            xTextfeld.setText(String.valueOf(df.format(EasyCoord.xmaus1)));
            yTextfeld.setText(String.valueOf(df.format(EasyCoord.ymaus1)));
            xTextfeld.lastValid=EasyCoord.xmaus1;
            yTextfeld.lastValid=EasyCoord.ymaus1;
        }
    }


    //------------------------------------------------------
    // Implementierte Methoden, hier wird auf Ereignisse
    // reagiert.
    //------------------------------------------------------
    // key == KeyEvent.VK_ENTER ||
    public void keyPressed(KeyEvent event) {
        int key = event.getKeyCode();
        if (key ==KeyEvent.VK_ENTER) {
            coordSys.spez=true;
            SpezBox.setSelected(true);
            EasyCoord.xmaus=(int) ((xTextfeld.get())*99+345);
            EasyCoord.ymaus=(int) ((yTextfeld.get())*(-99)+245);
            coordSys.spezLoesung(xTextfeld.get(),yTextfeld.get());

        }
    }

    public void actionPerformed(ActionEvent event) {

        // Quelle des Ereignisses (also: JButton oder
        // JMenuItem) in source speichern.
        Object source = event.getSource();

        // ...dann alle Mglichkeiten durchgehen und entsprechend reagieren

        if (source == DGLButton1) {
            coordSys.RFeld(1);
            LoesungIcon = iconL1;
        }
        if (source == DGLButton2) {
            coordSys.RFeld(2);
            LoesungIcon = iconL2;
        }
        if (source == DGLButton3) {
            coordSys.RFeld(3);
            LoesungIcon = iconL3;
        }
        if (source == DGLButton4) {
            coordSys.RFeld(4);
            LoesungIcon = iconL4;
        }
        if (source == DGLButton5) {
            coordSys.RFeld(5);
            LoesungIcon = iconL5;
        }
        if (source == DGLButton6) {
            coordSys.RFeld(6);
            LoesungIcon = iconL6;
        }
        if (source == DGLButton7) {
            coordSys.RFeld(7);
            LoesungIcon = iconL7;
        }
        if (source == DGLButton8) {
            coordSys.RFeld(8);
            LoesungIcon = iconL8;
        }
        if (source == DGLButton9) {
            coordSys.RFeld(9);
            LoesungIcon = iconL9;
        }
        if (source == DGLButton10) {
            coordSys.RFeld(10);
            LoesungIcon = iconL10;
        }
        if (source == DGLButton11) {
            coordSys.RFeld(11);
            LoesungIcon = iconL11;
        }
        // Lsungbeispiele
        if (HoeheBox.isSelected()) {
            coordSys.Hoehenlinien(true);
            LoesungLabel.setIcon(LoesungIcon);
        } else {
            coordSys.Hoehenlinien(false);
            LoesungLabel.setIcon(iconL0);
        }
        // spezielle Lsung
        if (SpezBox.isSelected()) {
            coordSys.spez=true;
            coordSys.spezLoesung(xTextfeld.get(),yTextfeld.get());
            EasyCoord.xmaus=(int) ((xTextfeld.get())*99+345);
            EasyCoord.ymaus=(int) ((yTextfeld.get())*(-99)+245);
        } else {
            coordSys.spez=false;
        }
        // Applet beenden
        if (source == endeMenuItem) {
            // Erst unsichtbar machen, dann ''wegwerfen''
            this.setVisible(false);
            this.dispose();
        }
        // HTML-Hilfdatei im Browser ffnen.
        if(source == htmlhelpMenuItem) {
            try {
                // So wird ein Browserfenster geffnet.
                appContext.showDocument(new URL(helpFile), "_blank");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        //	About-Dialog anzeigen...

        if (source == aboutMenuItem) {
            AboutDlg dlg = new AboutDlg(this, imageBase);
            dlg.setVisible(true);
            dlg.dispose();
        }

        // Alles Zurcksetzen

        if (source == zurueckMenuItem) {
            coordSys.RFeld(0);
            coordSys.Hoehenlinien(false);
            SpezBox.setSelected(false);
            HoeheBox.setSelected(false);
            xTextfeld.setText("1,00");
            yTextfeld.setText("1,00");
            xTextfeld.lastValid=1.0;
            yTextfeld.lastValid=1.0;
            group.clearSelection();
            LoesungIcon = iconL0;
            LoesungLabel.setIcon(LoesungIcon);
        }
    }

    // Gre gendert? In diesem Fall Anordnung der Komponenten

    public void componentResized(ComponentEvent event) {

        int width = this.getWidth();
        int height = this.getHeight();

        // Gre und Position der einzelnen Elemente ndern
        // 4 Pixel Rand beim Koordinatensystem
        // Koordinatensystem Standard 700x500
        // Applet nimmt am Rand 16x59 weg!

        HTWLabel.setBounds(10, 11, 210, 90);
        DGLButton1.setBounds(235, 10, 90, 44);
        DGLButton2.setBounds(330, 10, 90, 44);
        DGLButton3.setBounds(425, 10, 90, 44);
        DGLButton4.setBounds(520, 10, 90, 44);
        DGLButton5.setBounds(615, 10, 90, 44);
        DGLButton6.setBounds(710, 10, 95, 44);
        DGLButton7.setBounds(235, 59, 135, 44);
        DGLButton8.setBounds(375, 59, 110, 44);
        DGLButton9.setBounds(490, 59, 90, 44);
        DGLButton10.setBounds(585, 59, 90, 44);
        DGLButton11.setBounds(680, 59, 125, 44);

        coordPanel.setBounds(120, 115, width-130, height-168);

        TrennLabel.setBounds(10, 115, 100, 2);
        HoeheLabel.setBounds(8, 200, 100, 20);
        HoeheLabel1.setBounds(8, 217, 100, 20);
        HoeheBox.setBounds(90, 217, 20, 20);
        PfeilLabel.setBounds(30, 245, 60, 60);
        LoesungLabel.setBounds(-1, 310, 122, 90);


        SpezLabel.setBounds(8, 480, 100, 20);
        SpezLabel1.setBounds(8, 497, 100, 20);
        SpezBox.setBounds(90, 497, 20, 20);
        PunktLabel.setBounds(8, 530, 50, 20);
        xLabel.setBounds(60, 530, 50, 20);
        yLabel.setBounds(60, 555, 50, 20);
        xTextfeld.setBounds(72, 530, 35, 20);
        yTextfeld.setBounds(72, 555, 35, 20);

        // alles neu zeichnen
        SwingUtilities.updateComponentTreeUI(this);
    }

    @Override
    public void componentHidden(ComponentEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void componentMoved(ComponentEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void componentShown(ComponentEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseMoved(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }


    @Override
    public void keyReleased(KeyEvent arg0) {
        // TODO Auto-generated method stub

    }


    @Override
    public void keyTyped(KeyEvent arg0) {
        // TODO Auto-generated method stub

    }

}
