import java.awt.event.*;
import java.text.DecimalFormat;

import javax.swing.*;

// spezielles JTextField fr die Koordinaten

public class JAlphanumericTextField extends JTextField implements KeyListener, FocusListener {
    /**
     * Erfllt eigentlich keinen Zweck, aber Eclipse will,
     * da diese erstellt wird.
     */
    private static final long serialVersionUID = -7328168239454921791L;

    // Die letzte gltige Eingabe

    public double lastValid;

    private double min = 1.0;

    private double max = 1.0;

    DecimalFormat df = new DecimalFormat("0.00");

    // schreibt den Startwert in das Textfeld

    public JAlphanumericTextField(double startValue) {
        super();
        this.addKeyListener(this);
        this.addFocusListener(this);

        lastValid = startValue;

        setText(Double.valueOf(lastValid).toString());
    }

    /**
     * Ruft anderen Konstruktor auf, bernimmt dann Minimum
     * und Maximum. Beim Startwert wird die Einhaltung der
     * Grenzen nicht berprft.
     * @param startValue Startwert
     * @param minValue Minimaler Wert
     * @param maxValue Maximaler Wert
     */
    public JAlphanumericTextField(double startValue, double minValue, double maxValue) {
        this(startValue);

        min = minValue;
        max = maxValue;
    }

    // Gibt den Inhalt des TextFields als Double zurck
    //--> auerhalb der Grenzen --> Rckgabe  Maximum / Minimum

    public double get() {
        return lastValid;
    }

    // berprfung auf Einhaltung der Grenzen + Update des Feldes

    private void fixInput() {
        lastValid = Math.max(min, Math.min(max, lastValid));
        setText(df.format(Double.valueOf(lastValid)));
    }

    // Enter gedrckt? -> Eingabe korrigieren

    public void keyPressed(KeyEvent event) {
        int key = event.getKeyCode();

        if (key == KeyEvent.VK_ENTER) {
            fixInput();
        }
    }

    // Versuche Umwandlung in Double

    public void keyReleased(KeyEvent event) {
        try {
            String input = getText();
            input = input.replace(',', '.');
            lastValid = Double.valueOf(input).doubleValue();
        } catch(Exception e) {
        }
    }


    // Wenn TextField angewhlt: Ganze Zahl wird markiert

    public void focusGained(FocusEvent arg0) {
        setSelectionStart(0);
        setSelectionEnd(getText().length());
    }


    // TextField nicht mehr angewhlt/fokusiert: Eingabe korrigieren

    public void focusLost(FocusEvent arg0) {
        fixInput();
    }

    @Override
    public void keyTyped(KeyEvent event) {


    }

}
