
package source;

/**.
 * Klasse Skalierung
 * bernimmt die Berechnungen fr die Anzeige
 * der Cosinus-hyperbolicus-Funktion.
 * @author Denis Schneider
 */
public class SkalierungCosinusH {
    // Gre des Zeichenbereichs
    private int Width;
    private int Height;

    //Randbreite
    private int Rand = 20;

    // Breite der Grafik
    private double Breite = 4;

    // bestimmt ob die Darstellung abgeleitet ist
    private boolean isNormaled;

    // berechnete Werte fr Zeichenbereich
    private double von;
    private double bis;
    private double bereich_werte;
    private int bereich_grafik_X;
    private int bereich_grafik_Y;

    // Umfrechnungsfaktoren fr die Skalierung
    private double Skalierungs_Faktor_X;
    private double Skalierungs_Faktor_Y;
    private double Achsen_Verschiebung_X;

    private double ParameterA;
    private double ParameterB;
    private double ParameterC;

    // Nullpunktoffset
    private int Offset_X;
    private int Offset_Y;
    private int Offset_X1;

    // Referenz auf die Eingaben
    private ParameterEingabeCosinusH parameterEingabeCH = null;

    //Ableitungen
    private	double x0;
    private double f,f1,f2,f3;


    //Koeffizienten
    public double a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13,
           a14, a15, a16, a17, a18, a19, a20, a21, a22, a23, a24, a25;


    /**
     * Konstruktor
     * bernimmt die Parameter fr die Klasse
     * Referenz auf Eingaben und ob abgeleitete Darstellung
     *
     * @param PE 	Referenz auf das zugehrige Eingabefeld
     */
    public SkalierungCosinusH(ParameterEingabeCosinusH PE, boolean Normaled) {
        // Werte in den internen Variablen ablegen
        parameterEingabeCH = PE;
        isNormaled = Normaled;

        // Standardmig Skalierung = 0, so muss sie spter neu berechnet werden
        Skalierungs_Faktor_X = 0;
        Skalierungs_Faktor_Y = 0;
    }

    /**
     * Verlangt zwingend neue Berechnung, mit neuer Skalierung
     *
     * @param x	Breite des Zeichenbereiches
     * @param y	Hhe des Zeichenbereiches
     */
    public void ForceRechnen(int x, int y) {
        // Werte auf 0 setzen, damit sie neu berechnet werden
        Skalierungs_Faktor_X = 0;
        Skalierungs_Faktor_Y = 0;
        // jetzt neue Daten berechnen
        Rechnen(x, y);
    }

    /**
     * Berechnet neue interne Werte
     *
     * @param x	Breite des Zeichenbereiches
     * @param y	Hhe des Zeichenbereiches
     */
    public void Rechnen(int x, int y) {
        Width = x;
        Height = y;

        bereich_grafik_X = Width - Rand * 4 ;
        bereich_grafik_Y = Height - Rand * 4 ;

        // Unterscheidung bei abgeleiteter Darstellung
        if (!isNormaled) {
            // nicht abgeleitet
            // Funktionswerte bernehmen
            ParameterA = 1;//parameterEingabeCH.getParameterA_Wert();
            ParameterB = 1;//parameterEingabeCH.getParameterB_Wert();
            ParameterC = 0;//parameterEingabeCH.getParameterC_Wert();
        }
//		else
//			{
//			// Ableitung
//			ParameterA = parameterEingabeCH.getParameterA_Wert()*parameterEingabeCH.getParameterB_Wert();
//			ParameterB = parameterEingabeCH.getParameterB_Wert();
//			ParameterC = parameterEingabeCH.getParameterC_Wert();
//			}

        // die Y-Achse ist um den Mittelwert verschoben, so steht er immer in der Mitte
        Achsen_Verschiebung_X = 0;

        // Breite links und rechts vom Mittelwert ergeben Zeichengrenzen
        von = - Breite;
        bis = + Breite;

        // Bereich ausrechnen
        bereich_werte = (bis - von);

        // X-Skalierung neu berechnen
//		if (Skalierungs_Faktor_X==0)
        Skalierungs_Faktor_X = bereich_grafik_X / bereich_werte ;

        // Y-Skalierung neu berechnen
//		if ( Skalierungs_Faktor_Y==0)
        Skalierungs_Faktor_Y = bereich_grafik_Y /2/3;

        // Die Offsets geben den Abstand zum Ursprung an
        Offset_X = Width / 2;
        Offset_X1 = 0 + Rand*2;
        Offset_Y = Height/2 +Rand /2 ;
    }

    /**
     * liefert die Breite
     *
     * @return Breites des Zeichenbereiches in Pixel
     */
    public int getWidth() {
        return Width;
    }

    /**
     * liefert die Hhe
     *
     * @return Hhe des Zeichenbereiches in Pixel
     */
    public int getHeight() {
        return Height;
    }

    /**
     * liefert Position der Y-Achse
     *
     * @return X-Position der Y-Achse in Pixel
     */
    public int getPosYAchse() {
        // Position berechnen
        int Interner_Wert = (int) (Offset_X + Achsen_Verschiebung_X * Skalierungs_Faktor_X);
        // prfen ob ausserhalb und bei Bedarf begrenzen
        if (Interner_Wert > Width-3*Rand) Interner_Wert = Width-3*Rand;
        if (Interner_Wert < 3*Rand) Interner_Wert = 3*Rand;

        return Interner_Wert;
    }

    /**
     * Zeigt an ob Y-Achse versetzt
     *
     * @return 0 wenn nicht versetzt, -1 wenn links versetzt, 1 wenn rechts versetzt
     */
    public int getIfVersetzteYAchse() {
        // Position berechnen
        int Interner_Wert = (int) (Offset_X + Achsen_Verschiebung_X * Skalierungs_Faktor_X);
        // prfen ob ausserhalb und bei Bedarf begrenzen
        if (Interner_Wert > Width-3*Rand) return 1;
        if (Interner_Wert < 3*Rand) return -1;
        // sonst Standardrckgabe, als Kennzeichung fr nicht verschoben
        return 0;
    }

    /**
     * liefert Position der X-Achse
     *
     * @return Y-Position der X-Achse in Pixel
     */
    public int getPosXAchse() {
        return Offset_Y;
    }

    /**
     * liefert die Randbreite
     *
     * @return Breite des Randes in Pixel
     */
    public int getRand() {
        return Rand;
    }

    /**
     * liefert den Abstand der Beschriftungen auf der Achse
     * abhngig vom verfgbaren Platz
     *
     * @return Abstand der Beschriftungen auf der X Achse
     */
    public double getBeschriftungAbstandX() {
        double returnWert;

        returnWert = 35.0/Skalierungs_Faktor_X;

        if (returnWert <= 0.001) return  0.001;
        if (returnWert <= 0.002) return  0.002;
        if (returnWert <= 0.005) return  0.005;
        if (returnWert <= 0.01) return  0.01;
        if (returnWert <= 0.02) return  0.02;
        if (returnWert <= 0.05) return  0.05;
        if (returnWert <= 0.1) return  0.1;
        if (returnWert <= 0.2) return  0.2;
        if (returnWert <= 0.5) return  0.5;
        if (returnWert <= 1) return  1;
        if (returnWert <= 2) return  2;
        if (returnWert <= 5) return  5;
        if (returnWert <= 10) return  10;
        if (returnWert <= 20) return  20;
        if (returnWert <= 50) return  50;

        return returnWert;
    }
    public double getBeschriftungAbstandX1() {
        double returnWert;

        returnWert = 25.0/Skalierungs_Faktor_X;

        if (returnWert <= 0.001) return  0.001;
        if (returnWert <= 0.002) return  0.002;
        if (returnWert <= 0.005) return  0.005;
        if (returnWert <= 0.01) return  0.01;
        if (returnWert <= 0.02) return  0.02;
        if (returnWert <= 0.05) return  0.05;
        if (returnWert <= 0.1) return  0.1;
        if (returnWert <= 0.2) return  0.31;
        if (returnWert <= 0.5) return  0.5;
        if (returnWert <= 1) return  1;
        if (returnWert <= 2) return  2;
        if (returnWert <= 5) return  5;
        if (returnWert <= 10) return  10;
        if (returnWert <= 20) return  20;
        if (returnWert <= 50) return  50;

        return returnWert;
    }

    /**
     * liefert den Abstand der Beschriftungen auf der Achse
     * abhngig vom verfgbaren Platz
     *
     * @return Abstand der Beschriftungen auf der Y Achse
     */
    public double getBeschriftungAbstandY() {
        double int_wert = 18/Skalierungs_Faktor_Y;
        if (int_wert <= 0.001) return 0.001;
        if (int_wert <= 0.002) return 0.002;
        if (int_wert <= 0.005) return 0.005;
        if (int_wert <= 0.01) return 0.01;
        if (int_wert <= 0.02) return 0.02;
        if (int_wert <= 0.05) return 0.05;
        if (int_wert <= 0.10) return 0.10;
        if (int_wert <= 0.20) return 0.20;
        if (int_wert <= 0.50) return 0.50;
        if (int_wert <= 1.00) return 1.00;
        if (int_wert <= 2.00) return 2.00;
        if (int_wert <= 5.00) return 5.00;
        if (int_wert <= 10.00) return 10.00;

        return int_wert;
    }

//	public double getBeschriftung_X1()
//	{
//		double wert=parameterEingabeCH.getAK();
//		//System.out.println(wert);
//		if (wert == 1) return 0;
//		else if (wert == 2) return this.getBeschriftungAbstandX1();
//		else if (wert == 3) return 2*this.getBeschriftungAbstandX1();
//		else if (wert == 4) return 3*this.getBeschriftungAbstandX1();
//		else if (wert == 5) return 4*this.getBeschriftungAbstandX1();
//		else if (wert == 6) return 5*this.getBeschriftungAbstandX1();
//		else if (wert == 7) return 6*this.getBeschriftungAbstandX1();
//		else if (wert == 8) return 7*this.getBeschriftungAbstandX1();
//		else if (wert == 9) return 8*this.getBeschriftungAbstandX1();
//		else if (wert == 10) return 9*this.getBeschriftungAbstandX1();
//		else if (wert == 11) return 10*this.getBeschriftungAbstandX1();
//		else if (wert == 12) return 11*this.getBeschriftungAbstandX1();
//		else if (wert == 13) return 12*this.getBeschriftungAbstandX1();
//		else if (wert == 14) return 13*this.getBeschriftungAbstandX1();
//		else if (wert == 15) return 14*this.getBeschriftungAbstandX1();
//		else if (wert == 16) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 17) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 18) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 19) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 20) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 21) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 22) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 23) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 24) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//		else if (wert == 25) return 14*this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1()+this.getBeschriftungAbstandX1();
//
//		return wert;
//	}

    public double getBeschriftung_X1() {
        double wert=parameterEingabeCH.getAnzahlKoeff();

        switch((int)wert) {
        case 0:
            return 0;

        case 1:
            return this.getBeschriftungAbstandX1();

        case 2:
            return 2*this.getBeschriftungAbstandX1();

        case 3:
            return 3*this.getBeschriftungAbstandX1();

        case 4:
            return 4*this.getBeschriftungAbstandX1();

        case 5:
            return 5*this.getBeschriftungAbstandX1();

        case 6:
            return 6*this.getBeschriftungAbstandX1();

        case 7:
            return 7*this.getBeschriftungAbstandX1();

        case 8:
            return 8*this.getBeschriftungAbstandX1();

        case 9:
            return 9*this.getBeschriftungAbstandX1();

        case 10:
            return 10*this.getBeschriftungAbstandX1();

        case 11:
            return 11*this.getBeschriftungAbstandX1();

        case 12:
            return 12*this.getBeschriftungAbstandX1();

        case 13:
            return 13*this.getBeschriftungAbstandX1();

        case 14:
            return 14*this.getBeschriftungAbstandX1();
        }

        return wert;

    }

    /**
     * Liefert den Skalierungsfaktor in X Richtung
     *
     * @return Skalierungsfaktor in X Richtung
     */
    public double getSkalierungs_Faktor_X() {
        return Skalierungs_Faktor_X;
    }

    /**
     * Liefert den Skalierungsfaktor in Y Richtung
     *
     * @return Skalierungsfaktor in Y Richtung
     */
    public double getSkalierungs_Faktor_Y() {
        return Skalierungs_Faktor_Y;
    }

    /**
     * liefert die linke Begrenzung
     *
     * @return Position (Untergrenze) ab der die Kurve dargestellt wird
     */
    public double getVon() {
        return von;
    }

    /**
     * liefert die rechte Begrenzung
     *
     * @return Position (Obergrenze) ab der der Text dargestellt wird
     */
    public double getTextBis() {
        return bis;
    }

    /**
     * liefert die linke TextBegrenzung
     *
     * @return Position (Untergrenze) ab der der Text dargestellt wird
     */
    public double getTextVon() {
        return von;
    }

    /**
     * liefert die linke Begrenzung
     *
     * @return Position (Untergrenze) ab der die Kurve dargestellt wird
     */
    public double getBis() {
        return bis;
    }

    /**
     * liefert die Y-Textposition zum Y Wert
     *
     * @param value		gewnschte Position des Textes
     *
     * @return zugehriger skalierter Y Wert
     */
    public double getYAchseTextY(double value) {
        return Offset_Y -value* Skalierungs_Faktor_Y;
    }

    /**
     * liefert die X-Textposition zum X Wert
     *
     * @param value		gewnschte Position des Textes
     *
     * @return zugehriger skalierter X Wert
     */
    public double getXAchseTextX(double value) {
        return Offset_X +(value+Achsen_Verschiebung_X)* Skalierungs_Faktor_X;
    }

    /**
     * liefert die X-Textposition zum X Wert
     *
     * @param value		gewnschte Position des Textes
     *
     * @return zugehriger skalierter X Wert
     */
    public double getXAchseTextX1(double value) {
        return Offset_X1 +(value+Achsen_Verschiebung_X)* Skalierungs_Faktor_X;
    }

    /**
     * liefert den X Cosinus Wert
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter X Wert
     */
    public double getCosinusWertX(double value) {
        return Offset_X + (value) * Skalierungs_Faktor_X;
    }

    /**
     * liefert den Y Cosinus-hyperbolikus Wert
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double getCosinushWertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - ParameterA*Math.cosh(ParameterB*value+ParameterC) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Cosinus-hyperbolikus Wert
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double getCosinushAWertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - ParameterA*Math.sinh(ParameterB*value+ParameterC) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }


    /**
     * liefert den x-Abgriff y-Wert
     *
     * @param value		gewnschte Steigung der Tangente (mathematisch)
     *
     * @return zugehriger skalierter y-Wert (Zeichenbereich)
     */
    public double getSteigung(double value) {
        double Steigung;
        Steigung = ParameterA*Math.sinh(ParameterB*value+ParameterC);
        return Steigung;
    }

    /**
     * liefert Realen Funktionswert zu einem Wert in der Darstellung
     *
     * @param x 	Wert in der Darstellung
     *
     * @return realer (mathematischer) Funktionswert
     */
    public double getRealYWert(double x) {
        double hilf = ((double)Offset_Y - x) / Skalierungs_Faktor_Y;
        return hilf ;
    }

    /**
     * liefert Realen Funktionswert zu einem Wert in der Darstellung
     *
     * @param x 	Wert in der Darstellung
     *
     * @return realer (mathematischer) Funktionswert
     */
    public double getRealXWert(int x) {
        double hilf = ((double)x - (double)Offset_X) / Skalierungs_Faktor_X - Achsen_Verschiebung_X;
        return hilf ;
    }

    /**
     * liefert den skalierten Wert eines x-Wertes
     *
     * @param x
     *
     * @return skalierter x-Wert
     */
    public double getSkalXWert (double x) {
        double hilf = (x + Achsen_Verschiebung_X - (Skalierungs_Faktor_X/(double)Offset_X)/20);
        return hilf;
    }


    public double geta0WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom zweiten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta1WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+this.a1*value) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom dritten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta2WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value) + (this.a2*value*value))* Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom vierten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta3WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value) + (this.a2*value*value)
                                     + (this.a3*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom fnften Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta4WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value)+(this.a4*value*value*value*value))* Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom sechsten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta5WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom siebten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta6WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom achten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta7WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value)
                                     + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom neunten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta8WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom zehnten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta9WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom elften Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta10WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)
                                     + (this.a10*value*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom zwlften Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta11WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)
                                     + (this.a10*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a11*value*value*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom dreizehnten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta12WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)
                                     + (this.a10*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a11*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a12*value*value*value*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom vierzehnten Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta13WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)
                                     + (this.a10*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a11*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a12*value*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a13*value*value*value*value*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**
     * liefert den Y Polynom Wert vom fnfzehntnen Koeffizienten
     *
     * @param value		gewnschte Position der Kurve (mathematisch)
     *
     * @return zugehriger skalierter Y Wert (Zeichenbereich)
     */
    public double geta14WertY(double value) {
        double return_wert;
        return_wert =  1*Offset_Y - (this.a0+(this.a1*value)+(this.a2*value*value)
                                     + (this.a3*value*value*value) + (this.a4*value*value*value*value)
                                     + (this.a5*value*value*value*value*value)
                                     + (this.a6*value*value*value*value*value*value)
                                     + (this.a7*value*value*value*value*value*value*value)
                                     + (this.a8*value*value*value*value*value*value*value*value)
                                     + (this.a9*value*value*value*value*value*value*value*value*value)
                                     + (this.a10*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a11*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a12*value*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a13*value*value*value*value*value*value*value*value*value*value*value*value*value)
                                     + (this.a14*value*value*value*value*value*value*value*value*value*value*value*value*value*value)) * Skalierungs_Faktor_Y; //
        if (return_wert < Rand) return_wert = -1;
        return return_wert;
    }

    /**berechnet die Werte
     * der Koeffizienten der Potenzreihe
     */

    public void KoeffRechnen() {
        x0 = parameterEingabeCH.getAbgriff();

        f = Math.cosh(x0);
        f1 = Math.sinh(x0);
        f2 = f;
        f3 = f1;

        a0 = (f);
        if (a0==-0)
            a0 = 0;

        a1 = (f1/(1.0));
        if (a1==-0)
            a1 = 0;

        a2 = (f2/(1.0*2.0));
        if (a2==-0)
            a2 = 0;

        a3 = (f3/(1.0*2.0*3.0));
        if (a3==-0)
            a3 = 0;

        a4 = (f/(1.0*2.0*3.0*4.0));
        if (a4==-0)
            a4 = 0;

        a5 = (f1/(1.0*2.0*3.0*4.0*5.0));
        if (a5==-0)
            a5 = 0;

        a6 = (f2/(1.0*2.0*3.0*4.0*5.0*6.0));
        if (a6==-0)
            a6 = 0;

        a7 = (f3/(1.0*2.0*3.0*4.0*5.0*6.0*7.0));
        if (a7==-0)
            a7 = 0;

        a8 = (f/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0));
        if (a8==-0)
            a8 = 0;

        a9 = (f1/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0));
        if (a9==-0)
            a9 = 0;

        a10 = (f2/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0*10.0));
        if (a10==-0)
            a10 = 0;

        a11 = (f3/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0*10.0*11.0));
        if (a11==-0)
            a11 = 0;

        a12 = (f/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0*10.0*11.0*12.0));
        if (a12==-0)
            a12 = 0;

        a13 = (f1/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0*10.0*11.0*12.0*13.0));
        if (a13==-0)
            a13 = 0;

        a14 = (f2/(1.0*2.0*3.0*4.0*5.0*6.0*7.0*8.0*9.0*10.0*11.0*12.0*13.0*14.0));
        if (a14==-0)
            a14 = 0;

//	a15 = (f3/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15));
//	a16 = (f/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16));
//	a17 = (f1/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17));
//	a18 = (f2/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18));
//	a19 = (f3/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19));
//	a20 = (f/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19*20));
//	a21 = (f1/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19*20*21));
//	a22 = (f2/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19*20*21*22));
//	a23 = (f3/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19*20*21*22*23));
//	a24 = (f/(1*2*3*4*5*6*7*8*9*10*11*12*13*14*15*16*17*18*19*20*21*22*23*24));
    }
}
