
package source;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.GeneralPath;
import java.text.DecimalFormat;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import javax.imageio.ImageIO;
import java.io.IOException;

/**
 * Klasse Zeichenbereich
 * Erzeugt den eigentlichen Zeichenbereich auf dem die Grafik dargestellt wird
 * @author Denis Schneider
 */
public class Zeichenbereich1X extends JPanel {
    /**
     * automatisch generierte UID
     */
    private static final long serialVersionUID = 1L;
    // Pfeilgren
    private int Pfeil_laenge = 12;
    private int Pfeil_breite = 3;

    // Referenz auf die Eingabeparameter
    private ParameterEingabe1X parameterEingabe1X = null;

    // fr das Verschieben der Vertrauensbereiche
    private double Klick_Tolleranz = 0.05;
    // erlaubt Verschieben
    private boolean CanDrag = false;

    // Referenz auf die Skalierungsklasse
    private Skalierung1X skalierung = null;

    // bestimmt ob dieser Zeichenbereich normiert dargestellt werden soll
    private boolean IsNormaled = false;

    //Variablen
    double value,a,x0;
    int x = 0;

    private Image Formel2;

    /**
     * Konstruktor
     * erzeugt neuen Zeichenbereich
     *
     * @param w 	Referenz auf die Eingaben
     */
    public Zeichenbereich1X(ParameterEingabe1X w, boolean isNormaled) {
        // Parameter bernehmen und sichern
		
		try{
			Formel2 = ImageIO.read(Ableitung.class.getResourceAsStream("/images/1x.PNG"));
		}catch(IOException ioe){
		}
		
        parameterEingabe1X = w;
        parameterEingabe1X.setZeichenbereich(this);
        IsNormaled = isNormaled;

        // Mausaktionen verwenden
        addMouseListener(new MyMouseListener());
        addMouseMotionListener(new MyMouseMotionListener());

        // neue Skalierungsberechnung erzeugen
        skalierung = new Skalierung1X(parameterEingabe1X, IsNormaled);
    }

    /**
     * lscht den Zeichenbereich
     * und gibt interne Referenzen frei
     */
    public void delete() {
        parameterEingabe1X = null;
        skalierung = null;
    }

    /**
     * veranlasst ein neuzeichnen der Visualisierung
     */
    public void Dorepaint() {
        this.repaint();
    }

    /**
     * erzwingt eine Neuberechnung der Skalierung
     */
    public void DoRecalc() {
        skalierung.ForceRechnen(getWidth(), getHeight());
        this.repaint();
    }

    /**
     * Neuzeichnen der Grafik erforderlich
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    protected void paintComponent( Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // bergeordnetes Zeichen aktivieren
        super.paintComponent(g);
        // Skalierung aktualisieren
        skalierung.Rechnen(getWidth(), getHeight());
        skalierung.KoeffRechnen();
        parameterEingabe1X.KoeffizientenAusgeben();
        // einzelne Teile neu zeichnen
        PrintBackground(g);
        PrintKoeffizienten(g);
        PrintAchsen(g);
        PrintKonvergenzbereich (g);
        PrintAbgriff(g);
        PrintPolynomKurve(g);
        PrintNaeherung (g);

        //Formel im oberen Zeichenbereich einblenden
        if (!IsNormaled) {
            g2d.drawImage(Formel2, 3*skalierung.getRand()/2, 3*skalierung.getRand()/2, this); //

        }
    }

    /**
     * Zeichnet x-Abgriff
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintAbgriff (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        // Abgriff blau darstellen
        g2d.setColor(Color.BLUE);
        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        double lauf;
        int Punkt_x, Punkt_y;

        // x-Abgriff holen
        lauf=parameterEingabe1X.getAbgriff();
        // Umrechnen in Grafikkoordinaten
        Punkt_x = (int) skalierung.getFunkWertX(lauf);
        Punkt_y = (int) skalierung.getFunkWertY(lauf);

        if (!IsNormaled) {
            g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
            if (Punkt_y-8<(skalierung.getHeight()-skalierung.getRand()/1.1)&& skalierung.getRand()<=Punkt_y-8) {
                if (skalierung.a0 >=0) {
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, skalierung.getPosXAchse()+ skalierung.getHeight()/2-skalierung.getRand()-10);
                } else {
                    g.drawLine(Punkt_x, skalierung.getPosXAchse()- skalierung.getHeight()/2 + skalierung.getRand()/2, Punkt_x, Punkt_y);
                }
            } else {
                g.drawLine(Punkt_x,(int)( skalierung.getHeight()-20), Punkt_x, skalierung.getRand());
            }
            g2d.setStroke(OldStoke);
        }

        // alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }

    /**
     * zeichnet auf der X-Achse
     * den Korvegenzbereich der 1/X-Funktion
     *
     * @param g
     */
    private void PrintKonvergenzbereich(Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        // Linie lila darstellen
        g2d.setColor(Color.MAGENTA);
        g2d.setStroke(new BasicStroke(2,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

        x0 = parameterEingabe1X.getAbgriff();


        if (!IsNormaled) {
            if (x0 > 0) {
                if (2*x0 <= skalierung.getBis()) {
                    g.drawLine((int)skalierung.getXAchseTextX(0)+2, (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(0)+2, (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(0), (int)(1*skalierung.getPosXAchse()), (int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()));
                } else {
                    g.drawLine((int)skalierung.getXAchseTextX(0)+1, (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(0)+1, (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(0), (int)(1*skalierung.getPosXAchse()), (int)skalierung.getXAchseTextX(4), (int)(1*skalierung.getPosXAchse()));
                }
                //Beschriftung
                if(x0 >= 2.0) {
                    g.drawString("Konvergenzbereich",(int)skalierung.getXAchseTextX(0.5),(int)(1*skalierung.getPosXAchse()+skalierung.getRand()*1.5));
                } else {
                    g.drawString("Konvergenzbereich",(int)skalierung.getXAchseTextX(x0)+skalierung.getRand(),(int)(1*skalierung.getPosXAchse()+skalierung.getRand()*1.5));
                }
            } else if (x0 < 0) {
                if (2*x0 >= skalierung.getVon()) {
                    g.drawLine((int)skalierung.getXAchseTextX(0)-1, (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(0)-1, (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(0), (int)(1*skalierung.getPosXAchse()), (int)skalierung.getXAchseTextX(2*x0), (int)(1*skalierung.getPosXAchse()));
                } else {
                    g.drawLine((int)skalierung.getXAchseTextX(0)-1, (int)(1*skalierung.getPosXAchse()-3), (int)skalierung.getXAchseTextX(0)-1, (int)(1*skalierung.getPosXAchse()+3));
                    g.drawLine((int)skalierung.getXAchseTextX(0), (int)(1*skalierung.getPosXAchse()), (int)skalierung.getXAchseTextX(-4), (int)(1*skalierung.getPosXAchse()));
                }
                //Beschriftung
                if(x0 <= -2.0) {
                    g.drawString("Konvergenzbereich",(int)skalierung.getXAchseTextX(-1.8),(int)(1*skalierung.getPosXAchse()-skalierung.getRand()));
                } else {
                    g.drawString("Konvergenzbereich",(int)skalierung.getXAchseTextX(x0)-skalierung.getRand()*6,(int)(1*skalierung.getPosXAchse()-skalierung.getRand()));
                }
            }
        }
        //alten Stift wieder einstellen
        g2d.setStroke(OldStoke);
    }



    /** zeichnet die Nherungskurve
     *
     * abhngig von der gewhlten Potenz
     *
     * @param g
     */

    private void PrintNaeherung(Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // dafr einen anderen Stift verwenden
        Stroke OldStoke=g2d.getStroke();

        // Naeherung grn darstellen
        g2d.setColor(Color.GREEN);
        g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

        x0 = parameterEingabe1X.getAbgriff();

        // nur wenn x0 <=-0.4 oder >= 0.4 zeichnen
        if(x0 <= -0.4 || x0 >= 0.4) {

            if(parameterEingabe1X.getAnzahlKoeff()==0) {

                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.4d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta0WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==1) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta1WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==2) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta2WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }

                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==3) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta3WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }

                }
            } else if(parameterEingabe1X.getAnzahlKoeff()==4) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta4WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==5) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta5WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==6) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta6WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==7) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta7WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }


            else if(parameterEingabe1X.getAnzahlKoeff()==8) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta8WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==9) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta9WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==10) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta10WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==11) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta11WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==12) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta12WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==13) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta13WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }

            else if(parameterEingabe1X.getAnzahlKoeff()==14) {
                if (!IsNormaled) {

                    // Sichtbaren Bereich durchlaufen
                    for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.5d/skalierung.getSkalierungs_Faktor_X()) {
                        // Koordinaten umrechnen
                        int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                        int Punkt_y = (int) skalierung.geta14WertY(lauf-x0);
                        if (Punkt_y>0 && Punkt_y<( skalierung.getHeight()-skalierung.getRand()))
                            g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
                    }
                }
            }
        }
        g2d.setStroke(OldStoke);
    }


    /**
     * Zeichnet den eigentlichen Verlauf der Kurve
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintPolynomKurve (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        if (!IsNormaled) {	// Mit Rot Zeichnen
            g2d.setColor(Color.RED);
            g2d.setStroke(new BasicStroke(3,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

            for (double lauf=skalierung.getVon(); lauf < skalierung.getBis(); lauf+=0.4d/skalierung.getSkalierungs_Faktor_X()) {
                // Koordinaten umrechnen
                int Punkt_x = (int) skalierung.getFunkWertX(lauf);
                int Punkt_y = (int) skalierung.getFunkWertY(lauf);
                if (Punkt_y>0 && Punkt_y <(skalierung.getHeight()-skalierung.getRand()))
                    g.drawLine(Punkt_x, Punkt_y, Punkt_x, Punkt_y);
            }
        }
    }

    /**
     * Zeichnet die Entwicklung der Koeffizienten
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintKoeffizienten(Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;
        GeneralPath p = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
        Font Old_font = g.getFont();
        g2d.setColor(Color.GREEN);

        // Im zweiten Zeicehbereich zeichnen
        if(IsNormaled) {
            for (double lauf = 0; lauf <= skalierung.getBeschriftung_X1();  lauf +=skalierung.getBeschriftungAbstandX1()) {
                if (lauf == 0) {
                    a = skalierung.a0;
                } else if (lauf < (2*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a1;
                } else if (lauf < (3*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a2;
                } else if (lauf < (4*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a3;
                } else if (lauf < (5*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a4;
                } else if (lauf < (6*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a5;
                } else if (lauf < (7*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a6;
                } else if (lauf < (8*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a7;
                } else if (lauf < (9*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a8;
                } else if (lauf < (10*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a9;
                } else if (lauf < (11*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a10;
                } else if (lauf < (12*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a11;
                } else if (lauf < (13*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a12;
                } else if (lauf < (14*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a13;
                } else if (lauf < (15*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    a = skalierung.a14;
                }
//			else if (lauf < (16*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 16;
//				a = skalierung.geta16();}
//			else if (lauf < (17*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 17;
//				a = skalierung.geta17();}
//			else if (lauf < (18*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 18;
//				a = skalierung.geta18();}
//			else if (lauf < (19*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 19;
//				a = skalierung.geta19();}
//			else if (lauf < (20*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 20;
//				a = skalierung.geta20();}
//			else if (lauf < (21*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 21;
//				a = skalierung.geta21();}
//			else if (lauf < (22*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 22;
//				a = skalierung.geta22();}
//			else if (lauf < (23*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 23;
//				a = skalierung.geta23();}
//			else if (lauf < (24*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 24;
//				a = skalierung.geta24();}
//			else if (lauf < (25*skalierung.getBeschriftungAbstandX1()-0.1)){
//				x = 25;
//				a = skalierung.geta25();}

                // X - Koordinate fr die Koeffizienten
                int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX1(lauf))+20;

                if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2.5*skalierung.getRand()) {
                    //Koeffizienten zeichen
                    if(parameterEingabe1X.getAbgriff() <= -0.4 || parameterEingabe1X.getAbgriff() >= 0.4) {
                        if(a<=-0.05 || a>=0.05) {
                            p.moveTo ((float)(Punkt_x_Markierung-10), (float)(1*skalierung.getPosXAchse()));
                            p.lineTo((float)(Punkt_x_Markierung-10), (float)(1*skalierung.getPosXAchse()-a*skalierung.getSkalierungs_Faktor_Y()));
                            p.lineTo((float)(Punkt_x_Markierung+10), (float)(1*skalierung.getPosXAchse()-a*skalierung.getSkalierungs_Faktor_Y()));
                            p.lineTo((float)(Punkt_x_Markierung+10), (float)(1*skalierung.getPosXAchse()));
                            p.closePath();
                            g2d.draw(p);
                            g2d.fill(p);
                        }
                    }
                    //Fehlermeldung anzeigen
                    else {
                        Font font = new Font ("Verdana",Font.PLAIN, 15);
                        g.setFont(font);
                        g2d.setColor(Color.BLUE);
                        g.drawString("Koeffizienten und Taylorpolynom", skalierung.getPosYAchse()-skalierung.getWidth()/6, skalierung.getRand()*4);
                        g.drawString("hier nicht sinnvoll darstellbar", skalierung.getPosYAchse()-skalierung.getWidth()/7, skalierung.getRand()*5);
                    }
                }
            }
        }
        g.setFont(Old_font);
    }

    /**
     * Zeichnet die Achsen
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintAchsen (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Falls die Y-Achse nicht durch x=0 geht, dies mit Zeichen deutlichmachen
        if (skalierung.getIfVersetzteYAchse()<0) {
            g2d.setColor(Color.BLACK);
            g.drawString("//", (int)(3.01*skalierung.getRand()), skalierung.getPosXAchse()+5);
        }
        if (skalierung.getIfVersetzteYAchse()>0) {
            g2d.setColor(Color.BLACK);
            g.drawString("//", (int)(skalierung.getWidth()-3.4*skalierung.getRand()), skalierung.getPosXAchse()+5);
        }

        // X-Achse
        g2d.setColor(Color.BLACK);
        g.drawLine(2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()), skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()));

        // Y-Achse
        if(!IsNormaled) {
            g.drawLine(skalierung.getPosYAchse(), 2*skalierung.getRand(), skalierung.getPosYAchse(), skalierung.getHeight()-2*skalierung.getRand());
        } else {
            g.drawLine(skalierung.getPosYAchse()-skalierung.getWidth()/2+40, 2*skalierung.getRand(), skalierung.getPosYAchse()-skalierung.getWidth()/2+40, skalierung.getHeight()-2*skalierung.getRand());
        }
        // Pfeile an die Achsen
        for (int i=Pfeil_laenge; i>=0; i--) {
            // Oben
            if(!IsNormaled) {
                g.drawLine(skalierung.getPosYAchse() - Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);
                g.drawLine(skalierung.getPosYAchse() + Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse(), 2*skalierung.getRand()+i-Pfeil_laenge);
            } else {
                g.drawLine(skalierung.getPosYAchse()-skalierung.getWidth()/2+40 - Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse()-skalierung.getWidth()/2+40, 2*skalierung.getRand()+i-Pfeil_laenge);
                g.drawLine(skalierung.getPosYAchse()-skalierung.getWidth()/2+40 + Pfeil_breite, 2*skalierung.getRand(), skalierung.getPosYAchse()-skalierung.getWidth()/2+40, 2*skalierung.getRand()+i-Pfeil_laenge);
            }

            //Rechts
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()-Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
            g.drawLine(skalierung.getWidth()-2*skalierung.getRand(), (int)(1*skalierung.getPosXAchse()+Pfeil_breite), skalierung.getWidth()-2*skalierung.getRand()-i+Pfeil_laenge, (int)(1*skalierung.getPosXAchse()));
        }

        // Beschriftungen
        Font Old_font = g.getFont();
        Font font = new Font ("Arial", Font.PLAIN, 13);
        g.setFont(font);
        // X - Achse
        if (IsNormaled)
            g.drawString("Potenz n", (int)skalierung.getWidth()-2*skalierung.getRand()-30, (int)(1*skalierung.getPosXAchse()+13));
        else
            g.drawString("x", skalierung.getWidth()-2*skalierung.getRand()+5, (int)(1*skalierung.getPosXAchse()+13));

        // Y - Achse
        if (IsNormaled) {
            g.drawString("a", skalierung.getPosYAchse()-skalierung.getWidth()/2+20, skalierung.getRand()*2-3);
            g.drawString("n", skalierung.getPosYAchse()-skalierung.getWidth()/2+27, skalierung.getRand()*2);
        } else
            g.drawString("f(x)", skalierung.getPosYAchse()-28, skalierung.getRand()*2);

        g.setFont(Old_font);

        // Achsenbeschriftung
        // X - Achsen Beschriftung von 0 .. oo
        if(!IsNormaled) {
            for (double lauf=skalierung.getBeschriftungAbstandX(); lauf <= skalierung.getTextBis()  ; lauf+=skalierung.getBeschriftungAbstandX()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );

                if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
                if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
                if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
                int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height ) ;

                // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

                if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2.5*skalierung.getRand()) {
                    // Text ausgeben
                    g.drawString(Beschriftung , Punkt_x_Text, (int)(1.0*Punkt_y_Text));

                    // Kleine Markierungen anbringen, wo der Text steht
                    g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
                }
            }
        } else {
            for (double lauf = 0; lauf <= skalierung.getBeschriftung_X1();  lauf +=skalierung.getBeschriftungAbstandX1()) {
                if (lauf == 0) {
                    x = 0;
                } else if (lauf < (2*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 1;
                } else if (lauf < (3*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 2;
                } else if (lauf < (4*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 3;
                } else if (lauf < (5*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 4;
                } else if (lauf < (6*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 5;
                } else if (lauf < (7*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 6;
                } else if (lauf < (8*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 7;
                } else if (lauf < (9*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 8;
                } else if (lauf < (10*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 9;
                } else if (lauf < (11*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 10;
                } else if (lauf < (12*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 11;
                } else if (lauf < (13*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 12;
                } else if (lauf < (14*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 13;
                } else if (lauf < (15*skalierung.getBeschriftungAbstandX1()-0.1)) {
                    x = 14;
                }
//				else if (lauf < (16*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 16;
//					a = skalierung.geta16();}
//				else if (lauf < (17*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 17;
//					a = skalierung.geta17();}
//				else if (lauf < (18*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 18;
//					a = skalierung.geta18();}
//				else if (lauf < (19*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 19;
//					a = skalierung.geta19();}
//				else if (lauf < (20*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 20;
//					a = skalierung.geta20();}
//				else if (lauf < (21*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 21;
//					a = skalierung.geta21();}
//				else if (lauf < (22*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 22;
//					a = skalierung.geta22();}
//				else if (lauf < (23*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 23;
//					a = skalierung.geta23();}
//				else if (lauf < (24*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 24;
//					a = skalierung.geta24();}
//				else if (lauf < (25*skalierung.getBeschriftungAbstandX1()-0.1)){
//					x = 25;
//					a = skalierung.geta25();}

                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0" );
                String Beschriftung = df.format(x);
                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getXAchseTextX1(lauf) - (double)Beschriftung_Width/2d)+20;
                int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height    ) ;

                // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX1(lauf))+20;

                if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2.5*skalierung.getRand()) {
                    // Text ausgeben
                    g.drawString(Beschriftung , Punkt_x_Text, (int)(1.0*Punkt_y_Text));

                    // Kleine Markierungen anbringen, wo der Text steht
                    g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
                }
            }
        }


        // X - Achsen Beschriftung von -oo .. 0
        if(!IsNormaled) {
            for (double lauf=-skalierung.getBeschriftungAbstandX(); lauf > skalierung.getTextVon()  ; lauf-=skalierung.getBeschriftungAbstandX()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );

                if (skalierung.getBeschriftungAbstandX() >= 10) df = new DecimalFormat( "0" );
                if (skalierung.getBeschriftungAbstandX() <= 1) df = new DecimalFormat( "0.0" );
                if (skalierung.getBeschriftungAbstandX() <= 0.1) df = new DecimalFormat( "0.00" );

                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getXAchseTextX(lauf) - (double)Beschriftung_Width/2d );
                int Punkt_y_Text = (int) (skalierung.getPosXAchse() +  Beschriftung_Height) ;

                // X - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_x_Markierung = (int) (skalierung.getXAchseTextX(lauf));

                if (Punkt_x_Markierung>2*skalierung.getRand() && Punkt_x_Markierung<skalierung.getWidth()-2*skalierung.getRand()) {
                    // Text ausgeben
                    g.drawString(Beschriftung , Punkt_x_Text, (int)(1*Punkt_y_Text));

                    // Kleine Markierungen anbringen, wo der Text steht
                    g.drawLine(Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()-2), Punkt_x_Markierung, (int)(1*skalierung.getPosXAchse()+3));
                }
            }
        }

        // Y - Achsen Beschriftung von 0 .. oo
        if(!IsNormaled) {
            for (double lauf=0; lauf <= 3*1.1 ; lauf+=skalierung.getBeschriftungAbstandY()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );
                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
                int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

                // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
            }
        } else {
            for (double lauf=0; lauf <= 3*1.1 ; lauf+=skalierung.getBeschriftungAbstandY()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );
                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getPosYAchse()-skalierung.getWidth()/2+40-4 - Beschriftung_Width);
                int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

                // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(skalierung.getPosYAchse()-skalierung.getWidth()/2+40-3, Punkt_y_Markierung, skalierung.getPosYAchse()-skalierung.getWidth()/2+40+2 ,Punkt_y_Markierung);

            }
        }
        // Y - Achsen Beschriftung von -oo .. 0
        if(!IsNormaled) {
            for (double lauf=0; lauf >= -3*0.95  ; lauf-=skalierung.getBeschriftungAbstandY()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );
                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getPosYAchse()-4 - Beschriftung_Width);
                int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

                // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(skalierung.getPosYAchse()-3, Punkt_y_Markierung, skalierung.getPosYAchse()+2 ,Punkt_y_Markierung);
            }
        } else {
            for (double lauf=0; lauf >= -3*0.95  ; lauf-=skalierung.getBeschriftungAbstandY()) {
                // Formatierung der Achsenbeschriftung
                DecimalFormat df = new DecimalFormat( "0.0" );
                String Beschriftung = df.format(lauf);

                // Platzbedarf fr Schrift ausrechnen
                FontMetrics fm = getFontMetrics(getFont());
                int Beschriftung_Width = fm.stringWidth(Beschriftung);
                int Beschriftung_Height = fm.getHeight();

                // X und Y - Koordinate fr Textausgabe
                int Punkt_x_Text = (int) (skalierung.getPosYAchse()-skalierung.getWidth()/2+40-4 - Beschriftung_Width);
                int Punkt_y_Text = (int) (skalierung.getYAchseTextY(lauf) +  (double)Beschriftung_Height / 2d -2  ) ;

                // Y - Koordinate fr die Markierungen (ohne Textgrenkorrektur)
                int Punkt_y_Markierung = (int) (skalierung.getYAchseTextY(lauf));

                // Text ausgeben
                g.drawString(Beschriftung , Punkt_x_Text, Punkt_y_Text);

                // Kleine Markierungen anbringen, wo der Text steht
                g.drawLine(skalierung.getPosYAchse()-skalierung.getWidth()/2+40-3, Punkt_y_Markierung, skalierung.getPosYAchse()-skalierung.getWidth()/2+40+2 ,Punkt_y_Markierung);
            }
        }
    }


    /**
     * Hintergund wei darstellen
     *
     * @param g 	Graphics Objekt auf die Zeichenflche
     */
    private void PrintBackground (Graphics g) {
        // Referzenz fr 2D Grafik
        Graphics2D g2d = (Graphics2D) g;

        // Rahmen
//		Color Farbe = new Color(0xffdcdc00);
//		g2d.setColor(Farbe);
        g2d.setColor(Color.WHITE);
        g2d.fill(new Rectangle(skalierung.getRand(),skalierung.getRand(),skalierung.getWidth()-skalierung.getRand()*2,skalierung.getHeight()-skalierung.getRand()*2));
    }


    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseListener extends MouseAdapter {

        /**
         * Methode wird aufgerufen, falls Maustaste gedrckt wurde
         *
         * @param evt 	MouseEvent
         */
        public void mousePressed(MouseEvent evt) {
            // Mausposition holen
            int posX = evt.getX();
            // aus Mauskoordinaten Orginalwert errechnen
            skalierung.Rechnen(getWidth(), getHeight());
            double Calculated_X_Value = skalierung.getRealXWert(posX);

            // Tolleranz bercksichtigen
            Klick_Tolleranz = 0.05d ;

            // Gltiger Bereich?
            if (
                Calculated_X_Value >= parameterEingabe1X.getAbgriff() - Klick_Tolleranz
                &&
                Calculated_X_Value <= parameterEingabe1X.getAbgriff() + Klick_Tolleranz
            )
                CanDrag = true;
        }

        /**
         * Methode wird aufgerufen, falls die "Maus" das Koordinatensystem verlassen hat, d.h.
         * im Ausgabemen, darf nun keine Mausposition mehr angezeigt werden.
         *
         * @param evt 	MouseEvent
         */
        public void mouseExited(MouseEvent evt) {
            if (CanDrag) {
                CanDrag = false;
            }
            setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        }

        /**
         * Methode wird aufgerufen, falls die Maustaste losgelassen wurde, d.h.
         * ein Punkt wurde verschoben bzw. aus dem Koordinatensystem gezogen und
         * wird gelscht.
         *
         * @param evt 	MouseEvent
         */
        public void mouseReleased(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag) {
                // ende Verschiebung
                CanDrag = false;
                // derzeige Mausposition
                int posX = evt.getX();
                // Wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);
                parameterEingabe1X.setAbgriff(skalierung.getSkalXWert(Calculated_X_Value));
            }
        }
    }

    //************************************ M o u s e L i s t e n e r ******************************************
    class MyMouseMotionListener extends MouseMotionAdapter {
        /**
         * Methode wird aufgerufen, falls Maus bewegt wurde.
         * Maus Cursor wird angepasst
         *
         * @param evt 	MouseEvent
         */
        public void mouseMoved(MouseEvent evt) {
            if (CanDrag != true) {
                // derzeige Mausposition
                int posX = evt.getX();
                // Wert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);

                Klick_Tolleranz = 0.05d ;

                // Mauszeiger ber Abgrifflinie -> Zeiger ndern
                if (
                    Calculated_X_Value >= parameterEingabe1X.getAbgriff() - Klick_Tolleranz
                    &&
                    Calculated_X_Value <= parameterEingabe1X.getAbgriff() + Klick_Tolleranz
                ) {
                    setCursor(new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR  ));
                } else {	// Wenn nicht, normalen Cursor einstellen
                    setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR   ));
                }
            }
        }

        /**
         * Methode wird aufgerufen, falls Maus mit gedrckter Maustaste bewegt wurde.
         *
         * @param evt 	MouseEvent
         */
        public void mouseDragged(MouseEvent evt) {
            // nur wenn verschoben wird
            if (CanDrag == true) {
                // Mausposition holen
                int posX = evt.getX();
                // Orginalwert zurckrechnen
                skalierung.Rechnen(getWidth(), getHeight());
                double Calculated_X_Value = skalierung.getRealXWert(posX);
                parameterEingabe1X.setAbgriff(skalierung.getSkalXWert(Calculated_X_Value));
            }
        }
    }
}
